/* vi: set sw=4 ts=4: */
/*
 * Licensed under the GPL v2 or later, see the file LICENSE in this tarball.
 */
#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/wait.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <netinet/in.h>
#include <stdio.h>
#include <sys/socket.h>
#include <unistd.h>
#include <getopt.h>
#include <time.h>

#include "dhcpd.h"
#include "leases.h"
#include "libbb_udhcp.h"
#include "csid/csid_vlan.h"
#include "csid/csid_dhcp.h"
#include "csman.h"
#if defined(_AMIT_ENABLE_AIRLIVE_IPCAM)
#include "common.h"
#endif


#define REMAINING 0
#define ABSOLUTE 1


#ifndef IN_BUSYBOX
static void ATTRIBUTE_NORETURN show_usage(void)
{
	printf(
"Usage: dumpleases -f <file> -[r|a]\n\n"
"  -f, --file=FILENAME             Leases file to load\n"
"  -r, --remaining                 Interepret lease times as time remaing\n"
"  -a, --absolute                  Interepret lease times as expire time\n");
	exit(0);
}
#else
#define show_usage bb_show_usage
#endif


#ifdef IN_BUSYBOX
int dumpleases_main(int argc, char *argv[])
#else
int main(int argc, char *argv[])
#endif
{
	FILE *fp;
	int i, c, mode = REMAINING;
	long expires;
    long temp_expires;//horowitz added for initial csid
	const char *file = LEASES_FILE;
	struct dhcpOfferedAddr lease;
    struct dhcpOfferedAddr temp_lease;//horowitz added for initial csid
	struct in_addr addr;
	
	int csman_fd;
	unsigned int lease_cnt=0;
	unsigned char mac[256];

#if defined(_HANDLE_UPDATE_DHCPD_LIST_BY_CHECKING_ARP_TABLE)
	FILE *fp_arptable;
	char LineBuffer[1024], *LinePTR, *TokenPTR;
	int ReadLineNo = 0, TokenNo = 0, ExistFlag = 0, cur_dhcpclient_cnt = 0;
	char ARP_IPAddr[20], ARP_MAC[20], ARP_MAC_TMP[20], ARP_IFName[20];
	struct dhcpOfferedAddr lease_arp;
#endif

	static const struct option options[] = {
		{"absolute", 0, 0, 'a'},
		{"remaining", 0, 0, 'r'},
		{"file", 1, 0, 'f'},
		{0, 0, 0, 0}
	};

	while (1) {
		int option_index = 0;
		c = getopt_long(argc, argv, "arf:", options, &option_index);
		if (c == -1) break;

		switch (c) {
		case 'a': mode = ABSOLUTE; break;
		case 'r': mode = REMAINING; break;
		case 'f':
			file = optarg;
			break;
		default:
			show_usage();
		}
	}

	fp = xfopen(file, "r");

	csman_fd=open_csman(NULL, 0);
	if( csman_fd <0 ){
		perror("could not open Csman");
		return 0;	
	}



//***  initial(clear) all csid to zero  ***
        int j,k,count=0;
              for(j=0;j<= 255;j++){
                  for(k=0;k<6;k++){
                      temp_lease.chaddr[k]=0;
                  }
                  temp_lease.yiaddr=0;
                  strcpy(temp_lease.hostname,"");
                  temp_expires=0;
                  write_csman(csman_fd, (unsigned int)(CSID_S_DHCP_SVR_CLILIST_MAC+j), (const void *)temp_lease.chaddr, 6, CSM_W_RMID);
                  write_csman(csman_fd, (unsigned int)(CSID_S_DHCP_SVR_CLILIST_IP+j), (const void *)&temp_lease.yiaddr, 4, CSM_W_RMID);
                  write_csman(csman_fd, (unsigned int)(CSID_S_DHCP_SVR_CLILIST_HOSTNAME+j), (const void *)temp_lease.hostname, strlen(temp_lease.hostname)+1, CSM_W_RMID);
                  write_csman(csman_fd, (unsigned int)(CSID_S_DHCP_SVR_CLILIST_LEASETIME+j), (const void *)&temp_expires, sizeof(temp_expires), CSM_W_RMID);
                  #ifdef _HANDLE_DHCPD_LIST_ARP_DETECT_CLIENT
                    	write_csman(csman_fd, (CSID_S_DHCP_SVR_CLILIST_WLAN+j), (const void *)temp_lease.chaddr, 6, CSM_W_RMID);
                  #endif
                  count++;
                }
              printf("initial loops = %d \n",count);
//***  end initial/clear, horowitz added  ***

	printf("Mac Address       IP-Address      HOSTNAME    Expires %s\n", mode == REMAINING ? "in" : "at");  
	/*     "00:00:00:00:00:00 255.255.255.255 Wed Jun 30 21:49:08 1993" */
	lease_cnt=0;
	while (fread(&lease, sizeof(lease), 1, fp)) {
		for (i = 0; i < 6; i++) {
			printf("%02x", lease.chaddr[i]);
			if (i != 5) printf(":");
		}
		/* add by ahon */
		sprintf(mac,"%02x:%02x:%02x:%02x:%02x:%02x",lease.chaddr[0],lease.chaddr[1],lease.chaddr[2],lease.chaddr[3],lease.chaddr[4],lease.chaddr[5]);
		/* end */

		addr.s_addr = lease.yiaddr;
		printf(" %-15s", inet_ntoa(addr));
		printf(" %s", lease.hostname);		
		expires = ntohl(lease.expires);
		printf(" ");
		
		if( lease.hostname == NULL ) continue;
		
		if (mode == REMAINING) {
			if (!expires){ 
				printf("expired\n");
			 } 
      	else if(strcmp(mac,"00:00:00:00:00:00")) { // compare if mac is not zero, write to csid for UI show leases				
			write_csman(csman_fd, (unsigned int)(CSID_S_DHCP_SVR_CLILIST_MAC+lease_cnt), (const void *)lease.chaddr, 6, 0);
			write_csman(csman_fd, (unsigned int)(CSID_S_DHCP_SVR_CLILIST_IP+lease_cnt), (const void *)&addr, 4, 0);
			write_csman(csman_fd, (unsigned int)(CSID_S_DHCP_SVR_CLILIST_HOSTNAME+lease_cnt), (const void *)lease.hostname, strlen(lease.hostname)+1, 0);
#if defined(_AMIT_ENABLE_AIRLIVE_IPCAM)
			int ipcamIndex ;
			unsigned char airliveHostname[32];
			char ipcam_ip[64] = "";
			char list_ip[64] = "";
			struct in_addr ipcam_addr;

			bzero(list_ip, sizeof(list_ip));
			bzero(ipcam_ip, sizeof(ipcam_ip));
			bzero(airliveHostname, sizeof(airliveHostname));

			for(ipcamIndex = 0; ipcamIndex < AIRLIVE_CAM_NUMBER; ipcamIndex++)					
			{

				read_csman(csman_fd,CSID_C_IPCAM_AIRLIVE_IP + ipcamIndex, &ipcam_addr, sizeof(struct in_addr), CSM_R_ZERO);
        		strcpy(ipcam_ip, inet_ntoa(ipcam_addr));
        		strcpy(list_ip, inet_ntoa(addr));

				if(strcmp(ipcam_ip, list_ip) == 0)
				{

					write_csman(csman_fd,CSID_C_IPCAM_AIRLIVE_HOSTNAME+ipcamIndex, (const void *)lease.hostname, strlen(lease.hostname)+1,0);
					write_csman(csman_fd, CSID_C_IPCAM_AIRLIVE_MAC + ipcamIndex, (const void *)lease.chaddr, 6, 0);

					break;
				   }
				}
#endif
				write_csman(csman_fd, (unsigned int)(CSID_S_DHCP_SVR_CLILIST_LEASETIME+lease_cnt), (const void *)&expires, sizeof(expires), 0);				 

				lease_cnt++;

				if (expires > 60*60*24) {
					printf("%ld days, ", expires / (60*60*24));
					expires %= 60*60*24;
				}
				if (expires > 60*60) {
					printf("%ld hours, ", expires / (60*60));
					expires %= 60*60;
				}
				if (expires > 60) {
					printf("%ld minutes, ", expires / 60);
					expires %= 60;
				}
				printf("%ld seconds\n", expires);
			}printf("\n");//for print each lease normally, when reget lease. see udhcpd-action lease.
		} else printf("%s", ctime(&expires));
	}

/* Support updating LAN Client List by checking ARP (For LAN Client with static IP) */
#if defined(_HANDLE_UPDATE_DHCPD_LIST_BY_CHECKING_ARP_TABLE)
/* The format of file "/proc/net/arp" 
  IP address       HW type     Flags       HW address            Mask     Device
  192.168.0.100    0x1         0x2         00:01:0a:10:0f:17     *        br0
*/
	cur_dhcpclient_cnt = lease_cnt;
	strcpy(lease_arp.hostname,"");
	expires = 0xFFFFFFFF; /* Set 0xFFFFFFFF */
	fp_arptable = fopen("/proc/net/arp", "r");
	if (fp_arptable != NULL)
	{
		printf("\n");
		printf("ARP Table For Static IP:\n");
		printf("Mac Address       IP-Address\n");
		memset(LineBuffer, 0, sizeof(LineBuffer));
		ReadLineNo = 0;
		LinePTR = fgets(LineBuffer, sizeof(LineBuffer), fp_arptable);
		//printf("ARP Table Line [%d]: %s\n", ReadLineNo + 1, LinePTR);
		while ( LinePTR != NULL )
		{
			/* Ignore first line */
			if (ReadLineNo > 0)
			{
				memset(ARP_IPAddr, 0, sizeof(ARP_IPAddr));
				memset(ARP_MAC, 0, sizeof(ARP_MAC));
				memset(ARP_MAC_TMP, 0, sizeof(ARP_MAC_TMP));
				memset(ARP_IFName, 0, sizeof(ARP_IFName));

				TokenPTR = strtok(LineBuffer, " \t\n");
				TokenNo = 0;
				while (TokenPTR != NULL)
				{
					if (TokenNo > 5) {break;}
					switch (TokenNo)
					{
						case 0:
							strncpy(ARP_IPAddr, TokenPTR, sizeof(ARP_IPAddr));
							//printf("ARP Table Line [%d] [IP]: %s\n", ReadLineNo + 1, ARP_IPAddr);
							break;
						case 3:
							strncpy(ARP_MAC, TokenPTR, sizeof(ARP_MAC));
							//printf("ARP Table Line [%d] [MAC]: %s\n", ReadLineNo + 1, ARP_MAC);
							break;
						case 5:
							strncpy(ARP_IFName, TokenPTR, sizeof(ARP_IFName));
							//printf("ARP Table Line [%d] [IF]: %s\n", ReadLineNo + 1, ARP_IFName);
							break;
						default:
							break;
					}
					TokenPTR = strtok(NULL, " \t\n");
					TokenNo++;
				}
				/* Check if the interface name is LAN (br0, br1, ..., etc) */
				if (strncmp(ARP_IFName, "br", strlen("br")) == 0)
				{
					if ( (strlen(ARP_IPAddr) > 0 && strcmp(ARP_IPAddr, "0.0.0.0") != 0) &&
					     (strlen(ARP_MAC) > 0 && strcmp(ARP_MAC, "00:00:00:00:00:00") != 0) )
					{
						for(i=0 ; i< 6; i++)
						{
							lease_arp.chaddr[i] = 0;
						}
						strncpy(ARP_MAC_TMP, ARP_MAC, sizeof(ARP_MAC_TMP));
						TokenPTR = strtok(ARP_MAC_TMP, ":- ");
						TokenNo = 0;
						while (TokenPTR != NULL)
						{
							if (TokenNo >= 0 && TokenNo <= 5)	{lease_arp.chaddr[TokenNo] = strtoul(TokenPTR, NULL, 16);}
							else {break;}
							TokenPTR = strtok(NULL, ":- ");
							TokenNo++;
						}
						lease_arp.yiaddr = inet_addr(ARP_IPAddr);

						ExistFlag = 0;
						for (i=0; i < cur_dhcpclient_cnt; i++)
						{
							read_csman(csman_fd, (unsigned int)(CSID_S_DHCP_SVR_CLILIST_MAC+i), (const void *)lease.chaddr, 6, CSM_R_ZERO);
							read_csman(csman_fd, (unsigned int)(CSID_S_DHCP_SVR_CLILIST_IP+i), (const void *)&lease.yiaddr, 4, CSM_R_ZERO);
							if (lease_arp.chaddr[0] == lease.chaddr[0] && lease_arp.chaddr[1] == lease.chaddr[1] && 
							    lease_arp.chaddr[2] == lease.chaddr[2] && lease_arp.chaddr[3] == lease.chaddr[3] && 
							    lease_arp.chaddr[4] == lease.chaddr[4] && lease_arp.chaddr[5] == lease.chaddr[5] && 
							    lease_arp.yiaddr == lease.yiaddr)
							{
								ExistFlag = 1;
								break;
							}
						}
						if (ExistFlag == 0)
						{
							printf("%s %s\n", ARP_MAC, ARP_IPAddr);
							for (i=0; i < 4095; i++) {
  								read_csman(csman_fd, (unsigned int)(CSID_S_VLAN_SVR_CLILIST_MAC+i), (const void *)lease.chaddr, 6, CSM_R_ZERO);
  								read_csman(csman_fd, (unsigned int)(CSID_S_VLAN_SVR_CLILIST_IP+i), (const void *)&lease.yiaddr, 4, CSM_R_ZERO);
  								if (!lease.yiaddr) break;
                				if (lease_arp.chaddr[0] == lease.chaddr[0] && lease_arp.chaddr[1] == lease.chaddr[1] && 
  							    lease_arp.chaddr[2] == lease.chaddr[2] && lease_arp.chaddr[3] == lease.chaddr[3] && 
  							    lease_arp.chaddr[4] == lease.chaddr[4] && lease_arp.chaddr[5] == lease.chaddr[5] && 
  							    lease_arp.yiaddr == lease.yiaddr)
  								{
  							   		printf("VLAN Server:%s with dynamic IP:%s\n", ARP_IFName, ARP_IPAddr);
  							   		ExistFlag = 1;
                   					break;
  								}
							}
							if (ExistFlag == 0) {
								write_csman(csman_fd, (unsigned int)(CSID_S_DHCP_SVR_CLILIST_MAC+lease_cnt), (const void *)lease_arp.chaddr, 6, 0);
								write_csman(csman_fd, (unsigned int)(CSID_S_DHCP_SVR_CLILIST_IP+lease_cnt), (const void *)&lease_arp.yiaddr, 4, 0);
								write_csman(csman_fd, (unsigned int)(CSID_S_DHCP_SVR_CLILIST_HOSTNAME+lease_cnt), (const void *)lease_arp.hostname, strlen(lease_arp.hostname)+1, 0);
								write_csman(csman_fd, (unsigned int)(CSID_S_DHCP_SVR_CLILIST_LEASETIME+lease_cnt), (const void *)&expires, sizeof(expires), 0);
								lease_cnt++;
							}
						}
					}
				}
			}
			memset(LineBuffer, 0, sizeof(LineBuffer));
			ReadLineNo++;
			LinePTR = fgets(LineBuffer, sizeof(LineBuffer), fp_arptable);
			//printf("ARP Table Line [%d]: %s\n", ReadLineNo + 1, LinePTR);
		}

		fclose(fp_arptable);
	}
#endif

//***  write zero lease for record endless lease in CSID, thus UI can see where to end print leases  ***
	for(i=0;i<6;i++){
		lease.chaddr[i]=0;
	}
	addr.s_addr=0;
	strcpy(lease.hostname,"");
	expires=0;	
	
	write_csman(csman_fd, CSID_S_DHCP_SVR_CLILIST_MAC+lease_cnt, (const void *)lease.chaddr, 6, 0);
	write_csman(csman_fd, CSID_S_DHCP_SVR_CLILIST_IP+lease_cnt, (const void *)&addr, 4, 0);
	write_csman(csman_fd, CSID_S_DHCP_SVR_CLILIST_HOSTNAME+lease_cnt, (const void *)lease.hostname, strlen(lease.hostname)+1, 0);
	write_csman(csman_fd, CSID_S_DHCP_SVR_CLILIST_LEASETIME+lease_cnt, (const void *)&expires, sizeof(expires), 0);
	
    printf("lease count is %d now \n",lease_cnt);
//***  end write zero lease  ***
	
	close_csman(csman_fd);			
	
	fclose(fp);	
	return 0;
}
