/* $Id: minissdp.c,v 1.36 2012/05/27 22:16:26 nanard Exp $ */
/* MiniUPnP project
 * http://miniupnp.free.fr/ or http://miniupnp.tuxfamily.org/
 * (c) 2006-2012 Thomas Bernard
 * This software is subject to the conditions detailed
 * in the LICENCE file provided within the distribution */

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <errno.h>
#include <syslog.h>
#include "config.h"
#include "upnpdescstrings.h"
#include "miniupnpdpath.h"
#include "upnphttp.h"
#include "upnpglobalvars.h"
#include "minissdp.h"
#include "upnputils.h"
#include "codelength.h"

#include <csman.h>
#include "csid/csid_sys.h"
#include "csid/csid_local.h"
#include "csid/csid_fix.h"
#include "csid/csid_misc.h"
#include "csid/csid_snmp.h"
#include "csid/csid_dhcp.h"
#include "csid/csid_remote.h"
#include "csid/csid_wlanap.h"
#include "csid/csid_vlan.h"

/* SSDP ip/port */
#define SSDP_PORT (1900)
#define SSDP_MCAST_ADDR ("239.255.255.250")
#define LL_SSDP_MCAST_ADDR "FF02::C"
#define SL_SSDP_MCAST_ADDR "FF05::C"

int sta_not_ready=0;
/* structure containing variables used during "main loop"
 * that are filled during the init */
struct runtime_vars {
	/* LAN IP addresses for SSDP traffic and HTTP */
	/* moved to global vars */
	int port;	/* HTTP Port */
	int notify_interval;	/* seconds between SSDP announces */
	/* unused rules cleaning related variables : */
	int clean_ruleset_threshold;	/* threshold for removing unused rules */
	int clean_ruleset_interval;		/* (minimum) interval between checks */
};
/* OpenAndConfHTTPSocket() :
 * setup the socket used to handle incoming HTTP connections. */
/*
static int
OpenAndConfHTTPSocket(unsigned short port)
{
	int s;
	int i = 1;
#ifdef ENABLE_IPV6
	struct sockaddr_in6 listenname;
#else
	struct sockaddr_in listenname;
#endif
	socklen_t listenname_len;

	if( (s = socket(
#ifdef ENABLE_IPV6
	                PF_INET6,
#else
	                PF_INET,
#endif
	                SOCK_STREAM, 0)) < 0)
	{
		syslog(LOG_ERR, "socket(http): %m");
		return -1;
	}

	if(setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i)) < 0)
	{
		syslog(LOG_WARNING, "setsockopt(http, SO_REUSEADDR): %m");
	}
#if 0
	if(setsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY, &i, sizeof(i)) < 0)
	{
		syslog(LOG_WARNING, "setsockopt(http, IPV6_V6ONLY): %m");
	}
#endif

	if(!set_non_blocking(s))
	{
		syslog(LOG_WARNING, "set_non_blocking(http): %m");
	}

#ifdef ENABLE_IPV6
	memset(&listenname, 0, sizeof(struct sockaddr_in6));
	listenname.sin6_family = AF_INET6;
	listenname.sin6_port = htons(port);
	listenname.sin6_addr = in6addr_any;
	listenname_len =  sizeof(struct sockaddr_in6);
#else
	listenname.sin_family = AF_INET;
	listenname.sin_port = htons(port);
	listenname.sin_addr.s_addr = htonl(INADDR_ANY);
	listenname_len =  sizeof(struct sockaddr_in);
#endif

	if(bind(s, (struct sockaddr *)&listenname, listenname_len) < 0)
	{
		syslog(LOG_ERR, "bind(http): %m");
		close(s);
		return -1;
	}

	if(listen(s, 5) < 0)
	{
		syslog(LOG_ERR, "listen(http): %m");
		close(s);
		return -1;
	}

	return s;
}
*/
/* AddMulticastMembership()
 * param s		socket
 * param ifaddr	ip v4 address
 */
static int
AddMulticastMembership(int s, in_addr_t ifaddr)
{
	struct ip_mreq imr;	/* Ip multicast membership */

    /* setting up imr structure */
    imr.imr_multiaddr.s_addr = inet_addr(SSDP_MCAST_ADDR);
    /*imr.imr_interface.s_addr = htonl(INADDR_ANY);*/
    imr.imr_interface.s_addr = ifaddr;	/*inet_addr(ifaddr);*/

	if (setsockopt(s, IPPROTO_IP, IP_ADD_MEMBERSHIP, (void *)&imr, sizeof(struct ip_mreq)) < 0)
	{
        syslog(LOG_ERR, "setsockopt(udp, IP_ADD_MEMBERSHIP): %m");
		return -1;
    }

	return 0;
}

/* AddMulticastMembershipIPv6()
 * param s	socket (IPv6)
 * To be improved to target specific network interfaces */
#ifdef ENABLE_IPV6
static int
AddMulticastMembershipIPv6(int s)
{
	struct ipv6_mreq mr;
	/*unsigned int ifindex;*/

	memset(&mr, 0, sizeof(mr));
	inet_pton(AF_INET6, LL_SSDP_MCAST_ADDR, &mr.ipv6mr_multiaddr);
	/*mr.ipv6mr_interface = ifindex;*/
	mr.ipv6mr_interface = 0; /* 0 : all interfaces */
#ifndef IPV6_ADD_MEMBERSHIP
#define IPV6_ADD_MEMBERSHIP IPV6_JOIN_GROUP
#endif
	if(setsockopt(s, IPPROTO_IPV6, IPV6_ADD_MEMBERSHIP, &mr, sizeof(struct ipv6_mreq)) < 0)
	{
		syslog(LOG_ERR, "setsockopt(udp, IPV6_ADD_MEMBERSHIP): %m");
		return -1;
	}
	inet_pton(AF_INET6, SL_SSDP_MCAST_ADDR, &mr.ipv6mr_multiaddr);
	if(setsockopt(s, IPPROTO_IPV6, IPV6_ADD_MEMBERSHIP, &mr, sizeof(struct ipv6_mreq)) < 0)
	{
		syslog(LOG_ERR, "setsockopt(udp, IPV6_ADD_MEMBERSHIP): %m");
		return -1;
	}
	return 0;
}
#endif

/* Open and configure the socket listening for
 * SSDP udp packets sent on 239.255.255.250 port 1900
 * SSDP v6 udp packets sent on FF02::C, or FF05::C, port 1900 */
int
OpenAndConfSSDPReceiveSocket(int ipv6)
{
	int s;
	struct sockaddr_storage sockname;
	socklen_t sockname_len;
	struct lan_addr_s * lan_addr;
	int j = 1;

	if( (s = socket(ipv6 ? PF_INET6 : PF_INET, SOCK_DGRAM, 0)) < 0)
	{
		syslog(LOG_ERR, "%s: socket(udp): %m",
		       "OpenAndConfSSDPReceiveSocket");
		return -1;
	}

	memset(&sockname, 0, sizeof(struct sockaddr_storage));
	if(ipv6) {
		struct sockaddr_in6 * saddr = (struct sockaddr_in6 *)&sockname;
		saddr->sin6_family = AF_INET6;
		saddr->sin6_port = htons(SSDP_PORT);
		saddr->sin6_addr = in6addr_any;
		sockname_len = sizeof(struct sockaddr_in6);
	} else {
		struct sockaddr_in * saddr = (struct sockaddr_in *)&sockname;
		saddr->sin_family = AF_INET;
		saddr->sin_port = htons(SSDP_PORT);
		/* NOTE : it seems it doesnt work when binding on the specific address */
		/*saddr->sin_addr.s_addr = inet_addr(UPNP_MCAST_ADDR);*/
		saddr->sin_addr.s_addr = htonl(INADDR_ANY);
		/*saddr->sin_addr.s_addr = inet_addr(ifaddr);*/
		sockname_len = sizeof(struct sockaddr_in);
	}

	if(setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &j, sizeof(j)) < 0)
	{
		syslog(LOG_WARNING, "setsockopt(udp, SO_REUSEADDR): %m");
	}

	if(!set_non_blocking(s))
	{
		syslog(LOG_WARNING, "%s: set_non_blocking(): %m",
		       "OpenAndConfSSDPReceiveSocket");
	}

	if(bind(s, (struct sockaddr *)&sockname, sockname_len) < 0)
	{
		syslog(LOG_ERR, "%s: bind(udp%s): %m",
		       "OpenAndConfSSDPReceiveSocket", ipv6 ? "6" : "");
		close(s);
		return -1;
	}

#ifdef ENABLE_IPV6
	if(ipv6)
	{
		AddMulticastMembershipIPv6(s);
	}
	else
#endif
	{
		for(lan_addr = lan_addrs.lh_first; lan_addr != NULL; lan_addr = lan_addr->list.le_next)
		{
			if(AddMulticastMembership(s, lan_addr->addr.s_addr) < 0)
			{
				syslog(LOG_WARNING,
				       "Failed to add multicast membership for interface %s",
				       lan_addr->str ? lan_addr->str : "NULL");
			}
		}
	}

	return s;
}

/* open the UDP socket used to send SSDP notifications to
 * the multicast group reserved for them */
static int
OpenAndConfSSDPNotifySocket(in_addr_t addr)
{
	int s;
	unsigned char loopchar = 0;
	int bcast = 1;
	unsigned char ttl = 2; /* UDA v1.1 says :
		The TTL for the IP packet SHOULD default to 2 and
		SHOULD be configurable. */
	/* TODO: Make TTL be configurable */
	struct in_addr mc_if;
	struct sockaddr_in sockname;

	if( (s = socket(PF_INET, SOCK_DGRAM, 0)) < 0)
	{
		syslog(LOG_ERR, "socket(udp_notify): %m");
		return -1;
	}

	mc_if.s_addr = addr;	/*inet_addr(addr);*/

	if(setsockopt(s, IPPROTO_IP, IP_MULTICAST_LOOP, (char *)&loopchar, sizeof(loopchar)) < 0)
	{
		syslog(LOG_ERR, "setsockopt(udp_notify, IP_MULTICAST_LOOP): %m");
		close(s);
		return -1;
	}

	if(setsockopt(s, IPPROTO_IP, IP_MULTICAST_IF, (char *)&mc_if, sizeof(mc_if)) < 0)
	{
		syslog(LOG_ERR, "setsockopt(udp_notify, IP_MULTICAST_IF): %m");
		close(s);
		return -1;
	}

	if(setsockopt(s, IPPROTO_IP, IP_MULTICAST_TTL, &ttl, sizeof(ttl)) < 0)
	{
		syslog(LOG_WARNING, "setsockopt(udp_notify, IP_MULTICAST_TTL,): %m");
	}

	if(setsockopt(s, SOL_SOCKET, SO_BROADCAST, &bcast, sizeof(bcast)) < 0)
	{
		syslog(LOG_ERR, "setsockopt(udp_notify, SO_BROADCAST): %m");
		close(s);
		return -1;
	}

	memset(&sockname, 0, sizeof(struct sockaddr_in));
    sockname.sin_family = AF_INET;
    sockname.sin_addr.s_addr = addr;	/*inet_addr(addr);*/

    if (bind(s, (struct sockaddr *)&sockname, sizeof(struct sockaddr_in)) < 0)
	{
		syslog(LOG_ERR, "bind(udp_notify): %m");
		close(s);
		return -1;
    }

	return s;
}

#ifdef ENABLE_IPV6
/* open the UDP socket used to send SSDP notifications to
 * the multicast group reserved for them. IPv6 */
static int
OpenAndConfSSDPNotifySocketIPv6(unsigned int if_index)
{
	int s;
	unsigned int loop = 0;

	s = socket(PF_INET6, SOCK_DGRAM, 0);
	if(s < 0)
	{
		syslog(LOG_ERR, "socket(udp_notify IPv6): %m");
		return -1;
	}
	if(setsockopt(s, IPPROTO_IPV6, IPV6_MULTICAST_IF, &if_index, sizeof(if_index)) < 0)
	{
		syslog(LOG_ERR, "setsockopt(udp_notify IPv6, IPV6_MULTICAST_IF, %u): %m", if_index);
		close(s);
		return -1;
	}
	if(setsockopt(s, IPPROTO_IPV6, IPV6_MULTICAST_LOOP, &loop, sizeof(loop)) < 0)
	{
		syslog(LOG_ERR, "setsockopt(udp_notify, IPV6_MULTICAST_LOOP): %m");
		close(s);
		return -1;
	}
	return s;
}
#endif

int
OpenAndConfSSDPNotifySockets(int * sockets)
/*OpenAndConfSSDPNotifySockets(int * sockets,
                             struct lan_addr_s * lan_addr, int n_lan_addr)*/
{
	int i;
	struct lan_addr_s * lan_addr;

	for(i=0, lan_addr = lan_addrs.lh_first;
	    lan_addr != NULL;
	    lan_addr = lan_addr->list.le_next)
	{
		sockets[i] = OpenAndConfSSDPNotifySocket(lan_addr->addr.s_addr);
		if(sockets[i] < 0)
			goto error;
		i++;
#ifdef ENABLE_IPV6
		sockets[i] = OpenAndConfSSDPNotifySocketIPv6(lan_addr->index);
		if(sockets[i] < 0)
			goto error;
		i++;
#endif
	}
	return 0;
error:
	while(--i >= 0)
	{
		close(sockets[i]);
		sockets[i] = -1;
	}
	return -1;
}

char *str_replace (char *source, char *find,  char *rep)
{
	int find_L=strlen(find);   
	int rep_L=strlen(rep);  
	int length=strlen(source)+1;
	int gap=0;
	char *result = (char*)malloc(sizeof(char) * length);  
	strcpy(result, source);	
	char *former=source;
	char *location= strstr(former, find);

	while(location!=NULL){
		gap+=(location - former);
		result[gap]='\0';
		length+=(rep_L-find_L);
		result = (char*)realloc(result, length * sizeof(char));
		strcat(result, rep);
		gap+=rep_L;
		former=location+find_L;
		strcat(result, former);
		location= strstr(former, find);
	}
	return result;  
}

/*
 * response from a LiveBox (Wanadoo)
HTTP/1.1 200 OK
CACHE-CONTROL: max-age=1800
DATE: Thu, 01 Jan 1970 04:03:23 GMT
EXT:
LOCATION: http://192.168.0.1:49152/gatedesc.xml
SERVER: Linux/2.4.17, UPnP/1.0, Intel SDK for UPnP devices /1.2
ST: upnp:rootdevice
USN: uuid:75802409-bccb-40e7-8e6c-fa095ecce13e::upnp:rootdevice

 * response from a Linksys 802.11b :
HTTP/1.1 200 OK
Cache-Control:max-age=120
Location:http://192.168.5.1:5678/rootDesc.xml
Server:NT/5.0 UPnP/1.0
ST:upnp:rootdevice
USN:uuid:upnp-InternetGatewayDevice-1_0-0090a2777777::upnp:rootdevice
EXT:
 */

/* not really an SSDP "announce" as it is the response
 * to a SSDP "M-SEARCH" */
static void
SendSSDPAnnounce2(int s, const struct sockaddr * addr,
                  const char * st, int st_len, const char * suffix,
                  const char * host, unsigned short port)
{
	int l, n;
	char buf[1024];
	char addr_str[64];
	socklen_t addrlen;
	static char romid[30]="";
	char ext[64];
	int csmfd, ap_enable=0, ac_enable=0;
    	uint8_t beid[6];
    	memset(beid, 0, sizeof(beid));
	char ap_mac[32];
    	memset(ap_mac, '\0', sizeof(ap_mac));
	csmfd = open_csman(NULL, 0);
	/*
	 * follow guideline from document "UPnP Device Architecture 1.0"
	 * uppercase is recommended.
	 * DATE: is recommended
	 * SERVER: OS/ver UPnP/1.0 miniupnpd/1.0
	 * - check what to put in the 'Cache-Control' header
	 *
	 * have a look at the document "UPnP Device Architecture v1.1 */
	if(strlen(romid)<2){
         read_csman(csmfd, CSID_S_SYS_INFO_ROM_ID, romid, 30, 0);
	}
	if(strstr(st,"rootdevice")){
			sprintf(ext,"Amit-R:%s-P:80",romid);
	}else{
			memset(ext,'\0',sizeof(ext));
	}

	read_csman(csmfd, CSID_C_MISC_AP_CONTROLLER_AP, &ap_enable, sizeof(ap_enable) ,CSM_R_ZERO);
	read_csman(csmfd, CSID_C_MISC_AP_CONTROLLER_AC, &ac_enable, sizeof(ac_enable) ,CSM_R_ZERO);

        read_csman(csmfd, CSID_S_SYS_IF_LAN0_MAC, beid, 6, CSM_R_ZSTR);
        sprintf(ap_mac,"%02X:%02X:%02X:%02X:%02X:%02X",beid[0],beid[1],beid[2],beid[3],beid[4],beid[5]);

	if (ap_enable && strstr(st,"rootdevice") && strcmp(ap_mac, "00:00:00:00:00:00") != 0)
	{
		int snmp_local_enable=0, snmp_remote_enable=0, snmp_sta=1, deny_all=0, protocol_ap=0;
		char snmp_get_community[32];
		memset(snmp_get_community,'\0',sizeof(snmp_get_community));
		char device_name[256], device_name_tmp[256], read_buf[256], model[32], ac_mac[32], rom_id[16];
    		memset(device_name, '\0', sizeof(device_name));
    		memset(device_name_tmp, '\0', sizeof(device_name_tmp));
    		memset(read_buf, '\0', sizeof(read_buf));
    		memset(model, '\0', sizeof(model));
    		memset(ac_mac, '\0', sizeof(ac_mac));
    		memset(rom_id, '\0', sizeof(rom_id));
		FILE *f;

		read_csman(csmfd, CSID_C_SNMP_LOCAL_ENABLE, &snmp_local_enable, sizeof(snmp_local_enable) ,CSM_R_ZERO);
		read_csman(csmfd, CSID_C_SNMP_REMOTE_ENABLE, &snmp_remote_enable, sizeof(snmp_remote_enable) ,CSM_R_ZERO);
		if (snmp_local_enable || snmp_remote_enable)
		{
			int snmp_v1=0, snmp_v2=0, snmp_v3=0;
			read_csman(csmfd, CSID_C_SNMP_VERSION_V1, &snmp_v1, sizeof(snmp_v1) ,CSM_R_ZERO);
			read_csman(csmfd, CSID_C_SNMP_VERSION_V2C, &snmp_v2, sizeof(snmp_v2) ,CSM_R_ZERO);
			read_csman(csmfd, CSID_C_SNMP_VERSION_V3, &snmp_v3, sizeof(snmp_v3) ,CSM_R_ZERO);
			snmp_sta = snmp_v1 + snmp_v2*2 + snmp_v3*4;

			read_csman(csmfd, CSID_C_SNMP_GET_COMMUNITY, snmp_get_community, sizeof(snmp_get_community) ,CSM_R_ZSTR);
		}
		else
			snmp_sta = 0;

	
		if ((f = fopen("/tmp/ssdp_ap_customname", "r")) != NULL){
			fgets(read_buf,sizeof(read_buf),f);
			sscanf(read_buf,"%[^\n]",device_name_tmp);
			strcpy(device_name,str_replace(device_name_tmp,"#qutation#","\""));
			fclose(f);
		}
		else{	
			read_csman(csmfd,CSID_C_REMOTE_AP_TRUSTAP_CUSTOM_NAME,device_name,sizeof(device_name),CSM_R_ZSTR);
			if(!strcmp(device_name,"\0")){
    				memset(device_name, '\0', sizeof(device_name));
				read_csman(csmfd,CSID_S_SYS_INFO_MODEL_NAME ,device_name,sizeof(device_name),CSM_R_ZSTR);
			}
		}

		read_csman(csmfd, CSID_S_SYS_INFO_ROM_ID, rom_id, sizeof(rom_id), CSM_R_ZSTR);
		read_csman(csmfd, CSID_S_SYS_INFO_BOOT_PNAME, model, sizeof(model) ,CSM_R_ZSTR);
		if (strncmp(rom_id, "DDC", 3) == 0)
		{
			if (strcmp(model, "APW771AM_P01") == 0)
			{
    				memset(model, '\0', sizeof(model));
				strcpy(model, "WAP-6150");
			}
			else if (strcmp(rom_id, "DDC6E3706001") == 0)
			{
    				memset(model, '\0', sizeof(model));
				strcpy(model, "WAP-8101");
			}
			else if (strcmp(model, "APC772AM_P01") == 0)
			{
    				memset(model, '\0', sizeof(model));
				strcpy(model, "WAP-6101");
			}
			else if (strcmp(model, "APC761AM_P03") == 0)
			{
    				memset(model, '\0', sizeof(model));
				strcpy(model, "WAP-6103");
			}
		}
		else if (strcmp(rom_id, "SML6E3711001") == 0 && strcmp(model, "APC772AM_P01") == 0)
		{
    			memset(model, '\0', sizeof(model));
			strcpy(model, "DG-WM2005SI");
		}
		else if (strcmp(rom_id, "AAA6E3706001") == 0)
		{
    			memset(model, '\0', sizeof(model));
			strcpy(model, "APC77BAM-P01");
		}
		else if (strcmp(rom_id, "AAA6E3721001") == 0)
                {
                        memset(model, '\0', sizeof(model));
                        strcpy(model, "APP761AM-P01");
                }
		else if (strcmp(rom_id, "AAA6E3722001") == 0)
                {
                        memset(model, '\0', sizeof(model));
                        strcpy(model, "APO771AM-P01");
                }

		read_csman(csmfd, CSID_C_WLANAP_DENY_ALL, &deny_all, sizeof(deny_all) ,CSM_R_ZERO);
    		memset(beid, 0, sizeof(beid));
        	read_csman(csmfd, CSID_C_REMOTE_AP_TRUSTAP_LOCK_MAC, beid, 6, CSM_R_ZSTR);
        	sprintf(ac_mac,"%02X:%02X:%02X:%02X:%02X:%02X",beid[0],beid[1],beid[2],beid[3],beid[4],beid[5]);
		if (strcmp(ac_mac, "00:00:00:00:00:00") == 0)
    			memset(ac_mac, '\0', sizeof(ac_mac));

		read_csman(csmfd, CSID_C_REMOTE_AP_PROTOCOL_AP, &protocol_ap, sizeof(protocol_ap) ,CSM_R_ZERO);

		l = snprintf(buf, sizeof(buf), "HTTP/1.1 200 OK\r\n"
					//"CACHE-CONTROL: max-age=120\r\n"
					"CACHE-CONTROL: max-age=35\r\n"
					/*"DATE: ...\r\n"*/
					"ST: %.*s%s\r\n"
					"USN: %s::%.*s%s\r\n"
					"EXT: \r\n"
					"SERVER: " MINIUPNPD_SERVER_STRING "\r\n"
					"LOCATION: http://%s:%u" ROOTDESC_PATH "\r\n"
					"OPT: \"http://schemas.upnp.org/upnp/1/0/\"; ns=01\r\n" /* UDA v1.1 */
					"01-NLS: %u\r\n" /* same as BOOTID. UDA v1.1 */
					"BOOTID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
					"CONFIGID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
					"MAC: %s\r\n" /* AMIT specific */
					"MIB_LOCATION: http://%s/ap_mib\r\n" /* AMIT specific */
					"SNMP_STA: %d\r\n" /* AMIT specific */
					"SNMP_GET_COM: %s\r\n" /* AMIT specific */
					"DEVICE_NAME: %s\r\n" /* AMIT specific */
					"MODEL: %s\r\n" /* AMIT specific */
					"BLACK_LIST: %d\r\n" /* AMIT specific */
					"LOCK: %s\r\n" /* AMIT specific */
					"PROTOCOL_AP: %d\r\n" /* AMIT specific */
					"\r\n",
					st_len, st, suffix,
					uuidvalue, st_len, st, suffix,
					host, (unsigned int)port,
					upnp_bootid, upnp_bootid, upnp_configid, ap_mac, host, snmp_sta, snmp_get_community, device_name, model, 
					deny_all, ac_mac, protocol_ap);
	}
	else if (ac_enable && strstr(st,"rootdevice"))
	{
    		uint8_t beid[6];
    		memset(beid, 0, sizeof(beid));
		char ac_mac[32];
    		memset(ac_mac, '\0', sizeof(ac_mac));

        	read_csman(csmfd, CSID_S_SYS_IF_LAN0_MAC, beid, 6, CSM_R_ZSTR);
        	sprintf(ac_mac,"%02X:%02X:%02X:%02X:%02X:%02X",beid[0],beid[1],beid[2],beid[3],beid[4],beid[5]);

		l = snprintf(buf, sizeof(buf), "HTTP/1.1 200 OK\r\n"
					"CACHE-CONTROL: max-age=120\r\n"
					/*"DATE: ...\r\n"*/
					"ST: %.*s%s\r\n"
					"USN: %s::%.*s%s\r\n"
					"EXT: %s\r\n"
					"SERVER: " MINIUPNPD_SERVER_STRING "\r\n"
					"LOCATION: http://%s:%u" ROOTDESC_PATH "\r\n"
					"OPT: \"http://schemas.upnp.org/upnp/1/0/\"; ns=01\r\n" /* UDA v1.1 */
					"01-NLS: %u\r\n" /* same as BOOTID. UDA v1.1 */
					"BOOTID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
					"CONFIGID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
					"MAC: %s\r\n" /* AMIT specific */
					"\r\n",
					st_len, st, suffix,
					uuidvalue, st_len, st, suffix,
					ext,
					host, (unsigned int)port,
					upnp_bootid, upnp_bootid, upnp_configid, ac_mac);
	}
	else
	{
		l = snprintf(buf, sizeof(buf), "HTTP/1.1 200 OK\r\n"
					"CACHE-CONTROL: max-age=120\r\n"
					/*"DATE: ...\r\n"*/
					"ST: %.*s%s\r\n"
					"USN: %s::%.*s%s\r\n"
					"EXT: %s\r\n"
					"SERVER: " MINIUPNPD_SERVER_STRING "\r\n"
					"LOCATION: http://%s:%u" ROOTDESC_PATH "\r\n"
					"OPT: \"http://schemas.upnp.org/upnp/1/0/\"; ns=01\r\n" /* UDA v1.1 */
					"01-NLS: %u\r\n" /* same as BOOTID. UDA v1.1 */
					"BOOTID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
					"CONFIGID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
					"\r\n",
					st_len, st, suffix,
					uuidvalue, st_len, st, suffix,
					ext,
					host, (unsigned int)port,
					upnp_bootid, upnp_bootid, upnp_configid);
	}

    close_csman(csmfd);

	addrlen = (addr->sa_family == AF_INET6)
			? sizeof(struct sockaddr_in6) : sizeof(struct sockaddr_in);

	n = sendto(s, buf, l, 0,
					addr, addrlen);
	sockaddr_to_string(addr, addr_str, sizeof(addr_str));
	syslog(LOG_INFO, "SSDP Announce %d bytes to %s ST: %.*s",n,
					addr_str,
					l, buf);
	if(n < 0)
	{
			/* XXX handle EINTR, EAGAIN, EWOULDBLOCK */
			syslog(LOG_ERR, "sendto(udp): %m");
	}
}

static const char * const known_service_types[] =
{
		"upnp:rootdevice",
		"urn:schemas-upnp-org:device:InternetGatewayDevice:",
		"urn:schemas-upnp-org:device:WANConnectionDevice:",
		"urn:schemas-upnp-org:device:WANDevice:",
		"urn:schemas-upnp-org:service:WANCommonInterfaceConfig:",
		"urn:schemas-upnp-org:service:WANIPConnection:",
		"urn:schemas-upnp-org:service:WANPPPConnection:",
#ifdef ENABLE_L3F_SERVICE
		"urn:schemas-upnp-org:service:Layer3Forwarding:",
#endif
#ifdef ENABLE_6FC_SERVICE
		"url:schemas-upnp-org:service:WANIPv6FirewallControl:",
#endif
		0
};

		static void
SendSSDPNotifies(int s, const char * host, unsigned short port,
				unsigned int lifetime, int ipv6)
{
#ifdef ENABLE_IPV6
		struct sockaddr_storage sockname;
#else
		struct sockaddr_in sockname;
#endif
		int l, n, i=0, fd, ap_enable=0, ac_enable=0;
		char bufr[1024], snmp_get_community[32];
    		uint8_t beid[6];
    		memset(beid, 0, sizeof(beid));
		char ap_mac[32];
    		memset(ap_mac, '\0', sizeof(ap_mac));

		memset(&sockname, 0, sizeof(sockname));
		memset(snmp_get_community,'\0',sizeof(snmp_get_community));
#ifdef ENABLE_IPV6
		if(ipv6)
		{
				struct sockaddr_in6 * p = (struct sockaddr_in6 *)&sockname;
				p->sin6_family = AF_INET6;
				p->sin6_port = htons(SSDP_PORT);
				inet_pton(AF_INET6, LL_SSDP_MCAST_ADDR, &(p->sin6_addr));
		}
		else
#endif
		{
				struct sockaddr_in *p = (struct sockaddr_in *)&sockname;
				p->sin_family = AF_INET;
				p->sin_port = htons(SSDP_PORT);
				p->sin_addr.s_addr = inet_addr(SSDP_MCAST_ADDR);
		}

		while(known_service_types[i])
		{
			fd = open_csman(NULL, 0);
			read_csman(fd, CSID_C_MISC_AP_CONTROLLER_AP, &ap_enable, sizeof(ap_enable) ,CSM_R_ZERO);
			read_csman(fd, CSID_C_MISC_AP_CONTROLLER_AC, &ac_enable, sizeof(ac_enable) ,CSM_R_ZERO);

        		read_csman(fd, CSID_S_SYS_IF_LAN0_MAC, beid, 6, CSM_R_ZSTR);
        		sprintf(ap_mac,"%02X:%02X:%02X:%02X:%02X:%02X",beid[0],beid[1],beid[2],beid[3],beid[4],beid[5]);

			if (ap_enable && i == 0 && strcmp(ap_mac, "00:00:00:00:00:00") != 0)
			{
				int snmp_local_enable=0, snmp_remote_enable=0, snmp_sta=1, deny_all=0, protocol_ap=0;
				char device_name[256], device_name_tmp[256], read_buf[256], model[32], ac_mac[32], rom_id[16];
    				memset(device_name, '\0', sizeof(device_name));
    				memset(device_name_tmp, '\0', sizeof(device_name_tmp));
    				memset(read_buf, '\0', sizeof(read_buf));
    				memset(model, '\0', sizeof(model));
    				memset(ac_mac, '\0', sizeof(ac_mac));
    				memset(rom_id, '\0', sizeof(rom_id));
				FILE *f;

				read_csman(fd, CSID_C_SNMP_LOCAL_ENABLE, &snmp_local_enable, sizeof(snmp_local_enable) ,CSM_R_ZERO);
				read_csman(fd, CSID_C_SNMP_REMOTE_ENABLE, &snmp_remote_enable, sizeof(snmp_remote_enable) ,CSM_R_ZERO);
				if (snmp_local_enable || snmp_remote_enable)
				{
					int snmp_v1=0, snmp_v2=0, snmp_v3=0;
					read_csman(fd, CSID_C_SNMP_VERSION_V1, &snmp_v1, sizeof(snmp_v1) ,CSM_R_ZERO);
					read_csman(fd, CSID_C_SNMP_VERSION_V2C, &snmp_v2, sizeof(snmp_v2) ,CSM_R_ZERO);
					read_csman(fd, CSID_C_SNMP_VERSION_V3, &snmp_v3, sizeof(snmp_v3) ,CSM_R_ZERO);
					snmp_sta = snmp_v1 + snmp_v2*2 + snmp_v3*4;

					read_csman(fd, CSID_C_SNMP_GET_COMMUNITY, snmp_get_community, sizeof(snmp_get_community) ,CSM_R_ZSTR);
				}
				else
					snmp_sta = 0;
				if (sta_not_ready)
				{
					snmp_sta = 8;
					sta_not_ready = 0;
				}

				if ((f = fopen("/tmp/ssdp_ap_customname", "r")) != NULL){
					fgets(read_buf,sizeof(read_buf),f);
					sscanf(read_buf,"%[^\n]",device_name_tmp);
					strcpy(device_name,str_replace(device_name_tmp,"#qutation#","\""));
					fclose(f);
				}
				else{	
					read_csman(fd,CSID_C_REMOTE_AP_TRUSTAP_CUSTOM_NAME,device_name,sizeof(device_name),CSM_R_ZSTR);
					if(!strcmp(device_name,"\0")){
    						memset(device_name, '\0', sizeof(device_name));
						read_csman(fd,CSID_S_SYS_INFO_MODEL_NAME ,device_name,sizeof(device_name),CSM_R_ZSTR);
					}
				}

				read_csman(fd, CSID_S_SYS_INFO_BOOT_PNAME, model, sizeof(model) ,CSM_R_ZSTR);
				read_csman(fd, CSID_S_SYS_INFO_ROM_ID, rom_id, sizeof(rom_id), CSM_R_ZSTR);
				if (strncmp(rom_id, "DDC", 3) == 0)
				{
					if (strcmp(model, "APW771AM_P01") == 0)
					{
    						memset(model, '\0', sizeof(model));
						strcpy(model, "WAP-6150");
					}
					else if (strcmp(rom_id, "DDC6E3706001") == 0)
					{
    						memset(model, '\0', sizeof(model));
						strcpy(model, "WAP-8101");
					}
					else if (strcmp(model, "APC772AM_P01") == 0)
					{
    						memset(model, '\0', sizeof(model));
						strcpy(model, "WAP-6101");
					}
					else if (strcmp(model, "APC761AM_P03") == 0)
					{
    						memset(model, '\0', sizeof(model));
						strcpy(model, "WAP-6103");
					}
				}
				else if (strcmp(rom_id, "SML6E3711001") == 0 && strcmp(model, "APC772AM_P01") == 0)
				{
    					memset(model, '\0', sizeof(model));
					strcpy(model, "DG-WM2005SI");
				}
				else if (strcmp(rom_id, "AAA6E3706001") == 0)
				{
    					memset(model, '\0', sizeof(model));
					strcpy(model, "APC77BAM-P01");
				}
				else if (strcmp(rom_id, "AAA6E3721001") == 0)
                		{
                        		memset(model, '\0', sizeof(model));
                        		strcpy(model, "APP761AM-P01");
                		}
                		else if (strcmp(rom_id, "AAA6E3722001") == 0)
                		{
                        		memset(model, '\0', sizeof(model));
                        		strcpy(model, "APO771AM-P01");
                		}

				read_csman(fd, CSID_C_WLANAP_DENY_ALL, &deny_all, sizeof(deny_all) ,CSM_R_ZERO);
    				memset(beid, 0, sizeof(beid));
        			read_csman(fd, CSID_C_REMOTE_AP_TRUSTAP_LOCK_MAC, beid, 6, CSM_R_ZSTR);
        			sprintf(ac_mac,"%02X:%02X:%02X:%02X:%02X:%02X",beid[0],beid[1],beid[2],beid[3],beid[4],beid[5]);
				if (strcmp(ac_mac, "00:00:00:00:00:00") == 0)
    					memset(ac_mac, '\0', sizeof(ac_mac));

				read_csman(fd, CSID_C_REMOTE_AP_PROTOCOL_AP, &protocol_ap, sizeof(protocol_ap) ,CSM_R_ZERO);

				l = snprintf(bufr, sizeof(bufr),
								"NOTIFY * HTTP/1.1\r\n"
								"HOST: %s:%d\r\n"
								"CACHE-CONTROL: max-age=35\r\n"
								"LOCATION: http://%s:%d" ROOTDESC_PATH"\r\n"
								"SERVER: " MINIUPNPD_SERVER_STRING "\r\n"
								"NT: %s%s\r\n"
								"USN: %s::%s%s\r\n"
								"NTS: ssdp:alive\r\n"
								"OPT: \"http://schemas.upnp.org/upnp/1/0/\"; ns=01\r\n" /* UDA v1.1 */
								"01-NLS: %u\r\n" /* same as BOOTID field. UDA v1.1 */
								"BOOTID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
								"CONFIGID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
								"MAC: %s\r\n" /* AMIT specific */
								"MIB_LOCATION: http://%s/ap_mib\r\n" /* AMIT specific */
								"SNMP_STA: %d\r\n" /* AMIT specific */
								"SNMP_GET_COM: %s\r\n" /* AMIT specific */
								"DEVICE_NAME: %s\r\n" /* AMIT specific */
								"MODEL: %s\r\n" /* AMIT specific */
								"BLACK_LIST: %d\r\n" /* AMIT specific */
								"LOCK: %s\r\n" /* AMIT specific */
								"PROTOCOL_AP: %d\r\n" /* AMIT specific */
								"\r\n",
								ipv6 ? "[" LL_SSDP_MCAST_ADDR "]" : SSDP_MCAST_ADDR,
								SSDP_PORT,
								host, port,
								known_service_types[i], (i==0?"":"1"),
								uuidvalue, known_service_types[i], (i==0?"":"1"),
								upnp_bootid, upnp_bootid, upnp_configid, ap_mac, host, snmp_sta, snmp_get_community,
								device_name, model, deny_all, ac_mac, protocol_ap);
			}
			else if (ac_enable && i == 0)
			{
				char ac_mac[32], ac_ip_str[16], ac_nm_str[16], ip_pool_start_str[16], ip_pool_end_str[16];
    				memset(ac_mac, '\0', sizeof(ac_mac));
				memset(ac_ip_str, '\0', sizeof(ac_ip_str));
				memset(ac_nm_str, '\0', sizeof(ac_nm_str));
				memset(ip_pool_start_str, '\0', sizeof(ip_pool_start_str));
				memset(ip_pool_end_str, '\0', sizeof(ip_pool_end_str));
				struct in_addr ac_ip;
    				ac_ip.s_addr = 0;
				struct in_addr ac_nm;
    				ac_nm.s_addr = 0;
				int dhcp_enable=0;
				char dhcp_sta[4];
				memset(dhcp_sta, '\0', sizeof(dhcp_sta));
				struct in_addr ip_pool_start;
    				ip_pool_start.s_addr = 0;
				struct in_addr ip_pool_end;
    				ip_pool_end.s_addr = 0;
    				uint8_t beid[6];
    				memset(beid, 0, sizeof(beid));

        			read_csman(fd, CSID_S_SYS_IF_LAN0_MAC, beid, 6, CSM_R_ZSTR);
        			sprintf(ac_mac,"%02X:%02X:%02X:%02X:%02X:%02X",beid[0],beid[1],beid[2],beid[3],beid[4],beid[5]);

				read_csman(fd, CSID_C_LOCAL_LANIP, &ac_ip, 4, CSM_R_ZERO);
				inet_ntop(AF_INET, &ac_ip, ac_ip_str, INET_ADDRSTRLEN);

				read_csman(fd, CSID_C_DHCP_SVR_ENABLE, &dhcp_enable, sizeof(dhcp_enable) ,CSM_R_ZERO);
				if (dhcp_enable)
					strcpy(dhcp_sta, "ON");
				else
					strcpy(dhcp_sta, "OFF");

				if (strcmp(host, ac_ip_str) == 0)
				{
					read_csman(fd, CSID_C_LOCAL_LANNM, &ac_nm, 4, CSM_R_ZERO);
					read_csman(fd, CSID_C_DHCP_SVR_IPPOOL_START, &ip_pool_start, 4, CSM_R_ZERO);
					read_csman(fd, CSID_C_DHCP_SVR_IPPOOL_END, &ip_pool_end, 4, CSM_R_ZERO);
				}
				else
				{
					int j;
					for (j=0;j<4;j++)
					{
						read_csman(fd, CSID_C_VLAN_SVR_IP+j, &ac_ip, 4, CSM_R_ZERO);
						inet_ntop(AF_INET, &ac_ip, ac_ip_str, INET_ADDRSTRLEN);
						if (strcmp(host, ac_ip_str) == 0)
						{
							read_csman(fd, CSID_C_VLAN_SVR_MASK+j, &ac_nm, 4, CSM_R_ZERO);
							read_csman(fd, CSID_C_VLAN_SVR_IPPOOL_START+j, &ip_pool_start, 4, CSM_R_ZERO);
							read_csman(fd, CSID_C_VLAN_SVR_IPPOOL_END+j, &ip_pool_end, 4, CSM_R_ZERO);
							break;
						}
					}
				}

				inet_ntop(AF_INET, &ac_nm, ac_nm_str, INET_ADDRSTRLEN);
				inet_ntop(AF_INET, &ip_pool_start, ip_pool_start_str, INET_ADDRSTRLEN);
				inet_ntop(AF_INET, &ip_pool_end, ip_pool_end_str, INET_ADDRSTRLEN);

				l = snprintf(bufr, sizeof(bufr),
								"NOTIFY * HTTP/1.1\r\n"
								"HOST: %s:%d\r\n"
								"CACHE-CONTROL: max-age=%u\r\n"
								"LOCATION: http://%s:%d" ROOTDESC_PATH"\r\n"
								"SERVER: " MINIUPNPD_SERVER_STRING "\r\n"
								"NT: %s%s\r\n"
								"USN: %s::%s%s\r\n"
								"NTS: ssdp:alive\r\n"
								"OPT: \"http://schemas.upnp.org/upnp/1/0/\"; ns=01\r\n" /* UDA v1.1 */
								"01-NLS: %u\r\n" /* same as BOOTID field. UDA v1.1 */
								"BOOTID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
								"CONFIGID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
								"MAC: %s\r\n" /* AMIT specific */
								"EXT: AC_IP: %s, AC_NM: %s, AC_DHCP: %s, AC_AVAIL_IP: %s - %s\r\n" /* AMIT specific */
								"\r\n",
								ipv6 ? "[" LL_SSDP_MCAST_ADDR "]" : SSDP_MCAST_ADDR,
								SSDP_PORT,
								lifetime,
								host, port,
								known_service_types[i], (i==0?"":"1"),
								uuidvalue, known_service_types[i], (i==0?"":"1"),
								upnp_bootid, upnp_bootid, upnp_configid, ac_mac, host, ac_nm_str, dhcp_sta, ip_pool_start_str, ip_pool_end_str);
			}
			else
			{
				l = snprintf(bufr, sizeof(bufr),
								"NOTIFY * HTTP/1.1\r\n"
								"HOST: %s:%d\r\n"
								"CACHE-CONTROL: max-age=%u\r\n"
								"LOCATION: http://%s:%d" ROOTDESC_PATH"\r\n"
								"SERVER: " MINIUPNPD_SERVER_STRING "\r\n"
								"NT: %s%s\r\n"
								"USN: %s::%s%s\r\n"
								"NTS: ssdp:alive\r\n"
								"OPT: \"http://schemas.upnp.org/upnp/1/0/\"; ns=01\r\n" /* UDA v1.1 */
								"01-NLS: %u\r\n" /* same as BOOTID field. UDA v1.1 */
								"BOOTID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
								"CONFIGID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
								"\r\n",
								ipv6 ? "[" LL_SSDP_MCAST_ADDR "]" : SSDP_MCAST_ADDR,
								SSDP_PORT,
								lifetime,
								host, port,
								known_service_types[i], (i==0?"":"1"),
								uuidvalue, known_service_types[i], (i==0?"":"1"),
								upnp_bootid, upnp_bootid, upnp_configid );
			}
			close_csman(fd);

				if(l<0)
				{
						syslog(LOG_ERR, "SendSSDPNotifies() snprintf error");
						continue;
				}
				if((unsigned int)l >= sizeof(bufr))
				{
						syslog(LOG_WARNING, "SendSSDPNotifies(): truncated output");
						l = sizeof(bufr);
				}
				n = sendto(s, bufr, l, 0,
								(struct sockaddr *)&sockname,
#ifdef ENABLE_IPV6
								ipv6 ? sizeof(struct sockaddr_in6) : sizeof(struct sockaddr_in)
#else
								sizeof(struct sockaddr_in)
#endif
						  );
				if(n < 0)
				{
						/* XXX handle EINTR, EAGAIN, EWOULDBLOCK */
						syslog(LOG_ERR, "sendto(udp_notify=%d, %s): %m", s,
										host ? host : "NULL");
				}
				i++;
		}
}

		void
SendSSDPNotifies2(int * sockets,
				unsigned short port,
				unsigned int lifetime)
{
		int i;
		struct lan_addr_s * lan_addr;
		for(i=0, lan_addr = lan_addrs.lh_first;
						lan_addr != NULL;
						lan_addr = lan_addr->list.le_next)
		{
				SendSSDPNotifies(sockets[i], lan_addr->str, port,
								lifetime, 0);
				i++;
#ifdef ENABLE_IPV6
				SendSSDPNotifies(sockets[i], ipv6_addr_for_http_with_brackets, port,
								lifetime, 1);
				i++;
#endif
		}
}

/* ProcessSSDPRequest()
 * process SSDP M-SEARCH requests and responds to them */
		void
ProcessSSDPRequest(int s, unsigned short port)
{
		int n;
		char bufr[1500];
		socklen_t len_r;
#ifdef ENABLE_IPV6
		struct sockaddr_storage sendername;
		len_r = sizeof(struct sockaddr_storage);
#else
		struct sockaddr_in sendername;
		len_r = sizeof(struct sockaddr_in);
#endif

		n = recvfrom(s, bufr, sizeof(bufr), 0,
						(struct sockaddr *)&sendername, &len_r);
		if(n < 0)
		{
				/* EAGAIN, EWOULDBLOCK, EINTR : silently ignore (try again next time)
				 * other errors : log to LOG_ERR */
				if(errno != EAGAIN &&
								errno != EWOULDBLOCK &&
								errno != EINTR)
				{
						syslog(LOG_ERR, "recvfrom(udp): %m");
				}
				return;
		}
		ProcessSSDPData(s, bufr, n, (struct sockaddr *)&sendername, port);

}

int getrandnum(int min,int max)
{
            unsigned int ret;
            if(min>=max || max==0)
               return 0;
              srand((int)time(0));
        do
            {
                    ret = rand()%max;
            }
            while(ret>max || ret<min);
            return ret;
}

void
ProcessSSDPData(int s, const char *bufr, int n,
				const struct sockaddr * sender, unsigned short port) {
		int i, l, fd, altered = 1, j=0;
		struct lan_addr_s * lan_addr = NULL;
		const char * st = NULL;
		int st_len = 0, tmp_len = 0, tmp_len2 = 0;
		char sender_str[64], dhcp_enable[4], ac_nm_str[16], ap_ip_str[16], ac_ip_str[16], mib_location[64], buf[128], trap_ip_str[16];
		const char * announced_host = NULL;
		char * tmp = NULL;
		char * tmp2 = NULL;
		int ap_enable = 0, ac_enable = 0, lower_bound, counter=0, upper_bound, chosen_ip;
		struct in_addr ap_ip;
    		ap_ip.s_addr = 0;
  		char * pch;
		char *delim = ".";
		char ap_ip_str2[16], ap_ip_str3[16];
		memset(ap_ip_str2, '\0', sizeof(ap_ip_str2));
		memset(ap_ip_str3, '\0', sizeof(ap_ip_str3));
		char lock_mac[32], ac_mac_str[32];
		memset(lock_mac, '\0', sizeof(lock_mac));
		memset(ac_mac_str, '\0', sizeof(ac_mac_str));
    		uint8_t beid[6];
    		memset(beid, 0, sizeof(beid));

		/* get the string representation of the sender address */
		sockaddr_to_string(sender, sender_str, sizeof(sender_str));

		fd = open_csman(NULL, 0);
		read_csman(fd, CSID_C_MISC_AP_CONTROLLER_AP, &ap_enable, sizeof(ap_enable) ,CSM_R_ZERO);
		read_csman(fd, CSID_C_MISC_AP_CONTROLLER_AC, &ac_enable, sizeof(ac_enable) ,CSM_R_ZERO);

        	read_csman(fd, CSID_C_REMOTE_AP_TRUSTAP_LOCK_MAC, beid, 6, CSM_R_ZSTR);
        	sprintf(lock_mac,"%02X:%02X:%02X:%02X:%02X:%02X",beid[0],beid[1],beid[2],beid[3],beid[4],beid[5]);
		if (strcmp(lock_mac, "00:00:00:00:00:00") == 0)
    			memset(lock_mac, '\0', sizeof(lock_mac));

		if(memcmp(bufr, "NOTIFY", 6) == 0)
		{
				/* ignore NOTIFY packets. We could log the sender and device type */
			if (ap_enable)
			{
				i = 0;
				while (i < n)
				{
					while((i < n - 1) && (bufr[i] != '\r' || bufr[i+1] != '\n'))
						i++;
					i += 2;
					if((i < n - 3) && (strncasecmp(bufr+i, "MAC:", 4) == 0))
					{
						tmp = bufr+i+5;
						tmp_len2 = 0;
						while(tmp[tmp_len2]!='\r' && tmp[tmp_len2]!='\n' && (tmp + tmp_len2 < bufr + n))
							tmp_len2++;
						memcpy(ac_mac_str, tmp, tmp_len2);
					}

					if((i < n - 3) && (strncasecmp(bufr+i, "EXT:", 4) == 0))
					{
						if (strlen(lock_mac) > 0 && strcmp(lock_mac, ac_mac_str) != 0)
							break;

						memset(dhcp_enable, '\0', sizeof(dhcp_enable));
						memset(ac_nm_str, '\0', sizeof(ac_nm_str));
						memset(ap_ip_str, '\0', sizeof(ap_ip_str));
						memset(ac_ip_str, '\0', sizeof(ac_ip_str));
						memset(trap_ip_str, '\0', sizeof(trap_ip_str));
						tmp = strstr(bufr+i+4, "AC_DHCP");
						if (tmp != NULL)
						{
							tmp = tmp + 8;
							tmp_len = 0;
							while((*tmp == ' ' || *tmp == '\t') && (tmp < bufr + n))
								tmp++;
							while(tmp[tmp_len]!='\r' && tmp[tmp_len]!='\n' && tmp[tmp_len]!=',' && (tmp + tmp_len < bufr + n))
								tmp_len++;
							memcpy(dhcp_enable, tmp, tmp_len);

							tmp = strstr(bufr+i+4, "AC_IP");
							tmp = tmp + 6;
							tmp_len = 0;
							while((*tmp == ' ' || *tmp == '\t') && (tmp < bufr + n))
								tmp++;
							while(tmp[tmp_len]!='\r' && tmp[tmp_len]!='\n' && tmp[tmp_len]!=',' && (tmp + tmp_len < bufr + n))
								tmp_len++;
							memcpy(ac_ip_str, tmp, tmp_len);
							read_csman(fd, CSID_C_SNMP_TRIP_IP+1, trap_ip_str, sizeof(trap_ip_str), CSM_R_ZSTR);
							if (strcmp(ac_ip_str, trap_ip_str) != 0)
								write_csman(fd, CSID_C_SNMP_TRIP_IP+1, ac_ip_str, strlen(ac_ip_str)+1, 0);

							read_csman(fd, CSID_C_LOCAL_LANIP, &ap_ip, 4, CSM_R_ZERO);
							inet_ntop(AF_INET, &ap_ip, ap_ip_str, INET_ADDRSTRLEN);

							tmp = strstr(bufr+i+4, "AC_NM");
							tmp = tmp + 6;
							tmp_len = 0;
							while((*tmp == ' ' || *tmp == '\t') && (tmp < bufr + n))
								tmp++;
							while(tmp[tmp_len]!='\r' && tmp[tmp_len]!='\n' && tmp[tmp_len]!=',' && (tmp + tmp_len < bufr + n))
								tmp_len++;
							memcpy(ac_nm_str, tmp, tmp_len);
							//syslog(LOG_NOTICE, ">>>>>>>>>ac_nm:%s", ac_nm_str);
							struct in_addr ac_nm;
    							ac_nm.s_addr = 0;
							inet_pton(AF_INET, ac_nm_str, &ac_nm);
							struct in_addr ac_ip;
    							ac_ip.s_addr = 0;
							inet_pton(AF_INET, ac_ip_str, &ac_ip);

							if ((ac_ip.s_addr & ac_nm.s_addr) != (ap_ip.s_addr & ac_nm.s_addr))
							{
							if (strcmp(dhcp_enable, "ON") == 0)
							{
								int ap_cnt_type = 0;
								syslog(LOG_NOTICE, ">>>>>>>>>setting device network type to DHCP!");
								write_csman(fd, CSID_C_LOCAL_AP_CNT_TYPE, &ap_cnt_type, sizeof(ap_cnt_type), 0);
								write_csman(fd, CSID_S_LOCAL_AP_ALTERED, &altered, sizeof(altered), 0);
								write_csman(fd, CSID_S_LOCAL_LAN_ALTERED, &altered, sizeof(altered), 0);

								write_csman(fd, CSID_C_SNMP_LOCAL_ENABLE, &altered, sizeof(altered), 0);
							}
							else if (strcmp(dhcp_enable, "OFF") == 0)
							{
								int ap_cnt_type = 0;

								read_csman(fd, CSID_C_LOCAL_AP_CNT_TYPE, &ap_cnt_type, sizeof(ap_cnt_type) ,CSM_R_ZERO);

								syslog(LOG_NOTICE, ">>>>>>>>>setting device network type to static!");
								ap_cnt_type = 1;
								write_csman(fd, CSID_C_LOCAL_AP_CNT_TYPE, &ap_cnt_type, sizeof(ap_cnt_type), 0);

								write_csman(fd, CSID_C_FIX_AP_IF_NM, &ac_nm, sizeof(struct in_addr) , 0);

								memset(ap_ip_str, '\0', sizeof(ap_ip_str));

								tmp = strstr(bufr+i+4, "AC_AVAIL_IP");
								tmp = tmp + 12;
								tmp_len = 0;
								while((*tmp == ' ' || *tmp == '\t') && (tmp < bufr + n))
									tmp++;
								while(tmp[tmp_len]!='\r' && tmp[tmp_len]!='\n' && tmp[tmp_len]!=',' && tmp[tmp_len]!=' ' && tmp[tmp_len]!='-' && (tmp + tmp_len < bufr + n))
									tmp_len++;
								memcpy(ap_ip_str, tmp, tmp_len);
  								//pch = strtok(ap_ip_str,delim);
  								pch = strtok(ac_ip_str,delim);
  								while (pch != NULL)
  								{
									lower_bound = atoi(pch);
									if (counter < 3)
									{
										strcat(ap_ip_str3, pch);
										strcat(ap_ip_str3, ".");
										counter++;
									}
    									pch = strtok (NULL, delim);
  								}
								tmp_len += 3;
								tmp = tmp + tmp_len;
								tmp_len = 0;
								while(tmp[tmp_len]!='\r' && tmp[tmp_len]!='\n' && tmp[tmp_len]!=' ' && (tmp + tmp_len < bufr + n))
									tmp_len++;
								memcpy(ap_ip_str2, tmp, tmp_len);
  								pch = strtok(ap_ip_str2,delim);
  								while (pch != NULL)
  								{
									upper_bound = atoi(pch);
    									pch = strtok (NULL, delim);
  								}
								chosen_ip = getrandnum(lower_bound, upper_bound);
								memset(ap_ip_str, '\0', sizeof(ap_ip_str));
								sprintf(ap_ip_str, "%s%d", ap_ip_str3, chosen_ip);
								inet_pton(AF_INET, ap_ip_str, &ap_ip);
								write_csman(fd, CSID_C_FIX_AP_IF_IP, &ap_ip, sizeof(struct in_addr) , 0);
								write_csman(fd, CSID_C_LOCAL_LANIP, &ap_ip, sizeof(struct in_addr) , 0);

								write_csman(fd, CSID_S_LOCAL_AP_ALTERED, &altered, sizeof(altered), 0);
								write_csman(fd, CSID_S_LOCAL_LAN_ALTERED, &altered, sizeof(altered), 0);

								write_csman(fd, CSID_C_SNMP_LOCAL_ENABLE, &altered, sizeof(altered), 0);
							}
							}
						}
					}
				}
			}
			else if (ac_enable)
			{
				i = 0;
				struct in_addr ap_ip;
    				ap_ip.s_addr = 0;
				char ap_mac_str[32], tmp_mac_str[32];
				memset(ap_mac_str, '\0', sizeof(ap_mac_str));
				uint8_t beid[6];
				char *delim = ":";
				unsigned char *result;

				while (i < n)
				{
					while((i < n - 1) && (bufr[i] != '\r' || bufr[i+1] != '\n'))
						i++;
					i += 2;
					if((i < n - 3) && (strncasecmp(bufr+i, "LOCATION:", 9) == 0))
					{
						memset(ap_ip_str, '\0', sizeof(ap_ip_str));
						memset(mib_location, '\0', sizeof(mib_location));
						tmp = strstr(bufr+i+9, "http://");
						if (tmp != NULL)
						{
							tmp = tmp + 7;
							tmp_len = 0;
							while(tmp[tmp_len]!='\r' && tmp[tmp_len]!='\n' && tmp[tmp_len]!=':' && (tmp + tmp_len < bufr + n))
								tmp_len++;
							memcpy(ap_ip_str, tmp, tmp_len);
							inet_pton(AF_INET, ap_ip_str, &ap_ip);
							//syslog(LOG_NOTICE, ">>>>>>>>>location ip: %s", ap_ip_str);
						}
					}
					else if((i < n - 3) && (strncasecmp(bufr+i, "USN:", 4) == 0))
					{
						tmp = strstr(bufr+i+4, "InternetGatewayDevice");
						if (tmp != NULL)
						{
							struct in_addr tmp_ap_ip;
    							tmp_ap_ip.s_addr = 0;
							char tmp_ap_ip_str[16];
							int pass=0;

							for (j=0; j<128; j++)
							{	
								memset(tmp_ap_ip_str, '\0', sizeof(tmp_ap_ip_str));
    								tmp_ap_ip.s_addr = 0;

								read_csman(fd, CSID_S_REMOTE_AP_TRUSTAP_IP+j, &tmp_ap_ip, 4, CSM_R_ZERO);
								inet_ntop(AF_INET, &tmp_ap_ip, tmp_ap_ip_str, INET_ADDRSTRLEN);

								if (strcmp(tmp_ap_ip_str, "0.0.0.0") == 0)
								{
									break;
								}
								else if (strcmp(tmp_ap_ip_str, ap_ip_str) == 0)
								{
									pass=1;
									break;
								}
							}

							if (!pass)
							{
								for (j=0; j<128; j++)
								{	
									memset(tmp_ap_ip_str, '\0', sizeof(tmp_ap_ip_str));
    									tmp_ap_ip.s_addr = 0;

									read_csman(fd, CSID_S_REMOTE_AP_ROUGEAP_IP+j, &tmp_ap_ip, 4, CSM_R_ZERO);
									inet_ntop(AF_INET, &tmp_ap_ip, tmp_ap_ip_str, INET_ADDRSTRLEN);

									if (strcmp(tmp_ap_ip_str, "0.0.0.0") == 0)
									{
										write_csman(fd, CSID_S_REMOTE_AP_ROUGEAP_IP+j, &ap_ip, sizeof(struct in_addr) , 0);
										break;
									}
									else if (strcmp(tmp_ap_ip_str, ap_ip_str) == 0)
									{
										break;
									}
								}
/*
								FILE *pp;
								char buf[64];
								int read;
								int k=0;

								if ((pp = popen("arp", "r")) == NULL) {
									printf("popen() error!\n");
									syslog(LOG_NOTICE,">>>>>>>>popen(error!)\n");
								}
								else
								{
									while(feof(pp)== 0)
									{
										memset(buf,'\0',sizeof(buf));
										read = fscanf(pp, "%s\n", buf);
										tmp2 = strstr(buf, ap_ip_str);
										if (tmp2 != NULL)
										{
											memset(buf,'\0',sizeof(buf));
											read = fscanf(pp, "%s\n", buf);
											read = fscanf(pp, "%s\n", buf);
											strcpy(ap_mac_str, buf);
											//syslog(LOG_NOTICE,">>>>>>>>ap_mac=%s\n", ap_mac_str);
											break;
										}
									}
									pclose(pp);
								}

								for (j=0; j<128; j++)
								{	
									memset(tmp_mac_str, '\0', sizeof(tmp_mac_str));
                							memset(beid, 0, sizeof(beid));

									read_csman(fd, CSID_S_REMOTE_AP_ROUGEAP_MAC+j, beid, 6, CSM_R_ZSTR);

                							sprintf(tmp_mac_str,"%02x:%02x:%02x:%02x:%02x:%02x",beid[0],beid[1],beid[2],beid[3],beid[4],beid[5]);
									//syslog(LOG_NOTICE,">>>>>>>>j=%d, ap_mac=%s, tmp_mac=%s\n", j, ap_mac_str, tmp_mac_str);
									if (strcmp(tmp_mac_str, "00:00:00:00:00:00") == 0)
									{
										result = strtok( ap_mac_str, delim );

        									beid[k]=(unsigned)strtol(result,NULL,16);

       										for(k=1;k<6;k++)
        									{
            										result = strtok( NULL, delim );
            										beid[k]=(unsigned)strtol(result,NULL,16);
        									}
        									write_csman(fd, CSID_S_REMOTE_AP_ROUGEAP_MAC+j, beid , 6, 0);

										break;
									}
									else if (strcmp(tmp_mac_str, ap_mac_str) == 0)
									{
										break;
									}
								}
*/
							}
						}
					}
					else if((i < n - 3) && (strncasecmp(bufr+i, "MAC:", 4) == 0))
					{
						int k=0;
						tmp = bufr+i+5;
						tmp_len2 = 0;
						while(tmp[tmp_len2]!='\r' && tmp[tmp_len2]!='\n' && (tmp + tmp_len2 < bufr + n))
							tmp_len2++;
						memcpy(ap_mac_str, tmp, tmp_len2);

						for (j=0; j<128; j++)
						{	
							memset(tmp_mac_str, '\0', sizeof(tmp_mac_str));
                					memset(beid, 0, sizeof(beid));

							read_csman(fd, CSID_S_REMOTE_AP_TRUSTAP_MAC+j, beid, 6, CSM_R_ZSTR);

                					sprintf(tmp_mac_str,"%02X:%02X:%02X:%02X:%02X:%02X",beid[0],beid[1],beid[2],beid[3],beid[4],beid[5]);
							if (strcmp(tmp_mac_str, "00:00:00:00:00:00") == 0)
							{
								result = strtok( ap_mac_str, delim );

        							beid[k]=(unsigned)strtol(result,NULL,16);

       								for(k=1;k<6;k++)
        							{
            								result = strtok( NULL, delim );
            								beid[k]=(unsigned)strtol(result,NULL,16);
        							}
        							write_csman(fd, CSID_S_REMOTE_AP_TRUSTAP_MAC+j, beid , 6, 0);

								char get_proposal[16];
								memset(get_proposal, '\0', sizeof(get_proposal));
                						sprintf(get_proposal, "AP_Proposal_%d", j);
								write_csman(fd, CSID_S_REMOTE_AP_PROPOSAL_GET_SAVE_NAME+j, get_proposal, strlen(get_proposal)+1, 0);

								break;
							}
							else if (strcmp(tmp_mac_str, ap_mac_str) == 0)
							{
								break;
							}
						}
					}
					else if((i < n - 3) && (strncasecmp(bufr+i, "MIB_LOCATION:", 13) == 0))
					{
						memset(ap_ip_str, '\0', sizeof(ap_ip_str));
						memset(mib_location, '\0', sizeof(mib_location));
						tmp = strstr(bufr+i+13, "http://");
						if (tmp != NULL)
						{
							tmp_len2 = 0;
							while(tmp[tmp_len2]!='\r' && tmp[tmp_len2]!='\n' && (tmp + tmp_len2 < bufr + n))
								tmp_len2++;
							memcpy(mib_location, tmp, tmp_len2);

							tmp = tmp + 7;
							tmp_len = 0;
							while(tmp[tmp_len]!='\r' && tmp[tmp_len]!='\n' && tmp[tmp_len]!='/' && (tmp + tmp_len < bufr + n))
								tmp_len++;
							memcpy(ap_ip_str, tmp, tmp_len);
							inet_pton(AF_INET, ap_ip_str, &ap_ip);
							
							if (ap_ip.s_addr != 0)
								write_csman(fd, CSID_S_REMOTE_AP_TRUSTAP_IP+j, &ap_ip, sizeof(struct in_addr) , 0);

							system("mkdir /etc/ap_mib");
							//sprintf(buf, "cd /etc/ap_mib; wget %s", mib_location);
							//system(buf);
							//sprintf(buf, "mv /etc/ap_mib/ap_mib /etc/ap_mib/ap_mib_%d", j);
							//system(buf);
						}
					}
					else if((i < n - 3) && (strncasecmp(bufr+i, "SNMP_STA:", 9) == 0))
					{
						char snmp_sta_str[2];
						int snmp_sta, snmp_v1=0, snmp_v2=0, snmp_v3=0;
						memset(snmp_sta_str, '\0', sizeof(snmp_sta_str));
						memcpy(snmp_sta_str, bufr+i+10, 1);
						snmp_sta=atoi(snmp_sta_str);
						snmp_v1 = snmp_sta % 2;
						snmp_sta = snmp_sta / 2;
						snmp_v2 = snmp_sta % 2;
						snmp_sta = snmp_sta / 2;
						snmp_v3 = snmp_sta % 2;
						write_csman(fd, CSID_S_REMOTE_AP_SNMP_VERSION_V1+j, &snmp_v1, sizeof(snmp_v1), 0);
						write_csman(fd, CSID_S_REMOTE_AP_SNMP_VERSION_V2C+j, &snmp_v2, sizeof(snmp_v2), 0);
						write_csman(fd, CSID_S_REMOTE_AP_SNMP_VERSION_V3+j, &snmp_v3, sizeof(snmp_v3), 0);
					}
					else if((i < n - 3) && (strncasecmp(bufr+i, "SNMP_GET_COM:", 13) == 0))
					{
						char snmp_get_community[32];
						memset(snmp_get_community, '\0', sizeof(snmp_get_community));
						tmp = bufr+i+14;
						tmp_len2 = 0;
						while(tmp[tmp_len2]!='\r' && tmp[tmp_len2]!='\n' && (tmp + tmp_len2 < bufr + n))
							tmp_len2++;
						memcpy(snmp_get_community, tmp, tmp_len2);
						write_csman(fd, CSID_S_REMOTE_AP_SNMP_GET_COMMUNITY+j, snmp_get_community, strlen(snmp_get_community)+1, 0);
					}
					else if((i < n - 3) && (strncasecmp(bufr+i, "PROTOCOL_AP:", 12) == 0))
					{
						char protocol_ap_str[2];
						int protocol_ap=0;
						memset(protocol_ap_str, '\0', sizeof(protocol_ap_str));
						memcpy(protocol_ap_str, bufr+i+13, 1);
						protocol_ap=atoi(protocol_ap_str);
						write_csman(fd, CSID_S_REMOTE_AP_TRUSTAP_PROTOCOL+j, &protocol_ap, sizeof(protocol_ap), 0);
					}
				}
			}
			//return;
		}
		else if(memcmp(bufr, "M-SEARCH", 8) == 0)
		{
			extern int * snotify;
			extern struct runtime_vars v;
			i = 0;
			while(i < n)
			{
				while((i < n - 1) && (bufr[i] != '\r' || bufr[i+1] != '\n'))
					i++;
				i += 2;
				if((i < n - 3) && (strncasecmp(bufr+i, "st:", 3) == 0))
				{
					st = bufr+i+3;
					st_len = 0;
					while((*st == ' ' || *st == '\t') && (st < bufr + n))
						st++;
					while(st[st_len]!='\r' && st[st_len]!='\n' && (st + st_len < bufr + n))
						st_len++;
				}
				if (ap_enable)
				{
					if((i < n - 3) && (strncasecmp(bufr+i, "MAC:", 4) == 0))
					{
						tmp = bufr+i+5;
						tmp_len2 = 0;
						while(tmp[tmp_len2]!='\r' && tmp[tmp_len2]!='\n' && (tmp + tmp_len2 < bufr + n))
							tmp_len2++;
						memcpy(ac_mac_str, tmp, tmp_len2);
					}
					if((i < n - 3) && (strncasecmp(bufr+i, "EXT:", 4) == 0))
					{
						if (strlen(lock_mac) > 0 && strcmp(lock_mac, ac_mac_str) != 0)
							break;

						memset(dhcp_enable, '\0', sizeof(dhcp_enable));
						memset(ac_nm_str, '\0', sizeof(ac_nm_str));
						memset(ap_ip_str, '\0', sizeof(ap_ip_str));
						memset(ac_ip_str, '\0', sizeof(ac_ip_str));
						memset(trap_ip_str, '\0', sizeof(trap_ip_str));
						tmp = strstr(bufr+i+4, "AC_DHCP");
						if (tmp != NULL)
						{
							tmp = tmp + 8;
							tmp_len = 0;
							while((*tmp == ' ' || *tmp == '\t') && (tmp < bufr + n))
								tmp++;
							while(tmp[tmp_len]!='\r' && tmp[tmp_len]!='\n' && tmp[tmp_len]!=',' && (tmp + tmp_len < bufr + n))
								tmp_len++;
							memcpy(dhcp_enable, tmp, tmp_len);

							tmp = strstr(bufr+i+4, "AC_IP");
							tmp = tmp + 6;
							tmp_len = 0;
							while((*tmp == ' ' || *tmp == '\t') && (tmp < bufr + n))
								tmp++;
							while(tmp[tmp_len]!='\r' && tmp[tmp_len]!='\n' && tmp[tmp_len]!=',' && (tmp + tmp_len < bufr + n))
								tmp_len++;
							memcpy(ac_ip_str, tmp, tmp_len);
							read_csman(fd, CSID_C_SNMP_TRIP_IP, trap_ip_str, sizeof(trap_ip_str), CSM_R_ZSTR);
							if (strcmp(ac_ip_str, trap_ip_str) != 0)
								write_csman(fd, CSID_C_SNMP_TRIP_IP, ac_ip_str, strlen(ac_ip_str)+1, 0);

							read_csman(fd, CSID_C_LOCAL_LANIP, &ap_ip, 4, CSM_R_ZERO);
							inet_ntop(AF_INET, &ap_ip, ap_ip_str, INET_ADDRSTRLEN);

							tmp = strstr(bufr+i+4, "AC_NM");
							tmp = tmp + 6;
							tmp_len = 0;
							while((*tmp == ' ' || *tmp == '\t') && (tmp < bufr + n))
								tmp++;
							while(tmp[tmp_len]!='\r' && tmp[tmp_len]!='\n' && tmp[tmp_len]!=',' && (tmp + tmp_len < bufr + n))
								tmp_len++;
							memcpy(ac_nm_str, tmp, tmp_len);
							//syslog(LOG_NOTICE, ">>>>>>>>>ac_nm:%s", ac_nm_str);
							struct in_addr ac_nm;
    							ac_nm.s_addr = 0;
							inet_pton(AF_INET, ac_nm_str, &ac_nm);
							struct in_addr ac_ip;
    							ac_ip.s_addr = 0;
							inet_pton(AF_INET, ac_ip_str, &ac_ip);

							if ((ac_ip.s_addr & ac_nm.s_addr) != (ap_ip.s_addr & ac_nm.s_addr))
							{
							if (strcmp(dhcp_enable, "ON") == 0)
							{
								int ap_cnt_type = 0;
								sta_not_ready=1;
								SendSSDPNotifies2(snotify, (unsigned short)v.port, v.notify_interval << 1);
								syslog(LOG_NOTICE, ">>>>>>>>>setting device network type to DHCP!");
								write_csman(fd, CSID_C_LOCAL_AP_CNT_TYPE, &ap_cnt_type, sizeof(ap_cnt_type), 0);
								write_csman(fd, CSID_S_LOCAL_AP_ALTERED, &altered, sizeof(altered), 0);
								write_csman(fd, CSID_S_LOCAL_LAN_ALTERED, &altered, sizeof(altered), 0);

								write_csman(fd, CSID_C_SNMP_LOCAL_ENABLE, &altered, sizeof(altered), 0);
							}
							else if (strcmp(dhcp_enable, "OFF") == 0)
							{
								int ap_cnt_type = 0;
								char rom_id[16];
								memset(rom_id, '\0', sizeof(rom_id));

								read_csman(fd, CSID_C_LOCAL_AP_CNT_TYPE, &ap_cnt_type, sizeof(ap_cnt_type) ,CSM_R_ZERO);


								//if (ap_cnt_type != 1 || same_subnet(ac_ip_str, ap_ip_str) == -1)
								//{
									sta_not_ready=1;
									SendSSDPNotifies2(snotify, (unsigned short)v.port, v.notify_interval << 1);
									syslog(LOG_NOTICE, ">>>>>>>>>setting device network type to static!");
									ap_cnt_type = 1;
									write_csman(fd, CSID_C_LOCAL_AP_CNT_TYPE, &ap_cnt_type, sizeof(ap_cnt_type), 0);

									write_csman(fd, CSID_C_FIX_AP_IF_NM, &ac_nm, sizeof(struct in_addr) , 0);

									memset(ap_ip_str, '\0', sizeof(ap_ip_str));
									tmp = strstr(bufr+i+4, "AC_AVAIL_IP");
									tmp = tmp + 12;
									tmp_len = 0;
									while((*tmp == ' ' || *tmp == '\t') && (tmp < bufr + n))
										tmp++;
									while(tmp[tmp_len]!='\r' && tmp[tmp_len]!='\n' && tmp[tmp_len]!=',' && tmp[tmp_len]!=' ' && tmp[tmp_len]!='-' && (tmp + tmp_len < bufr + n))
										tmp_len++;
									memcpy(ap_ip_str, tmp, tmp_len);
  									//pch = strtok(ap_ip_str,delim);
  									pch = strtok(ac_ip_str,delim);
  									while (pch != NULL)
  									{
										lower_bound = atoi(pch);
										if (counter < 3)
										{
											strcat(ap_ip_str3, pch);
											strcat(ap_ip_str3, ".");
											counter++;
										}
    										pch = strtok (NULL, delim);
  									}
									syslog(LOG_NOTICE, ">>>>>>>>>ap_ip_str3:%s ,lower_bound:%d", ap_ip_str3, lower_bound);
									tmp_len += 3;
									tmp = tmp + tmp_len;
									tmp_len = 0;
									while(tmp[tmp_len]!='\r' && tmp[tmp_len]!='\n' && tmp[tmp_len]!=' ' && (tmp + tmp_len < bufr + n))
										tmp_len++;
									memcpy(ap_ip_str2, tmp, tmp_len);
  									pch = strtok(ap_ip_str2,delim);
  									while (pch != NULL)
  									{
										upper_bound = atoi(pch);
    										pch = strtok (NULL, delim);
  									}
									syslog(LOG_NOTICE, ">>>>>>>>>upper_bound:%d", upper_bound);
									chosen_ip = getrandnum(lower_bound, upper_bound);
									syslog(LOG_NOTICE, ">>>>>>>>>chosen_ip:%d", chosen_ip);
									memset(ap_ip_str, '\0', sizeof(ap_ip_str));
									sprintf(ap_ip_str, "%s%d", ap_ip_str3, chosen_ip);
									inet_pton(AF_INET, ap_ip_str, &ap_ip);
									write_csman(fd, CSID_C_FIX_AP_IF_IP, &ap_ip, sizeof(struct in_addr) , 0);
									write_csman(fd, CSID_C_LOCAL_LANIP, &ap_ip, sizeof(struct in_addr) , 0);

									write_csman(fd, CSID_S_LOCAL_AP_ALTERED, &altered, sizeof(altered), 0);
									write_csman(fd, CSID_S_LOCAL_LAN_ALTERED, &altered, sizeof(altered), 0);

									write_csman(fd, CSID_C_SNMP_LOCAL_ENABLE, &altered, sizeof(altered), 0);

									//when AC turns into AP mode, do extra stuff
									read_csman(fd, CSID_S_SYS_INFO_ROM_ID, rom_id, sizeof(rom_id), CSM_R_ZSTR);
									if (strstr(rom_id, "6E4001001"))
									{
										write_csman(fd, CSID_S_LOCAL_WANSET_ALTERED, &altered, sizeof(altered), 0);
										system("reboot");
									}
									//write_csman(fd, CSID_C_SNMP_REMOTE_ENABLE, &altered, sizeof(altered), 0);
									//write_csman(fd, CSID_S_SNMP_ALTERED, &altered, sizeof(altered), 0);
									//system("snmpd-action restart &");
								//}
							}
							}
						}
					}
				}
			}
				/*syslog(LOG_INFO, "SSDP M-SEARCH packet received from %s",
				  sender_str );*/
				if(st && (st_len > 0))
				{
						/* TODO : doesnt answer at once but wait for a random time */
						//syslog(LOG_NOTICE, "SSDP M-SEARCH from %s ST: %.*s", sender_str, st_len, st);
						/* find in which sub network the client is */
						if(sender->sa_family == AF_INET)
						{
								for(lan_addr = lan_addrs.lh_first;
												lan_addr != NULL;
												lan_addr = lan_addr->list.le_next)
								{
										if( (((const struct sockaddr_in *)sender)->sin_addr.s_addr & lan_addr->mask.s_addr)
														== (lan_addr->addr.s_addr & lan_addr->mask.s_addr))
												break;
								}
								if (lan_addr == NULL)
								{
										//syslog(LOG_ERR, "Can't find in which sub network the client is");
										close_csman(fd);
										return;
								}
								announced_host = lan_addr->str;
						}
#ifdef ENABLE_IPV6
						else
						{
								/* IPv6 address with brackets */
								announced_host = ipv6_addr_for_http_with_brackets;
						}
#endif
						/* Responds to request with a device as ST header */
						for(i = 0; known_service_types[i]; i++)
						{
								l = (int)strlen(known_service_types[i]);
								if(l<=st_len && (0 == memcmp(st, known_service_types[i], l)))
								{
										syslog(LOG_INFO, "Single search found");
										SendSSDPAnnounce2(s, sender,
														st, st_len, "",
														announced_host, port);
										break;
								}
						}
						/* Responds to request with ST: ssdp:all */
						/* strlen("ssdp:all") == 8 */
						if(st_len==8 && (0 == memcmp(st, "ssdp:all", 8)))
						{
								syslog(LOG_INFO, "ssdp:all found");
								for(i=0; known_service_types[i]; i++)
								{
										l = (int)strlen(known_service_types[i]);
										SendSSDPAnnounce2(s, sender,
														known_service_types[i], l, i==0?"":"1",
														announced_host, port);
								}
						}
						/* responds to request by UUID value */
						l = (int)strlen(uuidvalue);
						if(l==st_len && (0 == memcmp(st, uuidvalue, l)))
						{
								syslog(LOG_INFO, "ssdp:uuid found");
								SendSSDPAnnounce2(s, sender, st, st_len, "",
												announced_host, port);
						}
				}
				else
				{
						syslog(LOG_INFO, "Invalid SSDP M-SEARCH from %s", sender_str);
				}
		}
		else
		{
				syslog(LOG_NOTICE, "Unknown udp packet received from %s", sender_str);
		}
		close_csman(fd);
}

/* This will broadcast ssdp:byebye notifications to inform
 * the network that UPnP is going down. */
		int
SendSSDPGoodbye(int * sockets, int n_sockets)
{
		struct sockaddr_in sockname;
#ifdef ENABLE_IPV6
		struct sockaddr_in6 sockname6;
#endif
		int n, l;
		int i, j;
		char bufr[512];
		int ret = 0;
		int ipv6 = 0;

		memset(&sockname, 0, sizeof(struct sockaddr_in));
		sockname.sin_family = AF_INET;
		sockname.sin_port = htons(SSDP_PORT);
		sockname.sin_addr.s_addr = inet_addr(SSDP_MCAST_ADDR);
#ifdef ENABLE_IPV6
		memset(&sockname6, 0, sizeof(struct sockaddr_in6));
		sockname6.sin6_family = AF_INET6;
		sockname6.sin6_port = htons(SSDP_PORT);
		inet_pton(AF_INET6, LL_SSDP_MCAST_ADDR, &(sockname6.sin6_addr));
#endif

		for(j=0; j<n_sockets; j++)
		{
#ifdef ENABLE_IPV6
				ipv6 = j & 1;
#endif
				for(i=0; known_service_types[i]; i++)
				{
						l = snprintf(bufr, sizeof(bufr),
										"NOTIFY * HTTP/1.1\r\n"
										"HOST: %s:%d\r\n"
										"NT: %s%s\r\n"
										"USN: %s::%s%s\r\n"
										"NTS: ssdp:byebye\r\n"
										"OPT: \"http://schemas.upnp.org/upnp/1/0/\"; ns=01\r\n" /* UDA v1.1 */
										"01-NLS: %u\r\n" /* same as BOOTID field. UDA v1.1 */
										"BOOTID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
										"CONFIGID.UPNP.ORG: %u\r\n" /* UDA v1.1 */
										"\r\n",
										ipv6 ? "[" LL_SSDP_MCAST_ADDR "]" : SSDP_MCAST_ADDR,
										SSDP_PORT,
										known_service_types[i], (i==0?"":"1"),
										uuidvalue, known_service_types[i], (i==0?"":"1"),
										upnp_bootid, upnp_bootid, upnp_configid);
						n = sendto(sockets[j], bufr, l, 0,
#ifdef ENABLE_IPV6
										ipv6 ? (struct sockaddr *)&sockname6 : (struct sockaddr *)&sockname,
										ipv6 ? sizeof(struct sockaddr_in6) : sizeof(struct sockaddr_in)
#else
										(struct sockaddr *)&sockname,
										sizeof(struct sockaddr_in)
#endif
								  );
						if(n < 0)
						{
								syslog(LOG_ERR, "SendSSDPGoodbye: sendto(udp_shutdown=%d): %m",
												sockets[j]);
								ret = -1;
						}
				}
		}
		return ret;
}

/* SubmitServicesToMiniSSDPD() :
 * register services offered by MiniUPnPd to a running instance of
 * MiniSSDPd */
int
SubmitServicesToMiniSSDPD(const char * host, unsigned short port) {
		struct sockaddr_un addr;
		int s;
		unsigned char buffer[2048];
		char strbuf[256];
		unsigned char * p;
		int i, l;

		s = socket(AF_UNIX, SOCK_STREAM, 0);
		if(s < 0) {
				syslog(LOG_ERR, "socket(unix): %m");
				return -1;
		}
		addr.sun_family = AF_UNIX;
		strncpy(addr.sun_path, minissdpdsocketpath, sizeof(addr.sun_path));
		if(connect(s, (struct sockaddr *)&addr, sizeof(struct sockaddr_un)) < 0) {
				syslog(LOG_ERR, "connect(\"%s\"): %m", minissdpdsocketpath);
				return -1;
		}
		for(i = 0; known_service_types[i]; i++) {
				buffer[0] = 4;
				p = buffer + 1;
				l = (int)strlen(known_service_types[i]);
				if(i > 0)
						l++;
				CODELENGTH(l, p);
				memcpy(p, known_service_types[i], l);
				if(i > 0)
						p[l-1] = '1';
				p += l;
				l = snprintf(strbuf, sizeof(strbuf), "%s::%s%s",
								uuidvalue, known_service_types[i], (i==0)?"":"1");
				CODELENGTH(l, p);
				memcpy(p, strbuf, l);
				p += l;
				l = (int)strlen(MINIUPNPD_SERVER_STRING);
				CODELENGTH(l, p);
				memcpy(p, MINIUPNPD_SERVER_STRING, l);
				p += l;
				l = snprintf(strbuf, sizeof(strbuf), "http://%s:%u" ROOTDESC_PATH,
								host, (unsigned int)port);
				CODELENGTH(l, p);
				memcpy(p, strbuf, l);
				p += l;
				if(write(s, buffer, p - buffer) < 0) {
						syslog(LOG_ERR, "write(): %m");
						return -1;
				}
		}
		close(s);
		return 0;
}

