#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "csman.h"
#include "SysUserMgntGetUserList.h"
#include <stdio.h>
#include <string.h>

#define NUM_UTILITY_USERLIST	64

#ifdef SNMP_CUSTOMER_SMARTLINK
oid SysUserMgntGetUserList_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,5,11,1 };
#else
oid SysUserMgntGetUserList_variables_oid[] = { 1,3,6,1,4,1,12823,3,3,9,5,11,1 };
#endif

struct variable7 SysUserMgntGetUserList_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{SYSUSERMGNTGETUSERLISTINDEX,ASN_INTEGER,RONLY,var_SysUserMgntGetUserListTable, 3,  { 1 , 1, 1 }},
{SYSUSERMGNTGETUSERLISTDATA,ASN_OCTET_STR,RONLY,var_SysUserMgntGetUserListTable, 3,  { 1 , 1, 2 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the SysUserMgntGetUserList module */
void
init_SysUserMgntGetUserList(void)
{
    DEBUGMSGTL(("SysUserMgntGetUserList", "Initializing\n"));
    REGISTER_MIB("SysUserMgntGetUserList", SysUserMgntGetUserList_variables, variable7,
               SysUserMgntGetUserList_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_SysUserMgntGetUserList(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_SysUserMgntGetUserListTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_SysUserMgntGetUserList above.
 */
unsigned char *
var_SysUserMgntGetUserListTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	/*User data*/
	int i=0;
	int user_enable,user_status,user_level;
	char user_name[128],user_password[128];
	char user_leasetime[128];
	char user_idletimeout[64],user_groupto[64];
	char user_ip[64],user_expiretime[64];
	char user_idletime[64],user_logintime[64];
	char new_user_name[128],new_user_password[128];
	char new_user_leasetime[128];
	char new_user_idletimeout[64],new_user_groupto[64];
	char new_user_ip[64],new_user_expiretime[64];
	char new_user_idletime[64],new_user_logintime[64];
	char list_buf[1024];
	/* get index */
	register int index;
	index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_UTILITY_USERLIST);
		
	if ( index == MATCH_FAILED )
		return NULL;


    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case SYSUSERMGNTGETUSERLISTINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case SYSUSERMGNTGETUSERLISTDATA:
	{
		int index_range=0;
        memset(string,0,sizeof(string));

		index_range = (index-1)*8;
		
		for(i=0;i<8;i++){
			user_enable=0;
			user_status=0;
			user_level=0;
			memset(list_buf,0,sizeof(list_buf));
			memset(user_name,0,sizeof(user_name));
			memset(user_password,0,sizeof(user_password));
			memset(user_leasetime,0,sizeof(user_leasetime));
			memset(user_idletimeout,0,sizeof(user_idletimeout));
			memset(user_groupto,0,sizeof(user_groupto));
			memset(user_ip,0,sizeof(user_ip));
			memset(user_expiretime,0,sizeof(user_expiretime));
			memset(user_idletime,0,sizeof(user_idletime));
			memset(user_logintime,0,sizeof(user_logintime));
			memset(new_user_name,0,sizeof(new_user_name));
			memset(new_user_password,0,sizeof(new_user_password));
			memset(new_user_leasetime,0,sizeof(new_user_leasetime));
			memset(new_user_idletimeout,0,sizeof(new_user_idletimeout));
			memset(new_user_groupto,0,sizeof(new_user_groupto));
			memset(new_user_ip,0,sizeof(new_user_ip));
			memset(new_user_expiretime,0,sizeof(new_user_expiretime));
			memset(new_user_idletime,0,sizeof(new_user_idletime));
			memset(new_user_logintime,0,sizeof(new_user_logintime));
			
			/*Username,CSID_C_RADIUS_USER*/
			read_csman(fd,0x00994000+index_range+i,user_name,sizeof(user_name),TYPE_STR);
			/*UserPassword,CSID_C_RADIUS_PASSWD*/
			read_csman(fd,0x00994200+index_range+i,user_password,sizeof(user_password),TYPE_STR);
			/*UserLevel,CSID_C_RADIUS_LEVEL*/
			read_csman(fd,0x00994600+index_range+i,&user_level,sizeof(unsigned int),TYPE_U32);
			/*UserLeasetime,CSID_C_RADIUS_LEASETIME*/
			read_csman(fd,0x00994800+index_range+i,user_leasetime,sizeof(user_leasetime),TYPE_STR);
			/*UserIdleTime,CSID_C_RADIUS_IDLETIMEOUT*/
			read_csman(fd,0x00994A00+index_range+i,user_idletimeout,sizeof(user_idletimeout),TYPE_STR);
			/*UserGroupto,CSID_C_RADIUS_GROUPTO*/
			read_csman(fd,0x00994C00+index_range+i,user_groupto,sizeof(user_groupto),TYPE_STR);
			/*UserEnable,CSID_C_RADIUS_USER_ENABLE*/
			read_csman(fd,0x00994400+index_range+i,&user_enable,sizeof(unsigned int),TYPE_U32);
			/*UserIP,CSID_C_RADIUS_USER_IP*/
			read_csman(fd,0x00994E00+index_range+i,user_ip,sizeof(user_ip),TYPE_STR);
			/*UserStatus,CSID_C_RADIUS_USER_STATUS*/
			read_csman(fd,0x00995000+index_range+i,&user_status,sizeof(unsigned int),TYPE_U32);
			/*UserExpireTime,CSID_C_RADIUS_USER_EXPIRE*/
			read_csman(fd,0x00995200+index_range+i,user_expiretime,sizeof(user_expiretime),TYPE_STR);	
			/*UserIdleTime,CSID_C_RADIUS_USER_IDLE_TIME*/
			read_csman(fd,0x00995400+index_range+i,user_idletime,sizeof(user_idletime),TYPE_STR);	
			/*UserIdleTime,CSID_C_RADIUS_LAST_LOGIN_TIME*/
			read_csman(fd,0x00995600+index_range+i,user_logintime,sizeof(user_logintime),TYPE_STR);	

			strncpy(new_user_name,user_name,strlen(user_name)-1);
			strncpy(new_user_password,user_password,strlen(user_password)-1);
			strncpy(new_user_leasetime,user_leasetime,strlen(user_leasetime)-1);
			strncpy(new_user_idletimeout,user_idletimeout,strlen(user_idletimeout)-1);
			strncpy(new_user_groupto,user_groupto,strlen(user_groupto)-1);
			strncpy(new_user_ip,user_ip,strlen(user_ip)-1);
			strncpy(new_user_expiretime,user_expiretime,strlen(user_expiretime)-1);
			strncpy(new_user_idletime,user_idletime,strlen(user_idletime)-1);
			strncpy(new_user_logintime,user_logintime,strlen(user_logintime)-1);

			sprintf(list_buf,"%s,%s,%d,%s,%s,%s,%d,%s,%d,%s,%s,%s;", \
				new_user_name,new_user_password,user_level,new_user_leasetime, \
				new_user_idletimeout,new_user_groupto,user_enable,new_user_ip, \
				user_status,new_user_expiretime,new_user_idletime,new_user_logintime);
			strcat(string,list_buf);
		}
		*var_len = strlen(string);
		return (unsigned char *) string;
    }
    default:
      ERROR_MSG("");
    }
    return NULL;
}


