#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIAdvFWMACControl.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_ADV_MACCONTROL	100
int w_index_adv_macctl=0;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIAdvFWMACControl_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,3,1,5 };
#else
oid UIAdvFWMACControl_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,3,1,5 };
#endif 

struct variable7 UIAdvFWMACControl_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{ADVANCEMACCONTROLENABLE,ASN_INTEGER,RWRITE,var_UIAdvFWMACControl, 1,  { 1 }},
{ADVANCEMACCONTROLBLACKWHITELIST,ASN_INTEGER,RWRITE,var_UIAdvFWMACControl, 1,  { 2 }},
{ADVANCEMACCONTROLLOGALTERENABLE,ASN_INTEGER,RWRITE,var_UIAdvFWMACControl, 1,  { 3 }},
{ADVANCEMACCONTROLRULEINDEX,ASN_INTEGER,RONLY,var_AdvanceMACControlTable, 3,  { 4 , 1, 1 }},
{ADVANCEMACCONTROLRULENAME,ASN_OCTET_STR,RWRITE,var_AdvanceMACControlTable, 3,  { 4 , 1, 2 }},
{ADVANCEMACCONTROLRULEMACADDRESS,ASN_OCTET_STR,RWRITE,var_AdvanceMACControlTable, 3,  { 4 , 1, 3 }},
{ADVANCEMACCONTROLRULETIMESCHEDULE,ASN_INTEGER,RWRITE,var_AdvanceMACControlTable, 3,  { 4 , 1, 4 }},
{ADVANCEMACCONTROLRULEENABLE,ASN_INTEGER,RWRITE,var_AdvanceMACControlTable, 3,  { 4 , 1, 5 }},
};

void
init_UIAdvFWMACControl(void)
{
    DEBUGMSGTL(("UIAdvFWMACControl", "Initializing\n"));
    REGISTER_MIB("UIAdvFWMACControl", UIAdvFWMACControl_variables, variable7, UIAdvFWMACControl_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif

	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}


unsigned char *
var_UIAdvFWMACControl(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case ADVANCEMACCONTROLENABLE:
        *write_method = write_AdvanceMACControlEnable;
        read_csman(fd,0x000d0001,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEMACCONTROLBLACKWHITELIST:
        *write_method = write_AdvanceMACControlBlackWhiteList;
        read_csman(fd,0x000d0003 ,&long_ret,sizeof(unsigned int),TYPE_U32);
	return (unsigned char *) &long_ret;
    case ADVANCEMACCONTROLLOGALTERENABLE:
        *write_method = write_AdvanceMACControlLogAlterEnable;
        read_csman(fd,0x000d0007 ,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_AdvanceMACControlTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UIAdvFWMACControl above.
 */
unsigned char *
var_AdvanceMACControlTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;

	register int index;
    index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_ADV_MACCONTROL);
    w_index_adv_macctl=index;

    if ( index == MATCH_FAILED )
        return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case ADVANCEMACCONTROLRULEINDEX:
        long_ret = index;
      	return (unsigned int *) &long_ret;
    case ADVANCEMACCONTROLRULENAME:
        *write_method = write_AdvanceMACControlRuleName;
        memset(string,0,sizeof(string));
        read_csman(fd,0x000d0700+index-1,string,sizeof(string),TYPE_STR);
        *var_len = strlen(string)-1;
        return (unsigned char *) string;
    case ADVANCEMACCONTROLRULEMACADDRESS:
        *write_method = write_AdvanceMACControlRuleMACAddress;
        memset(string,0,sizeof(string));
        read_csman(fd,0x000d0100+index-1,string,sizeof(string),TYPE_MAC);
        *var_len = 6;
        return (unsigned char *) string;
    case ADVANCEMACCONTROLRULETIMESCHEDULE:
        *write_method = write_AdvanceMACControlRuleTimeSchedule;
        read_csman(fd,0x000d0600+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
        return (unsigned char *) &long_ret;
    case ADVANCEMACCONTROLRULEENABLE:
        *write_method = write_AdvanceMACControlRuleEnable;
        read_csman(fd,0x000d0800+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
        return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}



int
write_AdvanceMACControlEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned long long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWMACControl not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWMACControl: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;
			
        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
		{
			int fd=0;
			write_csman(fd,0x000d0001,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			write_csman(fd,0x000d0002,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			write_csman(fd,0x000d0008,&long_ret,sizeof(unsigned int) ,TYPE_U32);
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvanceMACControlBlackWhiteList(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned long long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWMACControl not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWMACControl: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
		{
			int i,value=0,fd=0;
			write_csman(fd,0x000d0003,&long_ret,sizeof(unsigned int) ,TYPE_U32);
				 
			read_csman(fd,0x000d0003,&value,sizeof(unsigned int) ,TYPE_U32);
			for(i=0;i<=128;i++){
				write_csman(fd,0x000d0300+i,&value,sizeof(unsigned int) ,TYPE_U32);
			}
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvanceMACControlLogAlterEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned long long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWMACControl not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWMACControl: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
		{
			int fd=0;
			write_csman(fd,0x000d0007,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}

int
write_AdvanceMACControlRuleName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIAdvFWMACControl not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIAdvFWMACControl: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			//for STR/MAC data type
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
		{
			int fd=0;
			write_csman(fd,0x000d0700+w_index_adv_macctl-1,string,strlen(string) ,TYPE_STR);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvanceMACControlRuleMACAddress(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIAdvFWMACControl not \n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIAdvFWMACControl: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			//for STR/MAC data type
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          {
              int fd=0;
              write_csman(fd,0x000d0100+w_index_adv_macctl-1,string,strlen(string) ,TYPE_MAC);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvanceMACControlRuleTimeSchedule(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWMACControl not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWMACControl: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
            long_ret = *(unsigned int *) var_val;
            break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          {
              int fd=0;
              write_csman(fd,0x000d0600+w_index_adv_macctl-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvanceMACControlRuleEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWMACControl not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWMACControl: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
            long_ret = *(unsigned int *) var_val;
            break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
         {
              int fd=0;
              write_csman(fd,0x000d0800+w_index_adv_macctl-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
