#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIAdvFWPacketFilter.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_ADV_PKFILTER	64
int w_index_adv_pkfilter=0;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIAdvFWPacketFilter_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,3,1,2 };
#else
oid UIAdvFWPacketFilter_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,3,1,2 };
#endif

struct variable7 UIAdvFWPacketFilter_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{ADVANCEPKFILTERENABLE,ASN_INTEGER,RWRITE,var_UIAdvFWPacketFilter, 1,  { 1 }},
{ADVANCEPKFILTERBLACKWHITELIST,ASN_INTEGER,RWRITE,var_UIAdvFWPacketFilter, 1,  { 2 }},
{ADVANCEPKFILTERLOGALTERENABLE,ASN_INTEGER,RWRITE,var_UIAdvFWPacketFilter, 1,  { 3 }},

{ADVANCEPKFILTERRULEINDEX,ASN_INTEGER,RONLY,var_AdvancePKFilterTable, 3,  { 4 , 1, 1 }},
{ADVANCEPKFILTERRULENAME,ASN_OCTET_STR,RWRITE,var_AdvancePKFilterTable, 3,  { 4 , 1, 2 }},
{ADVANCEPKFILTERRULEFROMINTERFACETYPE,ASN_INTEGER,RWRITE,var_AdvancePKFilterTable, 3,  { 4 , 1, 3 }},
{ADVANCEPKFILTERRULEFROMINTERFACEINDEX,ASN_INTEGER,RWRITE,var_AdvancePKFilterTable, 3,  { 4 , 1, 4 }},
{ADVANCEPKFILTERRULETOINTERFACETYPE,ASN_INTEGER,RWRITE,var_AdvancePKFilterTable, 3,  { 4 , 1, 5 }},
{ADVANCEPKFILTERRULETOINTERFACEINDEX,ASN_INTEGER,RWRITE,var_AdvancePKFilterTable, 3,  { 4 , 1, 6 }},
{ADVANCEPKFILTERRULESOURCEIPTYPE,ASN_INTEGER,RWRITE,var_AdvancePKFilterTable, 3,  { 4 , 1, 7 }},
{ADVANCEPKFILTERRULESOURCEIPSPECIFICIP,  ASN_IPADDRESS,RWRITE,var_AdvancePKFilterTable, 3,  { 4 , 1, 8 }},
{ADVANCEPKFILTERRULESOURCEIPRANGE,ASN_OCTET_STR,RWRITE,var_AdvancePKFilterTable, 3,  { 4 , 1, 9 }},
{ADVANCEPKFILTERRULEDESTINATIONIPTYPE,ASN_INTEGER,RWRITE,var_AdvancePKFilterTable, 3,  { 4 , 1, 10 }},
{ADVANCEPKFILTERRULEDESTINATIONIPSPECIFICIP,ASN_IPADDRESS,RWRITE,var_AdvancePKFilterTable, 3,  { 4 , 1, 11 }},
{ADVANCEPKFILTERRULEDESTINATIONIPRANGE,ASN_OCTET_STR,RWRITE,var_AdvancePKFilterTable, 3,  { 4 , 1, 12 }},
{ADVANCEPKFILTERRULEDESTINATIONPORT,ASN_OCTET_STR,RWRITE,var_AdvancePKFilterTable, 3,  { 4 , 1, 13 }},
{ADVANCEPKFILTERRULEPROTOCOL,ASN_INTEGER,RWRITE,var_AdvancePKFilterTable, 3,  { 4 , 1, 14 }},
{ADVANCEPKFILTERRULETIMESCHEDULE,ASN_INTEGER,RWRITE,var_AdvancePKFilterTable, 3,  { 4 , 1, 15 }},
{ADVANCEPKFILTERRULEENABLE,ASN_INTEGER,RWRITE,var_AdvancePKFilterTable, 3,  { 4 , 1, 16 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UIAdvFWPacketFilter module */
void
init_UIAdvFWPacketFilter(void)
{
    DEBUGMSGTL(("UIAdvFWPacketFilter", "Initializing\n"));
    REGISTER_MIB("UIAdvFWPacketFilter", UIAdvFWPacketFilter_variables, variable7,UIAdvFWPacketFilter_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif

	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UIAdvFWPacketFilter(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case ADVANCEPKFILTERENABLE:
        *write_method = write_AdvancePKFilterEnable;
        read_csman(fd,0x000e0007,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPKFILTERBLACKWHITELIST:
        *write_method = write_AdvancePKFilterBlackWhiteList;
        read_csman(fd,0x000e0008,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPKFILTERLOGALTERENABLE:
        *write_method = write_AdvancePKFilterLogAlterEnable;
        read_csman(fd,0x000e000c,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_AdvancePKFilterTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UIAdvFWPacketFilter above.
 */
unsigned char *
var_AdvancePKFilterTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;

	register int index;
    index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_ADV_PKFILTER);
    w_index_adv_pkfilter=index;

    if ( index == MATCH_FAILED )
        return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case ADVANCEPKFILTERRULEINDEX:
        long_ret = index;
       	return (unsigned int *) &long_ret;
    case ADVANCEPKFILTERRULENAME:
        *write_method = write_AdvancePKFilterRuleName;
        memset(string,0,sizeof(string));
		read_csman(fd,0x000e7900+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case ADVANCEPKFILTERRULEFROMINTERFACETYPE:
        *write_method = write_AdvancePKFilterRuleFromInterfaceType;
        read_csman(fd,0x000e7A00+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPKFILTERRULEFROMINTERFACEINDEX:
        *write_method = write_AdvancePKFilterRuleFromInterfaceIndex;
        read_csman(fd,0x000e7e00+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPKFILTERRULETOINTERFACETYPE:
        *write_method = write_AdvancePKFilterRuleToInterfaceType;
        read_csman(fd,0x000e7B00+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPKFILTERRULETOINTERFACEINDEX:
        *write_method = write_AdvancePKFilterRuleToInterfaceIndex;
        read_csman(fd,0x000e7f00+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPKFILTERRULESOURCEIPTYPE:
        *write_method = write_AdvancePKFilterRuleSourceIPType;
        read_csman(fd,0x00202400+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPKFILTERRULESOURCEIPSPECIFICIP:
        *write_method = write_AdvancePKFilterRuleSourceIPSpecificIP;
        memset(string,0,sizeof(string));
		read_csman(fd,0x000e7100+index-1,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
    case ADVANCEPKFILTERRULESOURCEIPRANGE:
        *write_method = write_AdvancePKFilterRuleSourceIPRange;
        memset(string,0,sizeof(string));
		read_csman(fd,0x000e7200+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case ADVANCEPKFILTERRULEDESTINATIONIPTYPE:
        *write_method = write_AdvancePKFilterRuleDestinationIPType;
        read_csman(fd,0x00202440+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPKFILTERRULEDESTINATIONIPSPECIFICIP:
        *write_method = write_AdvancePKFilterRuleDestinationIPSpecificIP;
        memset(string,0,sizeof(string));
		read_csman(fd,0x000e7500+index-1,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
    case ADVANCEPKFILTERRULEDESTINATIONIPRANGE:
        *write_method = write_AdvancePKFilterRuleDestinationIPRange;
        memset(string,0,sizeof(string));
		read_csman(fd,0x000e7600+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case ADVANCEPKFILTERRULEDESTINATIONPORT:
        *write_method = write_AdvancePKFilterRuleDestinationPort;
        memset(string,0,sizeof(string));
		read_csman(fd,0x000e7700+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case ADVANCEPKFILTERRULEPROTOCOL:
        *write_method = write_AdvancePKFilterRuleProtocol;
        read_csman(fd, 0x000e7400+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPKFILTERRULETIMESCHEDULE:
        *write_method = write_AdvancePKFilterRuleTimeSchedule;
        read_csman(fd, 0x000e7800+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPKFILTERRULEENABLE:
        *write_method = write_AdvancePKFilterRuleEnable;
        read_csman(fd, 0x000e7000+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}

int
write_AdvancePKFilterEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned long long_ret;
	
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWPacketFilter not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWPacketFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x000e0007,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvancePKFilterBlackWhiteList(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned long long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWPacketFilter not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWPacketFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
            int fd=0;
            write_csman(fd,0x000e0008,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvancePKFilterLogAlterEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned long long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWPacketFilter not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWPacketFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
            int fd=0;
            write_csman(fd,0x000e000c,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}

int
write_AdvancePKFilterRuleName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIAdvFWPacketFilter not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIAdvFWPacketFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			//for STR/MAC data type
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x000e7900+w_index_adv_pkfilter-1,string,strlen(string) ,TYPE_STR);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePKFilterRuleFromInterfaceType(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned long long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWPacketFilter not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWPacketFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x000e7A00+w_index_adv_pkfilter-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePKFilterRuleFromInterfaceIndex(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned long long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWPacketFilter not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWPacketFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x000e7e00+w_index_adv_pkfilter-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePKFilterRuleToInterfaceType(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned long long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWPacketFilter not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWPacketFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x000e7B00+w_index_adv_pkfilter-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePKFilterRuleToInterfaceIndex(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned long long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWPacketFilter not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWPacketFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x000e7f00+w_index_adv_pkfilter-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePKFilterRuleSourceIPType(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned long long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWPacketFilter not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWPacketFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			unsigned long csid_address;
			char cmd_buf[256];
			write_csman(fd,0x00202400+w_index_adv_pkfilter-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			if(long_ret == 0){
				/*Disclean IP Range 0x000e7200*/
				csid_address = 946688+w_index_adv_pkfilter-1;
				sprintf(cmd_buf,"rmcsman 0x%x",csid_address);
				system(cmd_buf);
			}
			else if(long_ret == 1){
				/*Disclean IP Range 0x000e7100*/
				csid_address = 946432+w_index_adv_pkfilter-1;
				sprintf(cmd_buf,"rmcsman 0x%x",csid_address);
				system(cmd_buf);
			}	
			break;	
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePKFilterRuleSourceIPSpecificIP(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIAdvFWPacketFilter not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIAdvFWPacketFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			memcpy(string , (char *) var_val, 4);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x000e7100+w_index_adv_pkfilter-1,string,strlen(string) ,TYPE_IPV4);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePKFilterRuleSourceIPRange(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIAdvFWPacketFilter not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UIAdvFWPacketFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			//for STR/MAC data type
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x000e7200+w_index_adv_pkfilter-1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePKFilterRuleDestinationIPType(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned long long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWPacketFilter not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWPacketFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			unsigned long csid_address;
			char cmd_buf[256];
			write_csman(fd,0x00202440+w_index_adv_pkfilter-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			if(long_ret == 0){
				/*Disclean IP Range 0x000e7500*/
				csid_address = 947456+w_index_adv_pkfilter-1;
				sprintf(cmd_buf,"rmcsman 0x%x",csid_address);
				system(cmd_buf);
			}
			else if(long_ret == 1){
				/*Disclean IP Range 0x000e7600*/
				csid_address = 947712+w_index_adv_pkfilter-1;
				sprintf(cmd_buf,"rmcsman 0x%x",csid_address);
				system(cmd_buf);
			}	
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePKFilterRuleDestinationIPSpecificIP(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIAdvFWPacketFilter not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIAdvFWPacketFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			//for IPv4 data type
			memset(string,0,sizeof(string));
			memcpy(string , (char *) var_val, 4);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x000e7500+w_index_adv_pkfilter-1,string,strlen(string) ,TYPE_IPV4);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePKFilterRuleDestinationIPRange(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIAdvFWPacketFilter not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UIAdvFWPacketFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			//for STR/MAC data type
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x000e7600+w_index_adv_pkfilter-1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePKFilterRuleDestinationPort(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIAdvFWPacketFilter not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIAdvFWPacketFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x000e7700+w_index_adv_pkfilter-1,string,strlen(string)+1,TYPE_STR);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePKFilterRuleProtocol(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned long long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWPacketFilter not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWPacketFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x000e7400+w_index_adv_pkfilter-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePKFilterRuleTimeSchedule(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned long long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWPacketFilter not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWPacketFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x000e7800+w_index_adv_pkfilter-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePKFilterRuleEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
   	static unsigned long long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWPacketFilter not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWPacketFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x000e7000+w_index_adv_pkfilter-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
