#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIAdvFWWebContentFilter.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_ADV_WCF	20
int w_index_adv_wcf=0;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIAdvFWWebContentFilter_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,3,1,4 };
#else
oid UIAdvFWWebContentFilter_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,3,1,4 };
#endif

struct variable7 UIAdvFWWebContentFilter_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{ADVANCEWEBCONTENTFILTERSENABLE,ASN_INTEGER,RWRITE,var_UIAdvFWWebContentFilter, 1,  { 1 }},
{ADVANCEWEBCONTENTFILTERPFELCOOKIEENABLE,ASN_INTEGER,RWRITE,var_UIAdvFWWebContentFilter, 1,  { 2 }},
{ADVANCEWEBCONTENTFILTERPFELJAVAENABLE,ASN_INTEGER,RWRITE,var_UIAdvFWWebContentFilter, 1,  { 3 }},
{ADVANCEWEBCONTENTFILTERPFELACTIVEXENABLE,ASN_INTEGER,RWRITE,var_UIAdvFWWebContentFilter, 1,  { 4 }},
{ADVANCEWEBCONTENTFILTERLOGALTERENABLE,ASN_INTEGER,RWRITE,var_UIAdvFWWebContentFilter, 1,  { 5 }},
{ADVANCEWEBCONTENTFILTERRULEINDEX,ASN_INTEGER,RONLY,var_AdvanceWebContentTable, 3,  { 6 , 1, 1 }},
{ADVANCEWEBCONTENTFILTERRULENAME,ASN_OCTET_STR,RWRITE,var_AdvanceWebContentTable, 3,  { 6 , 1, 2 }},
{ADVANCEWEBCONTENTFILTERRULEFILEEXTENSIONLIST,ASN_OCTET_STR,RWRITE,var_AdvanceWebContentTable, 3,  { 6 , 1, 3 }},
{ADVANCEWEBCONTENTFILTERRULETIMESCHEDULE,ASN_INTEGER,RWRITE,var_AdvanceWebContentTable, 3,  { 6 , 1, 4 }},
{ADVANCEWEBCONTENTFILTERRULEENABLE,ASN_INTEGER,RWRITE,var_AdvanceWebContentTable, 3,  { 6 , 1, 5 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UIAdvFWWebContentFilter module */
void
init_UIAdvFWWebContentFilter(void)
{
    DEBUGMSGTL(("UIAdvFWWebContentFilter", "Initializing\n"));
    REGISTER_MIB("UIAdvFWWebContentFilter", UIAdvFWWebContentFilter_variables, variable7,UIAdvFWWebContentFilter_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}


unsigned char *
var_UIAdvFWWebContentFilter(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case ADVANCEWEBCONTENTFILTERSENABLE:
        *write_method = write_AdvanceWebContentFiltersEnable;
        read_csman(fd,0x00100010,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEWEBCONTENTFILTERPFELCOOKIEENABLE:
        *write_method = write_AdvanceWebContentFilterPFELCookieEnable;
        read_csman(fd,0x00100013,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEWEBCONTENTFILTERPFELJAVAENABLE:
        *write_method = write_AdvanceWebContentFilterPFELJavaEnable;
        read_csman(fd,0x00100011,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEWEBCONTENTFILTERPFELACTIVEXENABLE:
        *write_method = write_AdvanceWebContentFilterPFELActiveXEnable;
        read_csman(fd,0x00100012 ,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEWEBCONTENTFILTERLOGALTERENABLE:
        *write_method = write_AdvanceWebContentFilterLogAlterEnable;
        read_csman(fd,0x00100018 ,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_AdvWebContentTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UIAdvFWWebContentFilter above.
 */
unsigned char *
var_AdvanceWebContentTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
   
    /* get index */
    register int index;
    index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_ADV_WCF);
    w_index_adv_wcf=index;

    if ( index == MATCH_FAILED )
        return NULL;
   

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case ADVANCEWEBCONTENTFILTERRULEINDEX:
		long_ret = index;
		return (unsigned int *) &long_ret;
    case ADVANCEWEBCONTENTFILTERRULENAME:
        *write_method = write_AdvanceWebContentFilterRuleName;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00100600+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case ADVANCEWEBCONTENTFILTERRULEFILEEXTENSIONLIST:
        *write_method = write_AdvanceWebContentFilterRuleFileExtensionList;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00100300+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case ADVANCEWEBCONTENTFILTERRULETIMESCHEDULE:
        *write_method = write_AdvanceWebContentFilterRuleTimeSchedule;
        read_csman(fd, 0x00100400+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEWEBCONTENTFILTERRULEENABLE:
        *write_method = write_AdvanceWebContentFilterRuleEnable;
        read_csman(fd, 0x00100500+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}



int
write_AdvanceWebContentFiltersEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned long long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWWebContentFilter not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWWebContentFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
           int fd=0;
           write_csman(fd,0x00100010,&long_ret,sizeof(unsigned int) ,TYPE_U32);
		   break;
        }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvanceWebContentFilterPFELCookieEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned long long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWWebContentFilter not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWWebContentFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
            int fd=0;
            write_csman(fd,0x00100013,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvanceWebContentFilterPFELJavaEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned long long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWWebContentFilter not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWWebContentFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00100011,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvanceWebContentFilterPFELActiveXEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned long long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWWebContentFilter not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWWebContentFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
           int fd=0;
           write_csman(fd,0x00100012,&long_ret,sizeof(unsigned int) ,TYPE_U32);
		   break;
        }

    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvanceWebContentFilterLogAlterEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned long long_ret;
	
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWWebContentFilter not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWWebContentFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = * (unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
           int fd=0;
           write_csman(fd,0x00100018,&long_ret,sizeof(unsigned int) ,TYPE_U32);
		   break;
        }

    }
    return SNMP_ERR_NOERROR;
}

int
write_AdvanceWebContentFilterRuleName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIAdvFWWebContentFilter not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIAdvFWWebContentFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			//for STR/MAC data type
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00100600+w_index_adv_wcf-1,string,strlen(string)+1,TYPE_STR);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvanceWebContentFilterRuleFileExtensionList(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIAdvFWWebContentFilter not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIAdvFWWebContentFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			//for STR/MAC data type
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00100300+w_index_adv_wcf-1,string,strlen(string)+1,TYPE_STR);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvanceWebContentFilterRuleTimeSchedule(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWWebContentFilter not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWWebContentFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          	long_ret = *(unsigned int *) var_val;
          	break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00100500+w_index_adv_wcf-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvanceWebContentFilterRuleEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvFWWebContentFilter not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvFWWebContentFilter: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          	long_ret = *(unsigned int *) var_val;
          	break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00100400+w_index_adv_wcf-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
