#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIAdvQoSConfiguration.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_ADV_QOS_CONF	5
int w_index_adv_qos_conf;


#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIAdvQoSConfiguration_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,3,2,1 };
#else
oid UIAdvQoSConfiguration_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,3,2,1 };
#endif

struct variable7 UIAdvQoSConfiguration_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{ADVQOSCONFTOTALPRIOTIYQUEUES,ASN_INTEGER,RWRITE,var_UIAdvQoSConfiguration, 1,  { 1 }},

{ADVQOSCONFWANINDEX,ASN_INTEGER,RONLY,var_AdvQoSConfTable, 3,  { 2 , 1, 1 }},
{ADVQOSCONFWANINTERFACENAME,ASN_OCTET_STR,RONLY,var_AdvQoSConfTable, 3,  { 2 , 1, 2 }},
{ADVQOSCONFWANBWOFUPSTREAM,ASN_INTEGER, RWRITE,var_AdvQoSConfTable, 3,  { 2 , 1, 3 }},
{ADVQOSCONFWANBWOFDOWNSTREAM,ASN_INTEGER, RWRITE,var_AdvQoSConfTable, 3,  { 2 , 1, 4 }},
{ADVQOSCONFWANTOTALCONNECTIONSESSIONS,ASN_INTEGER,RWRITE,var_AdvQoSConfTable, 3,  { 2 , 1, 5 }},
{ADVQOSCONFWANFLEXIBLEBWMANAGEENABLE,ASN_INTEGER,RWRITE,var_AdvQoSConfTable, 3,  { 2 , 1, 6 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UIAdvQoSConfiguration module */
void
init_UIAdvQoSConfiguration(void)
{
    DEBUGMSGTL(("UIAdvQoSConfiguration", "Initializing\n"));
    REGISTER_MIB("UIAdvQoSConfiguration", UIAdvQoSConfiguration_variables, variable7,
               UIAdvQoSConfiguration_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UIAdvQoSConfiguration(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case ADVQOSCONFTOTALPRIOTIYQUEUES:
        *write_method = write_AdvQoSConfTotalPriotiyQueues;
        read_csman(fd,0x00120060,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_AdvQoSConfTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UIAdvQoSConfiguration above.
 */
unsigned char *
var_AdvQoSConfTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
 	
    /* get index */
    register int index;
    index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_ADV_QOS_CONF);
    w_index_adv_qos_conf=index;

    if ( index == MATCH_FAILED )
        return NULL;


    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case ADVQOSCONFWANINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case ADVQOSCONFWANINTERFACENAME:
	{
        int wan_index=0;
		memset(string,0,sizeof(string));
		read_csman(fd,0x00010600+index-1,&wan_index,sizeof(unsigned int),TYPE_U32);
		if(wan_index != 0){
			sprintf(string,"WAN-%d ",index);
		}
		else	strcpy(string," ");
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
   	}
    case ADVQOSCONFWANBWOFUPSTREAM:
        *write_method = write_AdvQoSConfWANBWofUpStream;
        read_csman(fd,0x00120020+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVQOSCONFWANBWOFDOWNSTREAM:
        *write_method = write_AdvQoSConfWANBWofDownStream;
        read_csman(fd,0x00120030+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVQOSCONFWANTOTALCONNECTIONSESSIONS:
        *write_method = write_AdvQoSConfWANTotalConnectionSessions;
        read_csman(fd,0x00120040+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVQOSCONFWANFLEXIBLEBWMANAGEENABLE:
        *write_method = write_AdvQoSConfWANFlexibleBWManageEnable;
        read_csman(fd,0x00120015,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}



int
write_AdvQoSConfTotalPriotiyQueues(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvQoSConfiguration not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvQoSConfiguration: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          	long_ret = *(unsigned int *) var_val;
          	break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00120060,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}

int
write_AdvQoSConfWANBWofUpStream(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvQoSConfiguration not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvQoSConfiguration: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          	long_ret = *(unsigned int *) var_val;
          	break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00120020+w_index_adv_qos_conf-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvQoSConfWANBWofDownStream(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvQoSConfiguration not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvQoSConfiguration: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          	long_ret = *(unsigned int *) var_val;
          	break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00120030+w_index_adv_qos_conf-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvQoSConfWANTotalConnectionSessions(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvQoSConfiguration not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvQoSConfiguration: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          	long_ret = *(unsigned int *) var_val;
          	break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00120040+w_index_adv_qos_conf-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvQoSConfWANFlexibleBWManageEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvQoSConfiguration not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvQoSConfiguration: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          	long_ret = *(unsigned int *) var_val;
          	break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00120015,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
