#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIAdvSystemVRRPConfiguration.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIAdvSystemVRRPConfiguration_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,3,4,1 };
#else
oid UIAdvSystemVRRPConfiguration_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,3,4,1 };
#endif

struct variable7 UIAdvSystemVRRPConfiguration_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{ADVANCEVRRPENABLE,ASN_INTEGER,RWRITE,var_UIAdvSystemVRRPConfiguration, 1,  { 1 }},
{ADVANCEVRRPVIRTUALSERVERID,ASN_INTEGER,RWRITE,var_UIAdvSystemVRRPConfiguration, 1,  { 2 }},
{ADVANCEVRRPPRIORITYOFVIRTUALSERVER,ASN_INTEGER,RWRITE,var_UIAdvSystemVRRPConfiguration, 1,  { 3 }},
{ADVANCEVRRPVIRTUALSERVERIPADDRESS,ASN_IPADDRESS,RWRITE, var_UIAdvSystemVRRPConfiguration, 1,  { 4 }},

};
/*    (L = length of the oidsuffix) */


/** Initializes the UIAdvSystemVRRPConfiguration module */
void
init_UIAdvSystemVRRPConfiguration(void)
{
    DEBUGMSGTL(("UIAdvSystemVRRPConfiguration", "Initializing\n"));
    REGISTER_MIB("UIAdvSystemVRRPConfiguration", UIAdvSystemVRRPConfiguration_variables, variable7,
               UIAdvSystemVRRPConfiguration_variables_oid);
}

unsigned char *
var_UIAdvSystemVRRPConfiguration(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case ADVANCEVRRPENABLE:
        *write_method = write_AdvanceVRRPEnable;
        read_csman(fd,0x003a0001,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
    case ADVANCEVRRPVIRTUALSERVERID:
        *write_method = write_AdvanceVRRPVirtualServerID;
        read_csman(fd,0x003a0010,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
    case ADVANCEVRRPPRIORITYOFVIRTUALSERVER:
        *write_method = write_AdvanceVRRPPriorityofVirtualServer;
        read_csman(fd,0x003a0011,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
	case ADVANCEVRRPVIRTUALSERVERIPADDRESS:
		*write_method = write_AdvanceVRRPVirtualServerIPAddress;
		memset(string,0,sizeof(string));
		read_csman(fd,0x003a0020,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
		
    default:
      ERROR_MSG("");
    }
    return NULL;
}

int
write_AdvanceVRRPEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvSystemVRRPConfiguration not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvSystemVRRPConfiguration: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x003a0001,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvanceVRRPVirtualServerID(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvSystemVRRPConfiguration not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvSystemVRRPConfiguration: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          {
              int fd=0;
              write_csman(fd,0x003a0010,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvanceVRRPPriorityofVirtualServer(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvSystemVRRPConfiguration not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvSystemVRRPConfiguration: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          {
              int fd=0;
              write_csman(fd,0x003a0011,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvanceVRRPVirtualServerIPAddress(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to VRRP not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to VRRP: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  //for IPv4 data type
			  memset(string,0,sizeof(string));
			  memcpy(string , (char *) var_val, 4);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
		  {
			  int fd;
			  write_csman(fd,0x003a0020,string,strlen(string) ,TYPE_IPV4);
			  break;
		  }
    }
    return SNMP_ERR_NOERROR;
}

