#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIAdvVPNConfiguration.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#include "UIAdvVPNL2TPServer.h"

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIAdvVPNL2TPServer_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,3,3,9 };
#else
oid UIAdvVPNL2TPServer_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,3,3,9 };
#endif

struct variable7 UIAdvVPNL2TPServer_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{ADVANCEL2TPSENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNL2TPServer, 1,  { 1 }},
{ADVANCEL2TPSERVENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNL2TPServer, 1,  { 2 }},
{ADVANCEL2TPSERVOVERIPSECENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNL2TPServer, 1,  { 3 }},
{ADVANCEL2TPSERVPRESHAREKEY,ASN_OCTET_STR,RWRITE,var_UIAdvVPNL2TPServer, 1,  { 4 }},
{ADVANCEL2TPSERVVIRTUALIP,ASN_IPADDRESS,RWRITE,var_UIAdvVPNL2TPServer, 1,  { 5 }},
{ADVANCEL2TPSERVIPPOOLSTARTINGADDRESS,ASN_IPADDRESS,RWRITE,var_UIAdvVPNL2TPServer, 1,  { 6 }},
{ADVANCEL2TPSERVIPPOOLENDINGADDRESS,ASN_IPADDRESS,RWRITE,var_UIAdvVPNL2TPServer, 1,  { 7 }},
{ADVANCEL2TPSERVAUTHPROTOPAPENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNL2TPServer, 1,  { 8 }},
{ADVANCEL2TPSERVAUTHPROTOCHAPENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNL2TPServer, 1,  { 9 }},
{ADVANCEL2TPSERVAUTHPROTOMSCHAPENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNL2TPServer, 1,  { 10 }},
{ADVANCEL2TPSERVAUTHPROTOMSCHAPV2ENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNL2TPServer, 1,  { 11 }},
{ADVANCEL2TPSERVMPPEENCRYPTIONENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNL2TPServer, 1,  { 12 }},
{ADVANCEL2TPSERV40BITSENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNL2TPServer, 1,  { 13 }},
{ADVANCEL2TPSERV56BITSENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNL2TPServer, 1,  { 14 }},
{ADVANCEL2TPSERV128BITSENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNL2TPServer, 1,  { 15 }},
{ADVANCEL2TPSERVSERVICEPORT,ASN_INTEGER,RWRITE,var_UIAdvVPNL2TPServer, 1,  { 16 }},

};
/*    (L = length of the oidsuffix) */

void
init_UIAdvVPNL2TPServer(void)
{
    DEBUGMSGTL(("UIAdvVPNL2TPServer", "Initializing\n"));
    REGISTER_MIB("UIAdvVPNL2TPServer", UIAdvVPNL2TPServer_variables, variable7,
               UIAdvVPNL2TPServer_variables_oid);
}

unsigned char *
var_UIAdvVPNL2TPServer(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case ADVANCEL2TPSENABLE:
        *write_method = write_AdvanceL2TPSEnable;
        read_csman(fd,0x00201010,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEL2TPSERVENABLE:
        *write_method = write_AdvanceL2TPServEnable;
        read_csman(fd,0x001b0001,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEL2TPSERVOVERIPSECENABLE:
        *write_method = write_AdvanceL2TPServoverIPSecEnable;
        read_csman(fd,0x0070a600,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEL2TPSERVPRESHAREKEY:
        *write_method = write_AdvanceL2TPServPreshareKey;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00705300,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case ADVANCEL2TPSERVVIRTUALIP:
        *write_method = write_AdvanceL2TPServVirtualIP;
        memset(string,0,sizeof(string));
		read_csman(fd,0x001b0008,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
    case ADVANCEL2TPSERVIPPOOLSTARTINGADDRESS:
        *write_method = write_AdvanceL2TPServIPPoolStartingAddress;
        memset(string,0,sizeof(string));
		read_csman(fd,0x001b0009,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
    case ADVANCEL2TPSERVIPPOOLENDINGADDRESS:
        *write_method = write_AdvanceL2TPServIPPoolEndingAddress;
        memset(string,0,sizeof(string));
		read_csman(fd,0x001b000a,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
    case ADVANCEL2TPSERVAUTHPROTOPAPENABLE:
        *write_method = write_AdvanceL2TPServAuthProtoPAPEnable;
        read_csman(fd,0x001b0003,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEL2TPSERVAUTHPROTOCHAPENABLE:
        *write_method = write_AdvanceL2TPServAuthProtoCHAPEnable;
        read_csman(fd,0x001b0004,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEL2TPSERVAUTHPROTOMSCHAPENABLE:
        *write_method = write_AdvanceL2TPServAuthProtoMSCHAPEnable;
        read_csman(fd,0x001b0005,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEL2TPSERVAUTHPROTOMSCHAPV2ENABLE:
        *write_method = write_AdvanceL2TPServAuthProtoMSCHAPv2Enable;
        read_csman(fd,0x001b0006,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEL2TPSERVMPPEENCRYPTIONENABLE:
        *write_method = write_AdvanceL2TPServMPPEEncryptionEnable;
        read_csman(fd,0x001b0010,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEL2TPSERV40BITSENABLE:
        *write_method = write_AdvanceL2TPServ40bitsEnable;
        read_csman(fd,0x001b0011,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEL2TPSERV56BITSENABLE:
        *write_method = write_AdvanceL2TPServ56bitsEnable;
        read_csman(fd,0x001b0012,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEL2TPSERV128BITSENABLE:
        *write_method = write_AdvanceL2TPServ128bitsEnable;
        read_csman(fd,0x001b0013,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
	case ADVANCEL2TPSERVSERVICEPORT:
		*write_method = write_AdvanceL2TPServServicePort;
        read_csman(fd,0x001b001a,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}

int
write_AdvanceL2TPSEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to AdvVpnL2TPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to AdvVpnL2TPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          {
              int fd=0;
              write_csman(fd,0x00201010,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvanceL2TPServEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to AdvVpnL2TPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to AdvVpnL2TPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          {
              int fd=0;
              write_csman(fd,0x001b0001,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}



int
write_AdvanceL2TPServoverIPSecEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to AdvVpnL2TPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to AdvVpnL2TPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          {
              int fd=0;
              write_csman(fd,0x0070a600,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}



int
write_AdvanceL2TPServPreshareKey(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to AdvVpnL2TPServer not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to AdvVpnL2TPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  //for STR/MAC data type
			  memset(string,0,sizeof(string));
			  strncpy(string , (char *) var_val, var_val_len);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
         {
             int fd;
             write_csman(fd,0x00705300,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}

int
write_AdvanceL2TPServVirtualIP(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to AdvVpnL2TPServer not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to AdvVpnL2TPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  //for IPv4 data type
			  memset(string,0,sizeof(string));
			  memcpy(string , (char *) var_val, 4);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
		  {
			  int fd;
			  write_csman(fd,0x001b0008,string,strlen(string) ,TYPE_IPV4);
			  break;
		  }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvanceL2TPServIPPoolStartingAddress(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to AdvVpnL2TPServer not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to AdvVpnL2TPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  //for IPv4 data type
			  memset(string,0,sizeof(string));
			  memcpy(string , (char *) var_val, 4);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
		  {
			  int fd;
			  write_csman(fd,0x001b0009,string,strlen(string) ,TYPE_IPV4);
			  break;
		  }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvanceL2TPServIPPoolEndingAddress(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to AdvVpnL2TPServer not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to AdvVpnL2TPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  //for IPv4 data type
			  memset(string,0,sizeof(string));
			  memcpy(string , (char *) var_val, 4);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
		  {
			  int fd;
			  write_csman(fd,0x001b000a,string,strlen(string) ,TYPE_IPV4);
			  break;
		  }
    }
    return SNMP_ERR_NOERROR;
}



int
write_AdvanceL2TPServAuthProtoPAPEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to AdvVpnL2TPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to AdvVpnL2TPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          {
              int fd=0;
              write_csman(fd,0x001b0003,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvanceL2TPServAuthProtoCHAPEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to AdvVpnL2TPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to AdvVpnL2TPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          {
              int fd=0;
              write_csman(fd,0x001b0004,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvanceL2TPServAuthProtoMSCHAPEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to AdvVpnL2TPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to AdvVpnL2TPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          {
              int fd=0;
              write_csman(fd,0x001b0005,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}

int
write_AdvanceL2TPServAuthProtoMSCHAPv2Enable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to AdvVpnL2TPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to AdvVpnL2TPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          {
              int fd=0;
              write_csman(fd,0x001b0006,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}

int
write_AdvanceL2TPServMPPEEncryptionEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to AdvVpnL2TPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to AdvVpnL2TPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          {
              int fd=0;
              write_csman(fd,0x001b0010,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}



int
write_AdvanceL2TPServ40bitsEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to AdvVpnL2TPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to AdvVpnL2TPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          {
              int fd=0,disable=0;
              write_csman(fd,0x001b0011,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			  if(long_ret == 1){
			  	 write_csman(fd,0x001b0012,&disable,sizeof(unsigned int) ,TYPE_U32);
				 write_csman(fd,0x001b0013,&disable,sizeof(unsigned int) ,TYPE_U32);
			  }	
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}

int
write_AdvanceL2TPServ56bitsEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to AdvVpnL2TPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to AdvVpnL2TPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          {
              int fd=0,disable=0;
              write_csman(fd,0x001b0012,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			  if(long_ret == 1){
			  	 write_csman(fd,0x001b0011,&disable,sizeof(unsigned int) ,TYPE_U32);
				 write_csman(fd,0x001b0013,&disable,sizeof(unsigned int) ,TYPE_U32);
			  }	
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvanceL2TPServ128bitsEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to AdvVpnL2TPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to AdvVpnL2TPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          {
              int fd=0,disable=0;
              write_csman(fd,0x001b0013,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			  if(long_ret == 1){
			  	 write_csman(fd,0x001b0011,&disable,sizeof(unsigned int) ,TYPE_U32);
				 write_csman(fd,0x001b0012,&disable,sizeof(unsigned int) ,TYPE_U32);
			  }	
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}

int
write_AdvanceL2TPServServicePort(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to AdvVpnL2TPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to AdvVpnL2TPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          {
              int fd=0;
              write_csman(fd,0x001b001a,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}

