#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UIAdvVPNPPTPClient.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_ADV_VPN_PPTP_CLIENT	10
int w_index_adv_vpn_pptp_client=0;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIAdvVPNPPTPClient_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,3,3,7 };
#else
oid UIAdvVPNPPTPClient_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,3,3,7 };
#endif

struct variable7 UIAdvVPNPPTPClient_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{ADVANCEPPTPCENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNPPTPClient, 1,  { 1 }},
{ADVANCEPPTPCLIENTALTER,ASN_INTEGER,RWRITE,var_UIAdvVPNPPTPClient, 1,  { 2 }},
{ADVANCEPPTPCLIENTENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNPPTPClient, 1,  { 3 }},
{ADVANCEPPTPCLIENTINDEX,ASN_INTEGER,RONLY,var_AdvancePPTPClientTable, 3,  { 4 , 1, 1 }},
{ADVANCEPPTPCLIENTNAME,ASN_OCTET_STR,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 2 }},
{ADVANCEPPTPCLIENTOPERATIONMODE,ASN_INTEGER,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 3 }},
{ADVANCEPPTPCLIENTOPERATIONMODEFAILOVERTYPE,ASN_INTEGER,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 4 }},
{ADVANCEPPTPCLIENTOPERATIONMODEFAILOVERTUNNEL,ASN_INTEGER,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 5 }},
{ADVANCEPPTPCLIENTREMOTEIPFQDN,ASN_OCTET_STR,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 6 }},
{ADVANCEPPTPCLIENTUSERNAME,ASN_OCTET_STR,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 7 }},
{ADVANCEPPTPCLIENTPASSWORD,ASN_OCTET_STR,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 8 }},
{ADVANCEPPTPCLIENTDEFGATEWAY,ASN_OCTET_STR,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 9 }},
{ADVANCEPPTPCLIENTREMOTESUBNET,ASN_OCTET_STR,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 10 }},
{ADVANCEPPTPCLIENTAUTHENABLE,ASN_INTEGER,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 11 }},
{ADVANCEPPTPCLIENTAUTHPAPENABLE,ASN_INTEGER,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 12 }},
{ADVANCEPPTPCLIENTAUTHCHAPENABLE,ASN_INTEGER,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 13 }},
{ADVANCEPPTPCLIENTAUTHMSCHAPENABLE,ASN_INTEGER,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 14 }},
{ADVANCEPPTPCLIENTAUTHMSCHAPV2ENABLE,ASN_INTEGER,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 15 }},
{ADVANCEPPTPCLIENTMPPEENCRYPTIONENABLE,ASN_INTEGER,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 16 }},
{ADVANCEPPTPCLIENTNATBEFORETUNNELINGENABLE,ASN_INTEGER,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 17 }},
{ADVANCEPPTPCLIENTLCPECHOTYPE,ASN_INTEGER,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 18 }},
{ADVANCEPPTPCLIENTLCPECHOINTERVAL,ASN_INTEGER,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 19 }},
{ADVANCEPPTPCLIENTLCPECHOMAXFAILURETIME,ASN_INTEGER,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 20 }},
{ADVANCEPPTPCLIENTTUNNELENABLE,ASN_INTEGER,RWRITE,var_AdvancePPTPClientTable, 3,  { 4 , 1, 21 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UIAdvVPNPPTPClient module */
void
init_UIAdvVPNPPTPClient(void)
{
    DEBUGMSGTL(("UIAdvVPNPPTPClient", "Initializing\n"));
    REGISTER_MIB("UIAdvVPNPPTPClient", UIAdvVPNPPTPClient_variables, variable7,
               UIAdvVPNPPTPClient_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	/* in this case, oid len is 17*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}


unsigned char *
var_UIAdvVPNPPTPClient(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case ADVANCEPPTPCENABLE:
        *write_method = write_AdvancePPTPCEnable;
        read_csman(fd,0x0020100f,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPCLIENTALTER:
        *write_method = write_AdvancePPTPClientAlter;
        read_csman(fd,0x80051001,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPCLIENTENABLE:
        *write_method = write_AdvancePPTPClientEnable;
        read_csman(fd,0x00051000,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_AdvancePPTPClientTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UIAdvVPNPPTPClient above.
 */
unsigned char *
var_AdvancePPTPClientTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;

   	int fd=0;
    /* get index */
    register int index;
    index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_ADV_VPN_PPTP_CLIENT);
    w_index_adv_vpn_pptp_client=index;

    if ( index == MATCH_FAILED )
        return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case ADVANCEPPTPCLIENTINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case ADVANCEPPTPCLIENTNAME:
        *write_method = write_AdvancePPTPClientName;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00051200+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case ADVANCEPPTPCLIENTOPERATIONMODE:
        *write_method = write_AdvancePPTPClientOperationMode;
        read_csman(fd,0x00052700+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPCLIENTOPERATIONMODEFAILOVERTYPE:
        *write_method = write_AdvancePPTPClientOperationModeFailoverType;
        read_csman(fd,0x00052800+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPCLIENTOPERATIONMODEFAILOVERTUNNEL:
        *write_method = write_AdvancePPTPClientOperationModeFailoverTunnel;
        read_csman(fd,0x00052900+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPCLIENTREMOTEIPFQDN:
        *write_method = write_AdvancePPTPClientRemoteIPFQDN;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00051b00+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case ADVANCEPPTPCLIENTUSERNAME:
        *write_method = write_AdvancePPTPClientUserName;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00051400+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case ADVANCEPPTPCLIENTPASSWORD:
        *write_method = write_AdvancePPTPClientPassword;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00051500+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case ADVANCEPPTPCLIENTDEFGATEWAY:
        *write_method = write_AdvancePPTPClientDefGateway;
        memset(string,0,sizeof(string));
		strcpy(string,"0.0.0.0/0 ");
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case ADVANCEPPTPCLIENTREMOTESUBNET:
        *write_method = write_AdvancePPTPClientRemoteSubnet;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00051600+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case ADVANCEPPTPCLIENTAUTHENABLE:
        *write_method = write_AdvancePPTPClientAuthEnable;
        read_csman(fd,0x00051c00+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPCLIENTAUTHPAPENABLE:
        *write_method = write_AdvancePPTPClientAuthPAPEnable;
        read_csman(fd,0x00051d00+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPCLIENTAUTHCHAPENABLE:
        *write_method = write_AdvancePPTPClientAuthCHAPEnable;
        read_csman(fd,0x00051e00+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPCLIENTAUTHMSCHAPENABLE:
        *write_method = write_AdvancePPTPClientAuthMSCHAPEnable;
        read_csman(fd,0x00051f00+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPCLIENTAUTHMSCHAPV2ENABLE:
        *write_method = write_AdvancePPTPClientAuthMSCHAPv2Enable;
        read_csman(fd,0x00052000+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPCLIENTMPPEENCRYPTIONENABLE:
        *write_method = write_AdvancePPTPClientMPPEEncryptionEnable;
        read_csman(fd,0x00051800+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPCLIENTNATBEFORETUNNELINGENABLE:
        *write_method = write_AdvancePPTPClientNATbeforeTunnelingEnable;
        read_csman(fd,0x00051900+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPCLIENTLCPECHOTYPE:
        *write_method = write_AdvancePPTPClientLCPEchoType;
        read_csman(fd,0x00052300+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPCLIENTLCPECHOINTERVAL:
        *write_method = write_AdvancePPTPClientLCPEchoInterval;
        read_csman(fd,0x00052400+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPCLIENTLCPECHOMAXFAILURETIME:
        *write_method = write_AdvancePPTPClientLCPEchoMaxFailureTime;
        read_csman(fd,0x00052500+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPCLIENTTUNNELENABLE:
        *write_method = write_AdvancePPTPClientTunnelEnable;
        read_csman(fd,0x00051100+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


int
write_AdvancePPTPCEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x0020100f,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvancePPTPClientAlter(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;
	
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x80051001,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvancePPTPClientEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00051000,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}

int
write_AdvancePPTPClientName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
             int fd=0;
             write_csman(fd,0x00051200+w_index_adv_vpn_pptp_client-1,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePPTPClientOperationMode(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00052700+w_index_adv_vpn_pptp_client-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePPTPClientOperationModeFailoverType(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00052800+w_index_adv_vpn_pptp_client-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePPTPClientOperationModeFailoverTunnel(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00052900+w_index_adv_vpn_pptp_client-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePPTPClientRemoteIPFQDN(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  strncpy(string , (char *) var_val, var_val_len);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
             int fd=0;
             write_csman(fd,0x00051b00+w_index_adv_vpn_pptp_client-1,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePPTPClientUserName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  strncpy(string , (char *) var_val, var_val_len);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
             int fd=0;
             write_csman(fd,0x00051400+w_index_adv_vpn_pptp_client-1,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePPTPClientPassword(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  strncpy(string , (char *) var_val, var_val_len);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
             int fd=0;
             write_csman(fd,0x00051500+w_index_adv_vpn_pptp_client-1,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePPTPClientDefGateway(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  strncpy(string , (char *) var_val, var_val_len);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
             int fd=0;
             write_csman(fd,0x00051600+w_index_adv_vpn_pptp_client-1,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePPTPClientRemoteSubnet(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  strncpy(string , (char *) var_val, var_val_len);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
             int fd=0;
             write_csman(fd,0x00051600+w_index_adv_vpn_pptp_client-1,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePPTPClientAuthEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00051c00+w_index_adv_vpn_pptp_client-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePPTPClientAuthPAPEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00051d00+w_index_adv_vpn_pptp_client-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePPTPClientAuthCHAPEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00051e00+w_index_adv_vpn_pptp_client-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePPTPClientAuthMSCHAPEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;
	
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00051f00+w_index_adv_vpn_pptp_client-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePPTPClientAuthMSCHAPv2Enable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00052000+w_index_adv_vpn_pptp_client-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePPTPClientMPPEEncryptionEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00051800+w_index_adv_vpn_pptp_client-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePPTPClientNATbeforeTunnelingEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00051900+w_index_adv_vpn_pptp_client-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePPTPClientLCPEchoType(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00052300+w_index_adv_vpn_pptp_client-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePPTPClientLCPEchoInterval(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00052400+w_index_adv_vpn_pptp_client-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePPTPClientLCPEchoMaxFailureTime(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00052500+w_index_adv_vpn_pptp_client-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_AdvancePPTPClientTunnelEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPClient not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPClient: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00051100+w_index_adv_vpn_pptp_client-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
