#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIAdvVPNConfiguration.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#include "UIAdvVPNPPTPClientStatus.h"

#define NUM_ADV_VPN_PPTP_CLIENT_STATUS 10

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIAdvVPNPPTPClientStatus_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,3,3,8 };
#else
oid UIAdvVPNPPTPClientStatus_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,3,3,8 };
#endif

struct variable7 UIAdvVPNPPTPClientStatus_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{ADVANCEPPTPCLIENTSTATUSINDEX,ASN_INTEGER,RONLY,var_AdvVPNPPTPClientTable, 3,  { 1 , 1, 1 }},
{ADVANCEPPTPCLIENTSTATUSCLIENTNAME,ASN_OCTET_STR,RONLY,var_AdvVPNPPTPClientTable, 3,  { 1 , 1, 2 }},
{ADVANCEPPTPCLIENTSTATUSVIRTUALIP,ASN_IPADDRESS,RONLY,var_AdvVPNPPTPClientTable, 3,  { 1 , 1, 3 }},
{ADVANCEPPTPCLIENTSTATUSREMOTEIPFQDN,ASN_IPADDRESS,RONLY,var_AdvVPNPPTPClientTable, 3,  { 1 , 1, 4 }},
{ADVANCEPPTPCLIENTSTATUSDEFAULTGWREMOTESUBNET,ASN_OCTET_STR,RONLY,var_AdvVPNPPTPClientTable, 3,  { 1 , 1, 5 }},
{ADVANCEPPTPCLIENTINDEXSTATUS,ASN_INTEGER,RONLY,var_AdvVPNPPTPClientTable, 3,  { 1 , 1, 6 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UIAdvVPNPPTPClientStatus module */
void
init_UIAdvVPNPPTPClientStatus(void)
{
    DEBUGMSGTL(("UIAdvVPNPPTPClientStatus", "Initializing\n"));
    REGISTER_MIB("UIAdvVPNPPTPClientStatus", UIAdvVPNPPTPClientStatus_variables, variable7,
               UIAdvVPNPPTPClientStatus_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	/* in this case, oid len is 17*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}


unsigned char *
var_UIAdvVPNPPTPClientStatus(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_AdvVPNPPTPClientTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UIAdvVPNPPTPClientStatus above.
 */
unsigned char *
var_AdvVPNPPTPClientTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;

    int fd=0;
    /* get index */
    register int index;
    index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_ADV_VPN_PPTP_CLIENT_STATUS);

    if ( index == MATCH_FAILED )
        return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case ADVANCEPPTPCLIENTSTATUSINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case ADVANCEPPTPCLIENTSTATUSCLIENTNAME:
        memset(string,0,sizeof(string));
		read_csman(fd,0x00051200+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case ADVANCEPPTPCLIENTSTATUSVIRTUALIP:
        memset(string,0,sizeof(string));
		read_csman(fd,0x80052300+index-1,string,sizeof(string),TYPE_STR);
		*var_len = 4;
		return (unsigned char *) string;
    case ADVANCEPPTPCLIENTSTATUSREMOTEIPFQDN:
        memset(string,0,sizeof(string));
		read_csman(fd,0x80052600+index-1,string,sizeof(string),TYPE_STR);
		*var_len = 4;
		return (unsigned char *) string;
    case ADVANCEPPTPCLIENTSTATUSDEFAULTGWREMOTESUBNET:
        memset(string,0,sizeof(string));
		read_csman(fd,0x00051600+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case ADVANCEPPTPCLIENTINDEXSTATUS:
	{
		int peer_pid=0;
		read_csman(fd,0x80052400+index-1,&peer_pid,sizeof(unsigned int),TYPE_U32);

		if(peer_pid == 0){
        	read_csman(fd,0x00051700+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		}
		else if(peer_pid == 2)	long_ret=2;
		else if(peer_pid == -1)	long_ret=-1;
		return (unsigned char *) &long_ret;
    }
    default:
      ERROR_MSG("");
    }
    return NULL;
}


