#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIAdvVPNConfiguration.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#include "UIAdvVPNPPTPServer.h"

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIAdvVPNPPTPServer_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,3,3,4,1 };
#else
oid UIAdvVPNPPTPServer_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,3,3,4,1 };
#endif

struct variable7 UIAdvVPNPPTPServer_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{ADVANCEPPTPSENABLE,  ASN_INTEGER,RWRITE,var_UIAdvVPNPPTPServer, 1,  { 1 }},
{ADVANCEPPTPSERVENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNPPTPServer, 1,  { 2 }},
{ADVANCEPPTPSERVVIRTUALIP,ASN_IPADDRESS,RWRITE,var_UIAdvVPNPPTPServer, 1,  { 3 }},
{ADVANCEPPTPSERVIPPOOLSTARTINGADDRESS,ASN_IPADDRESS,RWRITE,var_UIAdvVPNPPTPServer, 1,  { 4 }},
{ADVANCEPPTPSERVIPPOOLENDINGADDRESS,ASN_IPADDRESS,RWRITE,var_UIAdvVPNPPTPServer, 1,  { 5 }},
{ADVANCEPPTPSERVAUTHPROTOPAPENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNPPTPServer, 1,  { 6 }},
{ADVANCEPPTPSERVAUTHPROTOCHAPENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNPPTPServer, 1,  { 7 }},
{ADVANCEPPTPSERVAUTHPROTOMSCHAPENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNPPTPServer, 1,  { 8 }},
{ADVANCEPPTPSERVAUTHPROTOMSCHAPV2ENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNPPTPServer, 1,  { 9 }},
{ADVANCEPPTPSERVMPPEENCRYPTIONENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNPPTPServer, 1,  { 10 }},
{ADVANCEPPTPSERVMPPEENCRYPTION40BITSENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNPPTPServer, 1,  { 11 }},
{ADVANCEPPTPSERVMPPEENCRYPTION56NITSENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNPPTPServer, 1,  { 12 }},
{ADVANCEPPTPSERVMPPEENCRYPTION128NITSENABLE,ASN_INTEGER,RWRITE,var_UIAdvVPNPPTPServer, 1,  { 13 }},

};
/*    (L = length of the oidsuffix) */


/** Initializes the UIAdvVPNPPTPServer module */
void
init_UIAdvVPNPPTPServer(void)
{
    DEBUGMSGTL(("UIAdvVPNPPTPServer", "Initializing\n"));
    REGISTER_MIB("UIAdvVPNPPTPServer", UIAdvVPNPPTPServer_variables, variable7,
               UIAdvVPNPPTPServer_variables_oid);
}

unsigned char *
var_UIAdvVPNPPTPServer(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case ADVANCEPPTPSENABLE:
        *write_method = write_AdvancePPTPSEnable;
        read_csman(fd,0x0020100f,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPSERVENABLE:
        *write_method = write_AdvancePPTPServEnable;
        read_csman(fd,0x001a0001,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPSERVVIRTUALIP:
        *write_method = write_AdvancePPTPServVirtualIP;
        memset(string,0,sizeof(string));
		read_csman(fd,0x001a0008,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
    case ADVANCEPPTPSERVIPPOOLSTARTINGADDRESS:
        *write_method = write_AdvancePPTPServIPPoolStartingAddress;
        memset(string,0,sizeof(string));
		read_csman(fd,0x001a0009,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
    case ADVANCEPPTPSERVIPPOOLENDINGADDRESS:
        *write_method = write_AdvancePPTPServIPPoolEndingAddress;
        memset(string,0,sizeof(string));
		read_csman(fd,0x001a000a,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
    case ADVANCEPPTPSERVAUTHPROTOPAPENABLE:
        *write_method = write_AdvancePPTPServAuthProtoPAPEnable;
        read_csman(fd,0x001a0003,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
    case ADVANCEPPTPSERVAUTHPROTOCHAPENABLE:
        *write_method = write_AdvancePPTPServAuthProtoCHAPEnable;
        read_csman(fd,0x001a0004,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
    case ADVANCEPPTPSERVAUTHPROTOMSCHAPENABLE:
        *write_method = write_AdvancePPTPServAuthProtoMSCHAPEnable;
        read_csman(fd,0x001a0005,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPSERVAUTHPROTOMSCHAPV2ENABLE:
        *write_method = write_AdvancePPTPServAuthProtoMSCHAPv2Enable;
        read_csman(fd,0x001a0006,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPSERVMPPEENCRYPTIONENABLE:
        *write_method = write_AdvancePPTPServMPPEEncryptionEnable;
        read_csman(fd,0x001a0010,&long_ret,sizeof(unsigned int),TYPE_U32);
		 return (unsigned char *) &long_ret;
    case ADVANCEPPTPSERVMPPEENCRYPTION40BITSENABLE:
        *write_method = write_AdvancePPTPServMPPEEncryption40bitsEnable;
        read_csman(fd,0x001a0011,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPSERVMPPEENCRYPTION56NITSENABLE:
        *write_method = write_AdvancePPTPServMPPEEncryption56nitsEnable;
        read_csman(fd,0x001a0012,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case ADVANCEPPTPSERVMPPEENCRYPTION128NITSENABLE:
        *write_method = write_AdvancePPTPServMPPEEncryption128nitsEnable;
        read_csman(fd,0x001a0013,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}

int
write_AdvancePPTPSEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x0020100f,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvancePPTPServEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x001a0001,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvancePPTPServVirtualIP(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];
	
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIAdvVPNPPTPServer not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(in_addr_t)) {
              fprintf(stderr,"write to UIAdvVPNPPTPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  memcpy(string , (char *) var_val, 4);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		  {
			  int fd=0;
			  write_csman(fd,0x001a0008,string,strlen(string) ,TYPE_IPV4);
			  break;
		  }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvancePPTPServIPPoolStartingAddress(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIAdvVPNPPTPServer not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(in_addr_t)) {
              fprintf(stderr,"write to UIAdvVPNPPTPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  memcpy(string , (char *) var_val, 4);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		  {
			  int fd=0;
			  write_csman(fd,0x001a0009,string,strlen(string) ,TYPE_IPV4);
			  break;
		  }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvancePPTPServIPPoolEndingAddress(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIAdvVPNPPTPServer not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(in_addr_t)) {
              fprintf(stderr,"write to UIAdvVPNPPTPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  memcpy(string , (char *) var_val, 4);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		  {
			  int fd=0;
			  write_csman(fd,0x001a000a,string,strlen(string) ,TYPE_IPV4);
			  break;
		  }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvancePPTPServAuthProtoPAPEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
              int fd=0;
              write_csman(fd,0x001a0003,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
         }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvancePPTPServAuthProtoCHAPEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
              int fd=0;
              write_csman(fd,0x001a0004,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
         }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvancePPTPServAuthProtoMSCHAPEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
              int fd=0;
              write_csman(fd,0x001a0005,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
         }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvancePPTPServAuthProtoMSCHAPv2Enable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
              int fd=0;
              write_csman(fd,0x001a0006,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
         }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvancePPTPServMPPEEncryptionEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
              int fd=0;
              write_csman(fd,0x001a0010,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
         }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvancePPTPServMPPEEncryption40bitsEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
              int fd=0;
              write_csman(fd,0x001a0011,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			  system("rmcsman 0x001a0012");
			  system("rmcsman 0x001a0013");
              break;
         }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvancePPTPServMPPEEncryption56nitsEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
              int fd=0;
              write_csman(fd,0x001a0012,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			  system("rmcsman 0x001a0011");
			  system("rmcsman 0x001a0013");
			  break;
         }
    }
    return SNMP_ERR_NOERROR;
}


int
write_AdvancePPTPServMPPEEncryption128nitsEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIAdvVPNPPTPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIAdvVPNPPTPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
              int fd=0;
              write_csman(fd,0x001a0013,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			  system("rmcsman 0x001a0011");
			  system("rmcsman 0x001a0012");
			  break;
         }
    }
    return SNMP_ERR_NOERROR;
}


