#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UIBasicClientServerDHCPServer.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_BAS_CLIENT_SERVER_DHCPSERVER 4
int w_index_bas_client_dhcpsrver;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIBasicClientServerDHCPServer_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,8,2 };
#else
oid UIBasicClientServerDHCPServer_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,8,2 };
#endif

struct variable7 UIBasicClientServerDHCPServer_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{BASICDHCPSERVERINDEX,ASN_INTEGER,RONLY,var_BasicClientServerDCHPTable, 3,  { 1 , 1, 1 }},
{BASICDHCPSERVERENABLE,ASN_INTEGER,RWRITE,var_BasicClientServerDCHPTable, 3,  { 1 , 1, 2 }},
{BASICDHCPSERVERLANIPADDRESS,ASN_IPADDRESS,RWRITE,var_BasicClientServerDCHPTable, 3,  { 1 , 1, 3 }},
{BASICDHCPSERVERSUBNETMASK,ASN_IPADDRESS,RWRITE,var_BasicClientServerDCHPTable, 3,  { 1 , 1, 4 }},
{BASICDHCPSERVERIPPOOLSTARTINGADDR,ASN_IPADDRESS,RWRITE,var_BasicClientServerDCHPTable, 3,  { 1 , 1, 5 }},
{BASICDHCPSERVERIPPOOLENDINGADDR,ASN_IPADDRESS,RWRITE,var_BasicClientServerDCHPTable, 3,  { 1 , 1, 6 }},
{BASICDHCPSERVERLEASETIME,ASN_INTEGER,RWRITE,var_BasicClientServerDCHPTable, 3,  { 1 , 1, 7 }},
{BASICDHCPSERVERDOMAINNAME,ASN_OCTET_STR,RWRITE,var_BasicClientServerDCHPTable, 3,  { 1 , 1, 8 }},
{BASICDHCPSERVERPRIMARYDNS,ASN_IPADDRESS,RWRITE,var_BasicClientServerDCHPTable, 3,  { 1 , 1, 9 }},
{BASICDHCPSERVERSECONDARYDNS,ASN_IPADDRESS,WRITE,var_BasicClientServerDCHPTable, 3,  { 1 , 1, 10 }},
{BASICDHCPSERVERPRIMARYWINS,ASN_IPADDRESS,RWRITE,var_BasicClientServerDCHPTable, 3,  { 1 , 1, 11 }},
{BASICDHCPSERVERSECONDARYWINS,ASN_IPADDRESS,RWRITE,var_BasicClientServerDCHPTable, 3,  { 1 , 1, 12 }},
{BASICDHCPSERVERGATEWAY,ASN_IPADDRESS,RWRITE,var_BasicClientServerDCHPTable, 3,  { 1 , 1, 13 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UIBasicClientServerDHCPServer module */
void
init_UIBasicClientServerDHCPServer(void)
{

    DEBUGMSGTL(("UIBasicClientServerDHCPServer", "Initializing\n"));
    /* register ourselves with the agent to handle our mib tree */
    REGISTER_MIB("UIBasicClientServerDHCPServer", UIBasicClientServerDHCPServer_variables, variable7,
               UIBasicClientServerDHCPServer_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UIBasicClientServerDHCPServer(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd = 0;
	
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_BasicClientServerDCHPTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UIBasicClientServerDHCPServer above.
 */
unsigned char *
var_BasicClientServerDCHPTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd = 0;

    /* get index */
    register int index;
    index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_BAS_CLIENT_SERVER_DHCPSERVER);
    w_index_bas_client_dhcpsrver=index;

    if ( index == MATCH_FAILED )
        return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICDHCPSERVERINDEX:
        long_ret = index;
        return (unsigned int *) &long_ret;
    case BASICDHCPSERVERENABLE:
	{
        *write_method = write_BasicDHCPServerEnable;
		if(index == 1)	read_csman(fd,0x00030001,&long_ret,sizeof(unsigned int),TYPE_U32);
		else 			read_csman(fd,0x00930010+index-2,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    }
    case BASICDHCPSERVERLANIPADDRESS:
	{
        *write_method = write_BasicDHCPServerLANIPAddress;
		memset(string,0,sizeof(string));
		if(index == 1)	read_csman(fd,0x00010004,string,sizeof(string),TYPE_IPV4);
		else read_csman(fd,0x009300d0+index-2,string,sizeof(string),TYPE_IPV4);
		*var_len = 4;
		return (unsigned char *) string;
    }
    case BASICDHCPSERVERSUBNETMASK:
	{
        *write_method = write_BasicDHCPServerSubnetMask;
		memset(string,0,sizeof(string));
		if(index == 1)	read_csman(fd,0x00010005,string,sizeof(string),TYPE_IPV4);
		else read_csman(fd,0x00930120+index-2,string,sizeof(string),TYPE_IPV4);
		*var_len = 4;
		return (unsigned char *) string;
    }
    case BASICDHCPSERVERIPPOOLSTARTINGADDR:
	{
        *write_method = write_BasicDHCPServerIPPoolStartingAddr;
        memset(string,0,sizeof(string));
		if(index == 1)	read_csman(fd,0x00030003,string,sizeof(string),TYPE_IPV4);
		else read_csman(fd,0x00930030+index-2,string,sizeof(string),TYPE_IPV4);
		*var_len = 4;
		return (unsigned char *) string;
    }
    case BASICDHCPSERVERIPPOOLENDINGADDR:
	{
        *write_method = write_BasicDHCPServerIPPoolEndingAddr;
        memset(string,0,sizeof(string));
		if(index == 1)	read_csman(fd,0x00030004,string,sizeof(string),TYPE_IPV4);
		else read_csman(fd,0x00930040+index-2,string,sizeof(string),TYPE_IPV4);
		*var_len = 4;
		return (unsigned char *) string;
    }
    case BASICDHCPSERVERLEASETIME:
	{
        *write_method = write_BasicDHCPServerLeaseTime;
        if(index == 1)	read_csman(fd,0x00030002,&long_ret,sizeof(unsigned int),TYPE_U32);
		else 			read_csman(fd,0x00930020+index-2,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
	}
    case BASICDHCPSERVERDOMAINNAME:
	{
        *write_method = write_BasicDHCPServerDomainName;
        memset(string,0,sizeof(string));
		if(index == 1)	read_csman(fd,0x00030005,string,sizeof(string),TYPE_STR);
		else 			read_csman(fd,0x00930050+index-2,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    }
    case BASICDHCPSERVERPRIMARYDNS:
	{
        *write_method = write_BasicDHCPServerPrimaryDNS;
        memset(string,0,sizeof(string));
		if(index == 1)	read_csman(fd,0x00030006 ,string,sizeof(string),TYPE_IPV4);
		else read_csman(fd,0x00930060+index-2,string,sizeof(string),TYPE_IPV4);
		*var_len = 4;
		return (unsigned char *) string;
    }
    case BASICDHCPSERVERSECONDARYDNS:
	{
        *write_method = write_BasicDHCPServerSecondaryDNS;
        memset(string,0,sizeof(string));
		if(index == 1)	read_csman(fd,0x00030007 ,string,sizeof(string),TYPE_IPV4);
		else read_csman(fd,0x00930070+index-2,string,sizeof(string),TYPE_IPV4);
		*var_len = 4;
		return (unsigned char *) string;
    }
    case BASICDHCPSERVERPRIMARYWINS:
	{
        *write_method = write_BasicDHCPServerPrimaryWINS;
        memset(string,0,sizeof(string));
		if(index == 1)	read_csman(fd,0x00030008 ,string,sizeof(string),TYPE_IPV4);
		else read_csman(fd,0x00930080+index-2,string,sizeof(string),TYPE_IPV4);
		*var_len = 4;
		return (unsigned char *) string;
    }
    case BASICDHCPSERVERSECONDARYWINS:
	{
        *write_method = write_BasicDHCPServerSecondaryWINS;
        memset(string,0,sizeof(string));
		if(index == 1)	read_csman(fd,0x00030009 ,string,sizeof(string),TYPE_IPV4);
		else read_csman(fd,0x00930090+index-2,string,sizeof(string),TYPE_IPV4);
		*var_len = 4;
		return (unsigned char *) string;
    }
    case BASICDHCPSERVERGATEWAY:
	{
        *write_method = write_BasicDHCPServerGateway;
        memset(string,0,sizeof(string));
		if(index == 1)	read_csman(fd,0x0003000a ,string,sizeof(string),TYPE_IPV4);
		else read_csman(fd,0x009300a0+index-2,string,sizeof(string),TYPE_IPV4);
		*var_len = 4;
		return (unsigned char *) string;
    }
    default:
      ERROR_MSG("");
    }
    return NULL;
}


int
write_BasicDHCPServerEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicClientServerDHCPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicClientServerDHCPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
            int fd=0;
			if(w_index_bas_client_dhcpsrver == 1)	write_csman(fd,0x00030001,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			else write_csman(fd,0x00930010+w_index_bas_client_dhcpsrver-2,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			
            break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicDHCPServerLANIPAddress(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIBasicClientServerDHCPServer not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicClientServerDHCPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			memcpy(string , (char *) var_val, 4);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			if(w_index_bas_client_dhcpsrver == 1)
				write_csman(fd,0x00010004,string,strlen(string) ,TYPE_IPV4);
			else
				write_csman(fd,0x009300d0+w_index_bas_client_dhcpsrver-2,string,strlen(string) ,TYPE_IPV4);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicDHCPServerSubnetMask(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIBasicClientServerDHCPServer not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicClientServerDHCPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			memcpy(string , (char *) var_val, 4);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			if(w_index_bas_client_dhcpsrver == 1)
				write_csman(fd,0x00010005,string,strlen(string) ,TYPE_IPV4);
			else
				write_csman(fd,0x00930120+w_index_bas_client_dhcpsrver-2,string,strlen(string) ,TYPE_IPV4);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicDHCPServerIPPoolStartingAddr(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIBasicClientServerDHCPServer not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicClientServerDHCPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  memcpy(string , (char *) var_val, 4);
			  break;
        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			if(w_index_bas_client_dhcpsrver == 1)
				write_csman(fd,0x00030003,string,strlen(string) ,TYPE_IPV4);
			else
				write_csman(fd,0x00930030+w_index_bas_client_dhcpsrver-2,string,strlen(string) ,TYPE_IPV4);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicDHCPServerIPPoolEndingAddr(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];
	
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIBasicClientServerDHCPServer not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicClientServerDHCPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			memcpy(string , (char *) var_val, 4);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			if(w_index_bas_client_dhcpsrver == 1)
				write_csman(fd,0x00030004,string,strlen(string) ,TYPE_IPV4);
			else
				write_csman(fd,0x00930040+w_index_bas_client_dhcpsrver-2,string,strlen(string) ,TYPE_IPV4);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicDHCPServerLeaseTime(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicClientServerDHCPServer not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicClientServerDHCPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			if(w_index_bas_client_dhcpsrver == 1)
				write_csman(fd,0x00030002,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			else 
				write_csman(fd,0x00930020+w_index_bas_client_dhcpsrver-2,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicDHCPServerDomainName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIBasicClientServerDHCPServer not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicClientServerDHCPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
            int fd=0;
			if(w_index_bas_client_dhcpsrver == 1)
				write_csman(fd,0x00030005,string,strlen(string)+1,TYPE_STR);
			else 
				write_csman(fd,0x00930050+w_index_bas_client_dhcpsrver-2,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicDHCPServerPrimaryDNS(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIBasicClientServerDHCPServer not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicClientServerDHCPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			memcpy(string , (char *) var_val, 4);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			if(w_index_bas_client_dhcpsrver == 1)
				write_csman(fd,0x00030006,string,strlen(string) ,TYPE_IPV4);
			else
				write_csman(fd,0x00930060+w_index_bas_client_dhcpsrver-2,string,strlen(string) ,TYPE_IPV4);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicDHCPServerSecondaryDNS(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIBasicClientServerDHCPServer not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicClientServerDHCPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  memcpy(string , (char *) var_val, 4);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			if(w_index_bas_client_dhcpsrver == 1)
				write_csman(fd,0x00030007,string,strlen(string) ,TYPE_IPV4);
			else
				write_csman(fd,0x00930070+w_index_bas_client_dhcpsrver-2,string,strlen(string) ,TYPE_IPV4);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicDHCPServerPrimaryWINS(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIBasicClientServerDHCPServer not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicClientServerDHCPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  memcpy(string , (char *) var_val, 4);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			if(w_index_bas_client_dhcpsrver == 1)
				write_csman(fd,0x00030008,string,strlen(string) ,TYPE_IPV4);
			else
				write_csman(fd,0x00930080+w_index_bas_client_dhcpsrver-2,string,strlen(string) ,TYPE_IPV4);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicDHCPServerSecondaryWINS(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIBasicClientServerDHCPServer not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicClientServerDHCPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  memcpy(string , (char *) var_val, 4);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			if(w_index_bas_client_dhcpsrver == 1)
				write_csman(fd,0x00030009,string,strlen(string) ,TYPE_IPV4);
			else
				write_csman(fd,0x00930090+w_index_bas_client_dhcpsrver-2,string,strlen(string) ,TYPE_IPV4);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicDHCPServerGateway(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIBasicClientServerDHCPServer not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicClientServerDHCPServer: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  memcpy(string , (char *) var_val, 4);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			if(w_index_bas_client_dhcpsrver == 1)
				write_csman(fd,0x0003000a,string,strlen(string) ,TYPE_IPV4);
			else
				write_csman(fd,0x009300a0+w_index_bas_client_dhcpsrver-2,string,strlen(string) ,TYPE_IPV4);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
