#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UIBasicClientServerDHCPServerClientList.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_BAS_CLIENT_SERVER_DHCPSERVER 256
int w_index_bas_client_dhcpsrver_clientlist;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIBasicClientServerDHCPServerClientList_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,8,3 };
#else
oid UIBasicClientServerDHCPServerClientList_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,8,3 };
#endif

struct variable7 UIBasicClientServerDHCPServerClientList_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{BASICDHCPCLIENTLISTINDEX,ASN_INTEGER,RONLY,var_BasicClientServerDCHPListTable, 3,  { 1 , 1, 1 }},
{BASICDHCPCLIENTLISTIPADDRESS,ASN_IPADDRESS,RONLY,var_BasicClientServerDCHPListTable, 3,  { 1 , 1, 2 }},
{BASICDHCPCLIENTLISTHOSTNAME,ASN_OCTET_STR,RONLY,var_BasicClientServerDCHPListTable, 3,  { 1 , 1, 3 }},
{BASICDHCPCLIENTLISTMACADDRESS,ASN_OCTET_STR,RONLY,var_BasicClientServerDCHPListTable, 3,  { 1 , 1, 4 }},
{BASICDHCPCLIENTLISTTYPE,ASN_INTEGER,RONLY,var_BasicClientServerDCHPListTable, 3,  { 1 , 1, 5 }},
{BASICDHCPCLIENTLISTLEASETIME,ASN_INTEGER,RONLY,var_BasicClientServerDCHPListTable, 3,  { 1 , 1, 6 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UIBasicClientServerDHCPServerClientList module */
void
init_UIBasicClientServerDHCPServerClientList(void)
{
    DEBUGMSGTL(("UIBasicClientServerDHCPServerClientList", "Initializing\n"));
    REGISTER_MIB("UIBasicClientServerDHCPServerClientList", UIBasicClientServerDHCPServerClientList_variables, variable7,
               UIBasicClientServerDHCPServerClientList_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}


/*
 * var_UIBasicClientServerDHCPServerClientList():
 *   This function is called every time the agent gets a request for
 *   a scalar variable that might be found within your mib section
 *   registered above.  It is up to you to do the right thing and
 *   return the correct value.
 *     You should also correct the value of "var_len" if necessary.
 *
 *   Please see the documentation for more information about writing
 *   module extensions, and check out the examples in the examples
 *   and mibII directories.
 */
unsigned char *
var_UIBasicClientServerDHCPServerClientList(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_BasicClientServerDCHPListTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UIBasicClientServerDHCPServerClientList above.
 */
unsigned char *
var_BasicClientServerDCHPListTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;

	/* get index */
    register int index;
    index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_BAS_CLIENT_SERVER_DHCPSERVER);
    w_index_bas_client_dhcpsrver_clientlist=index;

    if ( index == MATCH_FAILED )
        return NULL;
	
	system("udhcpd-action lease");
    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICDHCPCLIENTLISTINDEX:
        long_ret = index;
        return (unsigned int *) &long_ret;
    case BASICDHCPCLIENTLISTIPADDRESS:
        memset(string,0,sizeof(string));
		read_csman(fd,0x80030100+index-1,string,sizeof(string),TYPE_IPV4);
		*var_len = 4;
		return (unsigned char *) string;
    case BASICDHCPCLIENTLISTHOSTNAME:
        memset(string,0,sizeof(string));
		read_csman(fd,0x80030200+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICDHCPCLIENTLISTMACADDRESS:
        memset(string,0,sizeof(string));
		read_csman(fd,0x80030300+index-1,string,sizeof(string),TYPE_MAC);
		*var_len = 6;
		return (unsigned char *) string;
    case BASICDHCPCLIENTLISTTYPE:
        read_csman(fd,0x80030500+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICDHCPCLIENTLISTLEASETIME:
        read_csman(fd,0x80030400+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned int *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


