#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIBasicEthernetLAN.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIBasicEthernetLAN_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,2,1 };
#else
oid UIBasicEthernetLAN_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,2,1 };
#endif

struct variable7 UIBasicEthernetLAN_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{BASICETHERNETLANIPMODE,ASN_INTEGER,RONLY,var_UIBasicEthernetLAN, 1,  { 1 }},
{BASICETHERNETLANIPADDRESS,ASN_IPADDRESS,RWRITE,var_UIBasicEthernetLAN, 1,  { 2 }},
{BASICETHERNETLANIPSUBNETMASK,ASN_IPADDRESS,RWRITE,var_UIBasicEthernetLAN, 1,  { 3 }},
{BASICETHERNETLANAPMODECNTTYPE,ASN_INTEGER,RWRITE,var_UIBasicEthernetLAN, 1,  { 4 }},
{BASICETHERNETLANAPMODEGATEWAY,ASN_IPADDRESS,RWRITE,var_UIBasicEthernetLAN, 1,  { 5 }},
{BASICETHERNETLANAPMODEPRIDNS,ASN_IPADDRESS,RWRITE,var_UIBasicEthernetLAN, 1,  { 6 }},
{BASICETHERNETLANAPMODESECDNS,ASN_IPADDRESS,RWRITE,var_UIBasicEthernetLAN, 1,  { 7 }},
{BASICETHERNETLANAPMODELANIPADDRESS,ASN_IPADDRESS,RWRITE,var_UIBasicEthernetLAN, 1,  { 8 }},
{BASICETHERNETLANAPMODESUBNETMASK,ASN_IPADDRESS,RWRITE,var_UIBasicEthernetLAN, 1,  { 9 }},
{BASICETHERNETLANAPMODEALTER,ASN_INTEGER,RWRITE,var_UIBasicEthernetLAN, 1,  { 10 }},

};
/*    (L = length of the oidsuffix) */


/** Initializes the UIBasicEthernetLAN module */
void
init_UIBasicEthernetLAN(void)
{
    DEBUGMSGTL(("UIBasicEthernetLAN", "Initializing\n"));
    REGISTER_MIB("UIBasicEthernetLAN", UIBasicEthernetLAN_variables, variable7,
               UIBasicEthernetLAN_variables_oid);
}

/*
 * var_UIBasicEthernetLAN():
 *   This function is called every time the agent gets a request for
 *   a scalar variable that might be found within your mib section
 *   registered above.  It is up to you to do the right thing and
 *   return the correct value.
 *     You should also correct the value of "var_len" if necessary.
 *
 *   Please see the documentation for more information about writing
 *   module extensions, and check out the examples in the examples
 *   and mibII directories.
 */
unsigned char *
var_UIBasicEthernetLAN(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICETHERNETLANIPMODE:
		long_ret=0;
		return (unsigned char *) &long_ret;
	case BASICETHERNETLANIPADDRESS:
		*write_method = write_BasicEthernetLANIPAddress;
		memset(string,0,sizeof(string));
		read_csman(fd,0x00010004,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
	case BASICETHERNETLANIPSUBNETMASK:
		*write_method = write_BasicEthernetLANSubnetMask;
		memset(string,0,sizeof(string));
		read_csman(fd,0x00010005,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
    case BASICETHERNETLANAPMODECNTTYPE:
        *write_method = write_BasicEthernetLANAPModeCNTType;
        read_csman(fd,0x00010035,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
	case BASICETHERNETLANAPMODEGATEWAY:	
		*write_method = write_BasicEthernetLANAPModeGateWay;
		memset(string,0,sizeof(string));
		read_csman(fd,0x00110013,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
	case BASICETHERNETLANAPMODEPRIDNS:
		 *write_method = write_BasicEthernetLANAPModePriDNS;
		memset(string,0,sizeof(string));
		read_csman(fd,0x00110014 ,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
	case BASICETHERNETLANAPMODESECDNS:
		*write_method = write_BasicEthernetLANAPModeSecDNS;
		memset(string,0,sizeof(string));
		read_csman(fd,0x00110015,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
	case BASICETHERNETLANAPMODELANIPADDRESS:
		*write_method = write_BasicEthernetLANAPModeLANIPAddress;
		memset(string,0,sizeof(string));
		read_csman(fd, 0x00110011,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
	case BASICETHERNETLANAPMODESUBNETMASK:
		*write_method = write_BasicEthernetLANAPModeSubnetMask;
		memset(string,0,sizeof(string));
		read_csman(fd, 0x00110012,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
    case BASICETHERNETLANAPMODEALTER:
        *write_method = write_BasicEthernetLANAPModeAlter;
        long_ret=0;
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}

int
write_BasicEthernetLANIPAddress(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to BasicLANNetworkSetting not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to BasicLANNetworkSetting: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
		//for IPv4 data type
		memset(string,0,sizeof(string));
		memcpy(string , (char *) var_val, 4);
		break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
            {
             		int fd;
			write_csman(fd,0x00010004,string,strlen(string) ,TYPE_IPV4);
			break;
	     }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicEthernetLANSubnetMask(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to BasicLANNetworkSetting not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to BasicLANNetworkSetting: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
		//for IPv4 data type
		memset(string,0,sizeof(string));
		memcpy(string , (char *) var_val, 4);
		break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
            {
             		int fd;
			write_csman(fd,0x00010005,string,strlen(string) ,TYPE_IPV4);
			break;
	     }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicEthernetLANAPModeCNTType(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to BasicLANNetworkSetting not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to BasicLANNetworkSetting: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          {
              int fd=0;
              write_csman(fd,0x00010035,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicEthernetLANAPModeGateWay(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to BasicLANNetworkSetting not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to BasicLANNetworkSetting: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
		//for IPv4 data type
		memset(string,0,sizeof(string));
		memcpy(string , (char *) var_val, 4);
		break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
            {
             		int fd;
			write_csman(fd,0x00110013,string,strlen(string) ,TYPE_IPV4);
			break;
	     }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicEthernetLANAPModePriDNS(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];
    
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to BasicLANNetworkSetting not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to BasicLANNetworkSetting: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
		//for IPv4 data type
		memset(string,0,sizeof(string));
		memcpy(string , (char *) var_val, 4);
		break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
            {
             		int fd;
			write_csman(fd,0x00110014,string,strlen(string) ,TYPE_IPV4);
			break;
	     }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicEthernetLANAPModeSecDNS(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to BasicLANNetworkSetting not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to BasicLANNetworkSetting: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
		//for IPv4 data type
		memset(string,0,sizeof(string));
		memcpy(string , (char *) var_val, 4);
		break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
            {
             		int fd;
			write_csman(fd,0x00110015,string,strlen(string) ,TYPE_IPV4);
			break;
	     }
    }
    return SNMP_ERR_NOERROR;
}

int
write_BasicEthernetLANAPModeLANIPAddress(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to BasicLANNetworkSetting not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to BasicLANNetworkSetting: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
		//for IPv4 data type
		memset(string,0,sizeof(string));
		memcpy(string , (char *) var_val, 4);
		break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
            {
             		int fd;
			write_csman(fd,0x00110011,string,strlen(string) ,TYPE_IPV4);
			write_csman(fd,0x00010004,string,strlen(string) ,TYPE_IPV4);
			write_csman(fd,0x0011000a,string,strlen(string) ,TYPE_IPV4);
			break;
	     }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicEthernetLANAPModeSubnetMask(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to BasicLANNetworkSetting not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to BasicLANNetworkSetting: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
		//for IPv4 data type
		memset(string,0,sizeof(string));
		memcpy(string , (char *) var_val, 4);
		break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
            int fd=0;
			write_csman(fd,0x00110012,string,strlen(string) ,TYPE_IPV4);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}

int
write_BasicEthernetLANAPModeAlter(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to NATUI not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to NATUI: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          {
              int fd=0;
			  write_csman(fd,0x80010056 ,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}

