#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIBasicIPv66in4WANTypeConf.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIBasicIPv66in4WANTypeConf_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,5,6 };
#else
oid UIBasicIPv66in4WANTypeConf_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,5,6 };
#endif

struct variable7 UIBasicIPv66in4WANTypeConf_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{BASICIPV66IN4REMOTEIPV4ADDRESS,ASN_IPADDRESS,RWRITE,var_UIBasicIPv66in4WANTypeConf, 1,  { 1 }},
{BASICIPV66IN4LOCALIPV4ADDRESS,ASN_IPADDRESS,RWRITE,var_UIBasicIPv66in4WANTypeConf, 1,  { 2 }},
{BASICIPV66IN4LOCALIPV6ADDRESS,ASN_OCTET_STR,RWRITE,var_UIBasicIPv66in4WANTypeConf, 1,  { 3 }},
{BASICIPV66IN4PRIMARYDNS,ASN_OCTET_STR,RWRITE,var_UIBasicIPv66in4WANTypeConf, 1,  { 4 }},
{BASICIPV66IN4SECONDARYDNS,ASN_OCTET_STR,RWRITE,var_UIBasicIPv66in4WANTypeConf, 1,  { 5 }},
{BASICIPV66IN4MLDSNOOPINGENABLE,ASN_INTEGER,RWRITE,var_UIBasicIPv66in4WANTypeConf, 1,  { 6 }},

};
/*    (L = length of the oidsuffix) */


/** Initializes the UIBasicIPv66in4WANTypeConf module */
void
init_UIBasicIPv66in4WANTypeConf(void)
{
    DEBUGMSGTL(("UIBasicIPv66in4WANTypeConf", "Initializing\n"));
    REGISTER_MIB("UIBasicIPv66in4WANTypeConf", UIBasicIPv66in4WANTypeConf_variables, variable7,
               UIBasicIPv66in4WANTypeConf_variables_oid);
}

unsigned char *
var_UIBasicIPv66in4WANTypeConf(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICIPV66IN4REMOTEIPV4ADDRESS:
        *write_method = write_BasicIPv66in4RemoteIPv4Address;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00900500,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
    case BASICIPV66IN4LOCALIPV4ADDRESS:
        *write_method = write_BasicIPv66in4LocalIPv4Address;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00900502,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
    case BASICIPV66IN4LOCALIPV6ADDRESS:
        *write_method = write_BasicIPv66in4LocalIPv6Address;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00900503,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICIPV66IN4PRIMARYDNS:
        *write_method = write_BasicIPv66in4PrimaryDNS;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00900504,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICIPV66IN4SECONDARYDNS:
        *write_method = write_BasicIPv66in4SecondaryDNS;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00900505,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICIPV66IN4MLDSNOOPINGENABLE:
        *write_method = write_BasicIPv66in4MLDSnoopingEnable;
        read_csman(fd,0x00151010,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


int
write_BasicIPv66in4RemoteIPv4Address(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIBasicIPv66in4WANTypeConf not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicIPv66in4WANTypeConf: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			memcpy(string , (char *) var_val, 4);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		  {
			  int fd=0;
			  write_csman(fd,0x00900500,string,strlen(string) ,TYPE_IPV4);
			  break;
		  }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicIPv66in4LocalIPv4Address(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIBasicIPv66in4WANTypeConf not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicIPv66in4WANTypeConf: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  memcpy(string , (char *) var_val, 4);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		  {
			  int fd=0;
			  write_csman(fd,0x00900502,string,strlen(string) ,TYPE_IPV4);
			  break;
		  }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicIPv66in4LocalIPv6Address(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIBasicIPv66in4WANTypeConf not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicIPv66in4WANTypeConf: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
             int fd=0;
             write_csman(fd,0x00900503,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicIPv66in4PrimaryDNS(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIBasicIPv66in4WANTypeConf not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicIPv66in4WANTypeConf: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
             int fd=0;
             write_csman(fd,0x00900504,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicIPv66in4SecondaryDNS(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIBasicIPv66in4WANTypeConf not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicIPv66in4WANTypeConf: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
             int fd=0;
             write_csman(fd,0x00900505,string,strlen(string)+1,TYPE_STR);
             break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicIPv66in4MLDSnoopingEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;
	
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to BasicIPv66to4MLDSnoopingEnable not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to BasicIPv66to4MLDSnoopingEnable: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
             int fd=0;
             write_csman(fd,0x00151010,&long_ret,sizeof(unsigned int) ,TYPE_U32);
             break;
         }
    }
    return SNMP_ERR_NOERROR;

}

