#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIBasicIPv6MainConfiguration.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIBasicIPv6MainConfiguration_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,5,1 };
#else
oid UIBasicIPv6MainConfiguration_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,5,1 };
#endif

struct variable7 UIBasicIPv6MainConfiguration_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{BASICIPV6CONFENABLE,ASN_INTEGER,RWRITE,var_UIBasicIPv6MainConfiguration, 1,  { 1 }},
{BASICIPV6CONFCONNECTIONTYPE,ASN_INTEGER,RWRITE,var_UIBasicIPv6MainConfiguration, 1,  { 2 }},

};
/*    (L = length of the oidsuffix) */


/** Initializes the UIBasicIPv6MainConfiguration module */
void
init_UIBasicIPv6MainConfiguration(void)
{
    DEBUGMSGTL(("UIBasicIPv6MainConfiguration", "Initializing\n"));
    REGISTER_MIB("UIBasicIPv6MainConfiguration", UIBasicIPv6MainConfiguration_variables, variable7,
               UIBasicIPv6MainConfiguration_variables_oid);
}

unsigned char *
var_UIBasicIPv6MainConfiguration(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICIPV6CONFENABLE:
        *write_method = write_BasicIPv6ConfEnable;
        read_csman(fd,0x00900000,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICIPV6CONFCONNECTIONTYPE:
	{
        *write_method = write_BasicIPv6ConfConnectionType;
		unsigned int ipv6_conn_type;
        read_csman(fd,0x00900001,&ipv6_conn_type,sizeof(unsigned int),TYPE_U32);
		
        if(ipv6_conn_type == 0x0100) //Static IPv6
			long_ret = 1;
		else if (ipv6_conn_type == 0x0200) //DHCP IPv6
			long_ret = 2;
		else if (ipv6_conn_type == 0x0300) //PPPoE
			long_ret = 3;
		else if (ipv6_conn_type == 0x0400) //6 to 4
			long_ret = 4;
		else if (ipv6_conn_type == 0x0500) //6 in 4
			long_ret = 5;
		else if (ipv6_conn_type == 0x0600) //6 in 4
			long_ret = 6;
		else if (ipv6_conn_type == 0x0700) //6 in 4
			long_ret = 7;

		return (unsigned char *) &long_ret;
    }
    default:
      ERROR_MSG("");
    }
    return NULL;
}

int
write_BasicIPv6ConfEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicIPv6MainConfiguration not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicIPv6MainConfiguration: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00900000,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicIPv6ConfConnectionType(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicIPv6MainConfiguration not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicIPv6MainConfiguration: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00900001,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}

