#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIBasicIPv6PPPoEv6WANTypeConf.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIBasicIPv6PPPoEv6WANTypeConf_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,5,4 };
#else
oid UIBasicIPv6PPPoEv6WANTypeConf_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,5,4 };
#endif

struct variable7 UIBasicIPv6PPPoEv6WANTypeConf_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{BASICIPV6PPPOEACCOUNT,ASN_OCTET_STR,RWRITE,var_UIBasicIPv6PPPoEv6WANTypeConf, 1,  { 1 }},
{BASICIPV6PPPOEPASSWORD,ASN_OCTET_STR,RWRITE,var_UIBasicIPv6PPPoEv6WANTypeConf, 1,  { 2 }},
{BASICIPV6PPPOESERVICENAME,ASN_OCTET_STR,RWRITE,var_UIBasicIPv6PPPoEv6WANTypeConf, 1,  { 3 }},
{BASICIPV6PPPOECONNECTIONCONTROL,ASN_INTEGER,RONLY,var_UIBasicIPv6PPPoEv6WANTypeConf, 1,  { 4 }},
{BASICIPV6PPPOEMTU,ASN_INTEGER,RWRITE,var_UIBasicIPv6PPPoEv6WANTypeConf, 1,  { 5 }},
{BASICIPV6PPPOEMLDSNOOPINGENABLE,ASN_INTEGER,RWRITE,var_UIBasicIPv6PPPoEv6WANTypeConf, 1,  { 6 }},

};
/*    (L = length of the oidsuffix) */


/** Initializes the UIBasicIPv6PPPoEv6WANTypeConf module */
void
init_UIBasicIPv6PPPoEv6WANTypeConf(void)
{
    DEBUGMSGTL(("UIBasicIPv6PPPoEv6WANTypeConf", "Initializing\n"));
    REGISTER_MIB("UIBasicIPv6PPPoEv6WANTypeConf", UIBasicIPv6PPPoEv6WANTypeConf_variables, variable7,
               UIBasicIPv6PPPoEv6WANTypeConf_variables_oid);
}

unsigned char *
var_UIBasicIPv6PPPoEv6WANTypeConf(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICIPV6PPPOEACCOUNT:
        *write_method = write_BasicIPv6PPPoEAccount;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00900300,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICIPV6PPPOEPASSWORD:
        *write_method = write_BasicIPv6PPPoEPassword;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00900301,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICIPV6PPPOESERVICENAME:
        *write_method = write_BasicIPv6PPPoEServiceName;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00900302,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICIPV6PPPOECONNECTIONCONTROL:
        read_csman(fd,0x00900306,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICIPV6PPPOEMTU:
        *write_method = write_BasicIPv6PPPoEMTU;
        read_csman(fd,0x00900303,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
    case BASICIPV6PPPOEMLDSNOOPINGENABLE:
        *write_method = write_BasicIPv6PPPoEMLDSnoopingEnable;
        read_csman(fd,0x00151010,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}

int
write_BasicIPv6PPPoEAccount(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIBasicIPv6PPPoEv6WANTypeConf not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicIPv6PPPoEv6WANTypeConf: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
             int fd=0;
             write_csman(fd,0x00900300,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicIPv6PPPoEPassword(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIBasicIPv6PPPoEv6WANTypeConf not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicIPv6PPPoEv6WANTypeConf: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
             int fd=0;
             write_csman(fd,0x00900301,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicIPv6PPPoEServiceName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIBasicIPv6PPPoEv6WANTypeConf not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UIBasicIPv6PPPoEv6WANTypeConf: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
             int fd=0;
             write_csman(fd,0x00900302,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicIPv6PPPoEMTU(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicIPv6PPPoEv6WANTypeConf not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicIPv6PPPoEv6WANTypeConf: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00900303,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicIPv6PPPoEMLDSnoopingEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicIPv6PPPoEv6WANTypeConf not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicIPv6PPPoEv6WANTypeConf: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00151010,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}

