#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIBasicNATBridgingConfiguration.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIBasicNATBridgingConfiguration_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,6,1 };
#else
oid UIBasicNATBridgingConfiguration_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,6,1 };
#endif

struct variable7 UIBasicNATBridgingConfiguration_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{BASICNATBRIDGNATLOOPBACKALTER,ASN_INTEGER,RWRITE,var_UIBasicNATBridgingConfiguration, 1,  { 1 }},
{BASICNATBRIDGNATLOOPBACKENABLE,ASN_INTEGER,RWRITE,var_UIBasicNATBridgingConfiguration, 1,  { 2 }},

};
/*    (L = length of the oidsuffix) */


/** Initializes the UIBasicNATBridgingConfiguration module */
void
init_UIBasicNATBridgingConfiguration(void)
{
    DEBUGMSGTL(("UIBasicNATBridgingConfiguration", "Initializing\n"));
    REGISTER_MIB("UIBasicNATBridgingConfiguration", UIBasicNATBridgingConfiguration_variables, variable7,
               UIBasicNATBridgingConfiguration_variables_oid);
}


unsigned char *
var_UIBasicNATBridgingConfiguration(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    switch(vp->magic) {
    case BASICNATBRIDGNATLOOPBACKALTER:
        *write_method = write_BasicNATBridgNATLoopbackAlter;
        read_csman(fd,0x80150002,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICNATBRIDGNATLOOPBACKENABLE:
        *write_method = write_BasicNATBridgNATLoopbackEnable;
        read_csman(fd,0x00150013,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}

int
write_BasicNATBridgNATLoopbackAlter(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;
	
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNATBridgingConfiguration not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNATBridgingConfiguration: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x80150002,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicNATBridgNATLoopbackEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNATBridgingConfiguration not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNATBridgingConfiguration: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00150013,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}

