#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIBasicNATBridgingDMZ.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIBasicNATBridgingDMZ_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,6,5 };
#else
oid UIBasicNATBridgingDMZ_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,6,5 };
#endif

struct variable7 UIBasicNATBridgingDMZ_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */


{BASICNATBRIDGDMZIPADDRESSOFHOST,ASN_IPADDRESS,RWRITE,var_UIBasicNATBridgingDMZ, 1,  { 1 }},
{BASICNATBRIDGDMZIPADDRESSOFHOSTENABLE,ASN_INTEGER,RWRITE,var_UIBasicNATBridgingDMZ, 1,  { 2 }},
{BASICNATBRIDGDMZRELAYDHCPENABLE,ASN_INTEGER,RWRITE,var_UIBasicNATBridgingDMZ, 1,  { 3 }},
{BASICNATBRIDGDMZRELAYDHCPADDRESS,ASN_IPADDRESS,RWRITE,var_UIBasicNATBridgingDMZ, 1,  { 4 }},

};

void
init_UIBasicNATBridgingDMZ(void)
{
    DEBUGMSGTL(("UIBasicNATBridgingDMZ", "Initializing\n"));
    REGISTER_MIB("UIBasicNATBridgingDMZ", UIBasicNATBridgingDMZ_variables, variable7,
               UIBasicNATBridgingDMZ_variables_oid);
}

unsigned char *
var_UIBasicNATBridgingDMZ(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICNATBRIDGDMZIPADDRESSOFHOST:
	{
		int flag=0;
		*write_method = write_BasicNATBridgDMZIPAddressofHost;
		read_csman(fd,0x001500a1,&flag,sizeof(unsigned int),TYPE_U32);
		memset(string,0,sizeof(string));
		
		if(flag == 0)	read_csman(fd,0x00150009,string,sizeof(string),TYPE_IPV4);
		else	read_csman(fd,0x001500c0,string,sizeof(string),TYPE_IPV4);
		
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
	}
    case BASICNATBRIDGDMZIPADDRESSOFHOSTENABLE:     
	{
		int flag=0;
        *write_method = write_BasicNATBridgDMZIPAddressofHostEnable;
		read_csman(fd,0x001500a1,&flag,sizeof(unsigned int),TYPE_U32);

		if(flag==0)	read_csman(fd,0x00150008 ,&long_ret,sizeof(unsigned int),TYPE_U32);
		else	read_csman(fd,0x001500b0 ,&long_ret,sizeof(unsigned int),TYPE_U32);
			
		return (unsigned char *) &long_ret;
    }
    case BASICNATBRIDGDMZRELAYDHCPENABLE:
        *write_method = write_BasicNATBridgDMZRelayDHCPEnable;
        read_csman(fd,0x000351E0,&long_ret,sizeof(unsigned int),TYPE_U32);	
		return (unsigned char *) &long_ret;
    case BASICNATBRIDGDMZRELAYDHCPADDRESS:
        *write_method = write_BasicNATBridgDMZRelayDHCPAddress;
        read_csman(fd,0x000351E1,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
    default:
      ERROR_MSG("");
    }
    return NULL;
}

int
write_BasicNATBridgDMZIPAddressofHost(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIBasicNATBridgingDMZ not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicNATBridgingDMZ: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			//for IPv4 data type
			memset(string,0,sizeof(string));
			memcpy(string , (char *) var_val, 4);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0,flag=0;
			read_csman(fd,0x001500a1,&flag,sizeof(unsigned int),TYPE_U32);

			if(flag==0)	write_csman(fd,0x00150009,string,strlen(string) ,TYPE_IPV4);
			else	write_csman(fd,0x001500c0,string,strlen(string) ,TYPE_IPV4);
			  
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicNATBridgDMZIPAddressofHostEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    long value;
    int size;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNATBridgingDMZ not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to UIBasicNATBridgingDMZ: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          size  = var_val_len;
          value = * (long *) var_val;

          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
          break;
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicNATBridgDMZRelayDHCPEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNATBridgingDMZ not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNATBridgingDMZ: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{	
			int fd=0;
			write_csman(fd,0x000351E0,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicNATBridgDMZRelayDHCPAddress(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];
	
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIBasicNATBridgingDMZ not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicNATBridgingDMZ: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			//for IPv4 data type
			memset(string,0,sizeof(string));
			memcpy(string , (char *) var_val, 4);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x000351E1,string,strlen(string) ,TYPE_IPV4);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}

