#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIBasicNATBridgingVirtualComputerList.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_BAS_VIRTUAL_COMPUTER	64
int w_index_bas_nat_vc;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIBasicNATBridgingVirtualComputerList_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,6,3 };
#else
oid UIBasicNATBridgingVirtualComputerList_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,6,3 };
#endif

struct variable7 UIBasicNATBridgingVirtualComputerList_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{BASICNATBRIDGVCINDEX,ASN_INTEGER,RONLY,var_BasicVCTable, 3,  { 1 , 1, 1 }},
{BASICNATBRIDGVCGLOBALIP,ASN_IPADDRESS,RWRITE,var_BasicVCTable, 3,  { 1 , 1, 2 }},
{BASICNATBRIDGVCLOCALIP,ASN_IPADDRESS,RWRITE,var_BasicVCTable, 3,  { 1 , 1, 3 }},
{BASICNATBRIDGVCENABLE,ASN_INTEGER,RWRITE,var_BasicVCTable, 3,  { 1 , 1, 4 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UIBasicNATBridgingVirtualComputerList module */
void
init_UIBasicNATBridgingVirtualComputerList(void)
{
    DEBUGMSGTL(("UIBasicNATBridgingVirtualComputerList", "Initializing\n"));
    REGISTER_MIB("UIBasicNATBridgingVirtualComputerList", UIBasicNATBridgingVirtualComputerList_variables, variable7,
               UIBasicNATBridgingVirtualComputerList_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}


unsigned char *
var_UIBasicNATBridgingVirtualComputerList(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}


unsigned char *
var_BasicVCTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;

	/* get index */
    register int index;
    index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_BAS_VIRTUAL_COMPUTER);
    w_index_bas_nat_vc=index;

    if ( index == MATCH_FAILED )
        return NULL;

    switch(vp->magic) {
    case BASICNATBRIDGVCINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case BASICNATBRIDGVCGLOBALIP:
        *write_method = write_BasicNATBridgVCGlobalIP;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00142000+index-1,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
    case BASICNATBRIDGVCLOCALIP:
        *write_method = write_BasicNATBridgVCLocalIP;
        memset(string,0,sizeof(string));
		 read_csman(fd,0x00143000+index-1,string,sizeof(string),TYPE_IPV4);
		 *var_len = 4; //IP address length = 4
		 return (unsigned char *) string;
    case BASICNATBRIDGVCENABLE:
        *write_method = write_BasicNATBridgVCEnable;
		read_csman(fd,0x00141000+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


int
write_BasicNATBridgVCGlobalIP(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
     static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIBasicNATBridgingVirtualComputerList not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicNATBridgingVirtualComputerList: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			memcpy(string , (char *) var_val, 4);
			break;	

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		  {
			  int fd=0;
			  write_csman(fd,0x00142000+w_index_bas_nat_vc-1,string,strlen(string) ,TYPE_IPV4);
			  break;
		  }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicNATBridgVCLocalIP(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIBasicNATBridgingVirtualComputerList not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicNATBridgingVirtualComputerList: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  //for IPv4 data type
			  memset(string,0,sizeof(string));
			  memcpy(string , (char *) var_val, 4);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			  int fd=0;
			  write_csman(fd,0x00143000+w_index_bas_nat_vc-1,string,strlen(string) ,TYPE_IPV4);
			  break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicNATBridgVCEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNATBridgingVirtualComputerList not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to UIBasicNATBridgingVirtualComputerList: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00141000+w_index_bas_nat_vc-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
