#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIBasicNATBridgingVirtualServerList.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_BAS_VIRTUAL_SERVER	64
int w_index_bas_nat_vs;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIBasicNATBridgingVirtualServerList_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,6,2 };
#else
oid UIBasicNATBridgingVirtualServerList_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,6,2 };
#endif

struct variable7 UIBasicNATBridgingVirtualServerList_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{BASICNATBRIDGVSINDEX,ASN_INTEGER,RONLY,var_BasicVSTable, 3,  { 1 , 1, 1 }},
{BASICNATBRIDGVSPUBLICPORT,ASN_OCTET_STR,RWRITE,var_BasicVSTable, 3,  { 1 , 1, 2 }},
{BASICNATBRIDGVSSERVERIP,ASN_IPADDRESS,RWRITE,var_BasicVSTable, 3,  { 1 , 1, 3 }},
{BASICNATBRIDGVSPRIVATEPORT,ASN_OCTET_STR,RWRITE,var_BasicVSTable, 3,  { 1 , 1, 4 }},
{BASICNATBRIDGVSPROTOCOL,ASN_INTEGER,RWRITE,var_BasicVSTable, 3,  { 1 , 1, 5 }},
{BASICNATBRIDGVSTIMESCHEDULERULE,ASN_INTEGER,RWRITE,var_BasicVSTable, 3,  { 1 , 1, 6 }},
{BASICNATBRIDGVSRULEENABLE,ASN_INTEGER,RWRITE,var_BasicVSTable, 3,  { 1 , 1, 7 }},
};
/*    (L = length of the oidsuffix) */


void
init_UIBasicNATBridgingVirtualServerList(void)
{
    DEBUGMSGTL(("UIBasicNATBridgingVirtualServerList", "Initializing\n"));
    REGISTER_MIB("UIBasicNATBridgingVirtualServerList", UIBasicNATBridgingVirtualServerList_variables, variable7,
               UIBasicNATBridgingVirtualServerList_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}


unsigned char *
var_UIBasicNATBridgingVirtualServerList(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_BasicVSTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UIBasicNATBridgingVirtualServerList above.
 */
unsigned char *
var_BasicVSTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    /* get index */
    register int index;
    index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_BAS_VIRTUAL_SERVER);
    w_index_bas_nat_vs=index;

    if ( index == MATCH_FAILED )
        return NULL;	

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICNATBRIDGVSINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case BASICNATBRIDGVSPUBLICPORT:
        *write_method = write_BasicNATBridgVSPublicPort;
        memset(string,0,sizeof(string));
		read_csman(fd,0x000c3000+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICNATBRIDGVSSERVERIP:
        *write_method = write_BasicNATBridgVSServerIP;
        memset(string,0,sizeof(string));
		read_csman(fd,0x000c2000+index-1,string,sizeof(string),TYPE_IPV4);
		*var_len = 4;
		return (unsigned char *) string;
    case BASICNATBRIDGVSPRIVATEPORT:
        *write_method = write_BasicNATBridgVSPrivatePort;
        memset(string,0,sizeof(string));
		read_csman(fd,0x000c5000+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICNATBRIDGVSPROTOCOL:
        *write_method = write_BasicNATBridgVSProtocol;
        read_csman(fd,0x000c6000+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICNATBRIDGVSTIMESCHEDULERULE:
        *write_method = write_BasicNATBridgVSTimeScheduleRule;
        read_csman(fd,0x000c4000+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICNATBRIDGVSRULEENABLE:
        *write_method = write_BasicNATBridgVSRuleEnable;
        read_csman(fd,0x000c1000+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


int
write_BasicNATBridgVSPublicPort(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIBasicNATBridgingVirtualServerList not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicNATBridgingVirtualServerList: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  //for STR/MAC data type
			  memset(string,0,sizeof(string));
			  strncpy(string , (char *) var_val, var_val_len);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
             int fd;
             write_csman(fd,0x000c3000+w_index_bas_nat_vs-1,string,strlen(string)+1,TYPE_STR);
             break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicNATBridgVSServerIP(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIBasicNATBridgingVirtualServerList not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicNATBridgingVirtualServerList: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			memcpy(string , (char *) var_val, 4);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		  {
			  int fd;
			  write_csman(fd,0x000c2000+w_index_bas_nat_vs-1,string,strlen(string) ,TYPE_IPV4);
			  break;
		  }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicNATBridgVSPrivatePort(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIBasicNATBridgingVirtualServerList not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicNATBridgingVirtualServerList: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
             int fd=0;
             write_csman(fd,0x000c5000+w_index_bas_nat_vs-1,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicNATBridgVSProtocol(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNATBridgingVirtualServerList not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNATBridgingVirtualServerList: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
              int fd=0;
              write_csman(fd,0x000c6000+w_index_bas_nat_vs-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicNATBridgVSTimeScheduleRule(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNATBridgingVirtualServerList not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNATBridgingVirtualServerList: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
              int fd=0;
              write_csman(fd,0x000c4000+w_index_bas_nat_vs-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicNATBridgVSRuleEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNATBridgingVirtualServerList not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNATBridgingVirtualServerList: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x000c1000+w_index_bas_nat_vs-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
