#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIBasicNetworkLoadBalance.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_LOADBALANCE_USER	256
int w_index_wan_loadbalance_user;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIBasicNetworkLoadBalance_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,1,16 };
#else
oid UIBasicNetworkLoadBalance_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,1,16 };
#endif

struct variable7 UIBasicNetworkLoadBalance_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */


{BASICLOADBALANCEALTER,ASN_INTEGER,RWRITE,var_UIBasicNetworkLoadBalance, 1,  { 1 }},
{BASICLOADBALANCEENABLE,ASN_INTEGER,RWRITE,var_UIBasicNetworkLoadBalance, 1,  { 2 }},
{BASICLOADBALANCEPRIORITYWAN1,ASN_INTEGER,RWRITE,var_UIBasicNetworkLoadBalance, 1,  { 3 }},
{BASICLOADBALANCEPRIORITYWAN2,ASN_INTEGER,RWRITE,var_UIBasicNetworkLoadBalance, 1,  { 4 }},
{BASICLOADBALANCEPRIORITYWAN3,ASN_INTEGER,RWRITE,var_UIBasicNetworkLoadBalance, 1,  { 5 }},

{BASICLOADBALANCEUSERPRIORITYINDEX,ASN_INTEGER,RONLY,var_BasicLoadBalanceUserPriorityTable, 3,  { 10 , 1, 1 }},
{BASICLOADBALANCEUSERSOURCEIPTYPE,ASN_INTEGER,RWRITE,var_BasicLoadBalanceUserPriorityTable, 3,  { 10 , 1, 2 }},
{BASICLOADBALANCEUSERSOURCEIPSUBNET,ASN_OCTET_STR,RWRITE,var_BasicLoadBalanceUserPriorityTable, 3,  { 10 , 1, 3 }},
{BASICLOADBALANCEUSERSOURCEIPRANGESTART,ASN_OCTET_STR,RWRITE,var_BasicLoadBalanceUserPriorityTable, 3,  { 10 , 1, 4 }},
{BASICLOADBALANCEUSERSOURCEIPRANGEEND,ASN_OCTET_STR,RWRITE,var_BasicLoadBalanceUserPriorityTable, 3,  { 10 , 1, 5 }},
{BASICLOADBALANCEUSERSOURCEIPSINGLE,ASN_OCTET_STR,RWRITE,var_BasicLoadBalanceUserPriorityTable, 3,  { 10 , 1, 6 }},
{BASICLOADBALANCEUSERDESTINATIONIPTYPE,ASN_INTEGER,RWRITE,var_BasicLoadBalanceUserPriorityTable, 3,  { 10 , 1, 7 }},
{BASICLOADBALANCEUSERDESTINATIONIPSUBNET,ASN_OCTET_STR,RWRITE,var_BasicLoadBalanceUserPriorityTable, 3,  { 10 , 1, 8 }},
{BASICLOADBALANCEUSERDESTINATIONIPRANGESTART,ASN_OCTET_STR,RWRITE,var_BasicLoadBalanceUserPriorityTable, 3,  { 10 , 1, 9 }},
{BASICLOADBALANCEUSERDESTINATIONIPRANGEEND,ASN_OCTET_STR,RWRITE,var_BasicLoadBalanceUserPriorityTable, 3,  { 10 , 1, 10 }},
{BASICLOADBALANCEUSERDESTINATIONIPSINGLE,ASN_OCTET_STR,RWRITE,var_BasicLoadBalanceUserPriorityTable, 3,  { 10 , 1, 11 }},
{BASICLOADBALANCEUSERDESTINATIONIPDOMAINNAME,ASN_OCTET_STR,RWRITE,var_BasicLoadBalanceUserPriorityTable, 3,  { 10 , 1, 12 }},
{BASICLOADBALANCEUSERDESTINATIONPORTTYPE,ASN_INTEGER,RWRITE,var_BasicLoadBalanceUserPriorityTable, 3,  { 10 , 1, 13 }},
{BASICLOADBALANCEUSERDESTINATIONPORTRANGESTART,ASN_INTEGER,RWRITE,var_BasicLoadBalanceUserPriorityTable, 3,  { 10 , 1, 14 }},
{BASICLOADBALANCEUSERDESTINATIONPORTRANGEEND,ASN_INTEGER,RWRITE,var_BasicLoadBalanceUserPriorityTable, 3,  { 10 , 1, 15 }},
{BASICLOADBALANCEUSERDESTINATIONPORTSINGLE,ASN_INTEGER,RWRITE,var_BasicLoadBalanceUserPriorityTable, 3,  { 10 , 1, 16 }},
{BASICLOADBALANCEUSERPROTOCOL,ASN_INTEGER,RWRITE,var_BasicLoadBalanceUserPriorityTable, 3,  { 10 , 1, 17 }},
{BASICLOADBALANCEUSERWANINTERFACE,ASN_INTEGER,RWRITE,var_BasicLoadBalanceUserPriorityTable, 3,  { 10 , 1, 18 }},
{BASICLOADBALANCEUSERPOLICYENABLE,ASN_INTEGER,RWRITE,var_BasicLoadBalanceUserPriorityTable, 3,  { 10 , 1, 19 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UIBasicNetworkLoadBalance module */
void
init_UIBasicNetworkLoadBalance(void)
{
    DEBUGMSGTL(("UIBasicNetworkLoadBalance", "Initializing\n"));
    REGISTER_MIB("UIBasicNetworkLoadBalance", UIBasicNetworkLoadBalance_variables, variable7,
               UIBasicNetworkLoadBalance_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif

	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UIBasicNetworkLoadBalance(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICLOADBALANCEALTER:
        *write_method = write_BasicLoadBalanceAlter;
      	long_ret=0;
		return (unsigned char *) &long_ret;
    case BASICLOADBALANCEENABLE:
        *write_method = write_BasicLoadBalanceEnable;
        read_csman(fd,0x0001000E,&long_ret,sizeof(unsigned int),TYPE_U32);
		if(long_ret > 1)	long_ret=1;
		return (unsigned char *) &long_ret;
    case BASICLOADBALANCEPRIORITYWAN1:
	{	
		int wan_index=0;	
        *write_method = write_BasicLoadBalancePriorityWAN1;
		read_csman(fd,0x00010600,&wan_index,sizeof(unsigned int),TYPE_U32);
		if(wan_index == 99)
        	read_csman(fd,0x0001000C,&long_ret,sizeof(unsigned int),TYPE_U32);
		else{
			read_csman(fd,0x00010130+wan_index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		}
		return (unsigned char *) &long_ret;
    }
    case BASICLOADBALANCEPRIORITYWAN2:
	{
		int wan_index=0;
        *write_method = write_BasicLoadBalancePriorityWAN2;
        read_csman(fd,0x00010601,&wan_index,sizeof(unsigned int),TYPE_U32);
		if(wan_index == 99)
        	read_csman(fd,0x0001000C,&long_ret,sizeof(unsigned int),TYPE_U32);
		else{
			read_csman(fd,0x00010130+wan_index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		}
		return (unsigned char *) &long_ret;
    }
    case BASICLOADBALANCEPRIORITYWAN3:
	{
		int wan_index=0;
        *write_method = write_BasicLoadBalancePriorityWAN3;
		read_csman(fd,0x00010602,&wan_index,sizeof(unsigned int),TYPE_U32);
		if(wan_index == 99)
        	read_csman(fd,0x0001000C,&long_ret,sizeof(unsigned int),TYPE_U32);
		else{
			read_csman(fd,0x00010130+wan_index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		}
        read_csman(fd,0x00190300,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    }
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_BasicLoadBalanceUserPriorityTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UIBasicNetworkLoadBalance above.
 */
unsigned char *
var_BasicLoadBalanceUserPriorityTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    register int index;
    index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_LOADBALANCE_USER);
    w_index_wan_loadbalance_user=index;

    if ( index == MATCH_FAILED )
        return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICLOADBALANCEUSERPRIORITYINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case BASICLOADBALANCEUSERSOURCEIPTYPE:
        *write_method = write_BasicLoadBalanceUserSourceIPType;
        read_csman(fd,0x00134000+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
    case BASICLOADBALANCEUSERSOURCEIPSUBNET:
        *write_method = write_BasicLoadBalanceUserSourceIPSubnet;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00130400+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICLOADBALANCEUSERSOURCEIPRANGESTART:
        *write_method = write_BasicLoadBalanceUserSourceIPRangeStart;
       	memset(string,0,sizeof(string));
		read_csman(fd,0x00130400+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICLOADBALANCEUSERSOURCEIPRANGEEND:
        *write_method = write_BasicLoadBalanceUserSourceIPRangeEnd;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00130500+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICLOADBALANCEUSERSOURCEIPSINGLE:
        *write_method = write_BasicLoadBalanceUserSourceIPSingle;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00130400+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICLOADBALANCEUSERDESTINATIONIPTYPE:
        *write_method = write_BasicLoadBalanceUserDestinationIPType;
        read_csman(fd,0x00134100+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
    case BASICLOADBALANCEUSERDESTINATIONIPSUBNET:
        *write_method = write_BasicLoadBalanceUserDestinationIPSubnet;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00130600+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICLOADBALANCEUSERDESTINATIONIPRANGESTART:
        *write_method = write_BasicLoadBalanceUserDestinationIPRangeStart;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00130600+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICLOADBALANCEUSERDESTINATIONIPRANGEEND:
        *write_method = write_BasicLoadBalanceUserDestinationIPRangeEnd;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00130700+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICLOADBALANCEUSERDESTINATIONIPSINGLE:
        *write_method = write_BasicLoadBalanceUserDestinationIPSingle;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00130600+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICLOADBALANCEUSERDESTINATIONIPDOMAINNAME:
        *write_method = write_BasicLoadBalanceUserDestinationIPDomainName;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00130600+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICLOADBALANCEUSERDESTINATIONPORTTYPE:
        *write_method = write_BasicLoadBalanceUserDestinationPortType;
        read_csman(fd,0x00134200+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
    case BASICLOADBALANCEUSERDESTINATIONPORTRANGESTART:
        *write_method = write_BasicLoadBalanceUserDestinationPortRangeStart;
        read_csman(fd,0x00130800+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
    case BASICLOADBALANCEUSERDESTINATIONPORTRANGEEND:
        *write_method = write_BasicLoadBalanceUserDestinationPortRangeEnd;
        read_csman(fd,0x00130900+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
    case BASICLOADBALANCEUSERDESTINATIONPORTSINGLE:
        *write_method = write_BasicLoadBalanceUserDestinationPortSingle;
        read_csman(fd,0x00130800+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
    case BASICLOADBALANCEUSERPROTOCOL:
        *write_method = write_BasicLoadBalanceUserProtocol;
        read_csman(fd,0x00130200+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
    case BASICLOADBALANCEUSERWANINTERFACE:
        *write_method = write_BasicLoadBalanceUserWANInterface;
        read_csman(fd,0x00130300+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
    case BASICLOADBALANCEUSERPOLICYENABLE:
        *write_method = write_BasicLoadBalanceUserPolicyEnable;
        read_csman(fd,0x00130100+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}



int
write_BasicLoadBalanceAlter(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x8001004a,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicLoadBalanceEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
			long_ret = *(unsigned int *) var_val;
			break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x0001000E,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicLoadBalancePriorityWAN1(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0,wan_index=0;
			unsigned long csid_address=0;
			read_csman(fd,0x00010600,&wan_index,sizeof(unsigned int),TYPE_U32);
			if(wan_index == 99)
        		write_csman(fd,0x00190008,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			else{
				/*CSID_C_LOCAL_MULTIWAN_WEIGHTB0x00010130Bu32*/
				write_csman(fd,0x00010130+wan_index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
			}
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicLoadBalancePriorityWAN2(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0,wan_index=0;
			unsigned long csid_address=0;
			read_csman(fd,0x00010601,&wan_index,sizeof(unsigned int),TYPE_U32);
			if(wan_index == 99)
        		write_csman(fd,0x00190008,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			else{
				/*CSID_C_LOCAL_MULTIWAN_WEIGHTB0x00010130Bu32*/
				write_csman(fd,0x00010130+wan_index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
			}
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicLoadBalancePriorityWAN3(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0,wan_index=0;
			unsigned long csid_address=0;
			read_csman(fd,0x00010602,&wan_index,sizeof(unsigned int),TYPE_U32);
			if(wan_index == 99)
        		write_csman(fd,0x00190008,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			else{
				/*CSID_C_LOCAL_MULTIWAN_WEIGHTB0x00010130Bu32*/
				write_csman(fd,0x00010130+wan_index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
			}
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}

int
write_BasicLoadBalanceUserSourceIPType(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			unsigned long csid_address=0;
			char cmd_buf[256];
        	write_csman(fd,0x00134000+w_index_wan_loadbalance_user-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			/*disclean CSID_C_MULTIROUTE_SRCIP_END*/
			if(long_ret == 1 || long_ret == 3){
				csid_address=1246464;
				sprintf(cmd_buf,"rmcsman 0x%x",(csid_address+w_index_wan_loadbalance_user-1));
				system(cmd_buf);
			}
			break;
        }

    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicLoadBalanceUserSourceIPSubnet(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
             int fd=0;
             write_csman(fd,0x00130400+w_index_wan_loadbalance_user,string,strlen(string)+1,TYPE_STR);
             break;
	  	}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicLoadBalanceUserSourceIPRangeStart(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
             int fd=0;
             write_csman(fd,0x00130400+w_index_wan_loadbalance_user,string,strlen(string)+1,TYPE_STR);
             break;
	  	}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicLoadBalanceUserSourceIPRangeEnd(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
             int fd=0;
             write_csman(fd,0x00130500+w_index_wan_loadbalance_user,string,strlen(string)+1,TYPE_STR);
             break;
	  	}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicLoadBalanceUserSourceIPSingle(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
             int fd=0;
             write_csman(fd,0x00130400+w_index_wan_loadbalance_user,string,strlen(string)+1,TYPE_STR);
             break;
	  	}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicLoadBalanceUserDestinationIPType(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			unsigned long csid_address=0;
			char cmd_buf[256];
        	write_csman(fd,0x00134100+w_index_wan_loadbalance_user-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			/*disclean CSID_C_MULTIROUTE_SRCIP_END*/
			if(long_ret == 1 || long_ret == 3){
				csid_address=1246976;
				sprintf(cmd_buf,"rmcsman 0x%x",(csid_address+w_index_wan_loadbalance_user-1));
				system(cmd_buf);
			}
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicLoadBalanceUserDestinationIPSubnet(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          	memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00130600+w_index_wan_loadbalance_user,string,strlen(string)+1,TYPE_STR);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicLoadBalanceUserDestinationIPRangeStart(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          	memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00130600+w_index_wan_loadbalance_user,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicLoadBalanceUserDestinationIPRangeEnd(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          	memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00130700+w_index_wan_loadbalance_user,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicLoadBalanceUserDestinationIPSingle(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          	memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00130600+w_index_wan_loadbalance_user,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicLoadBalanceUserDestinationIPDomainName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00130600+w_index_wan_loadbalance_user,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicLoadBalanceUserDestinationPortType(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
 		{
			int fd=0;
			unsigned long csid_address=0;
			char cmd_buf[256];
			write_csman(fd,0x00134200+w_index_wan_loadbalance_user-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			/*disclean CSID_C_MULTIROUTE_DSTPORT_END*/
			if(long_ret == 2){
				csid_address=1247488;
				sprintf(cmd_buf,"rmcsman 0x%x",(csid_address+w_index_wan_loadbalance_user-1));
				system(cmd_buf);
			}
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicLoadBalanceUserDestinationPortRangeStart(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00130800+w_index_wan_loadbalance_user-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicLoadBalanceUserDestinationPortRangeEnd(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00130900+w_index_wan_loadbalance_user-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicLoadBalanceUserDestinationPortSingle(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00130800+w_index_wan_loadbalance_user-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicLoadBalanceUserProtocol(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00130200+w_index_wan_loadbalance_user-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicLoadBalanceUserWANInterface(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00130300+w_index_wan_loadbalance_user-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicLoadBalanceUserPolicyEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicNetworkLoadBalance not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicNetworkLoadBalance: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00130100+w_index_wan_loadbalance_user-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
