#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIBasicRoutingDynamicBGP.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_BAS_ROUTING_DYNAMIC_BGP	8
int w_index_bas_routing_dynamic_bgp;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIBasicRoutingDynamicBGP_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,7,4 };
#else
oid UIBasicRoutingDynamicBGP_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,7,4 };
#endif

struct variable7 UIBasicRoutingDynamicBGP_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{BASICROUTINGDRBGPENABLE,ASN_INTEGER,RWRITE,var_UIBasicRoutingDynamicBGP, 1,  { 1 }},
{BASICROUTINGDRBGPSELFID,ASN_INTEGER,RWRITE,var_UIBasicRoutingDynamicBGP, 1,  { 2 }},
{BASICROUTINGDRBGPINDEX,ASN_INTEGER,RONLY,var_BasicDynamicRoutingBGPTable, 3,  { 3 , 1, 1 }},
{BASICROUTINGDRBGPNEIGHBORIP,ASN_IPADDRESS,RWRITE,var_BasicDynamicRoutingBGPTable, 3,  { 3 , 1, 2 }},
{BASICROUTINGDRBGPNEIGHBORID,ASN_INTEGER,RWRITE,var_BasicDynamicRoutingBGPTable, 3,  { 3 , 1, 3 }},
{BASICROUTINGDRBGPINDEXENABLE,ASN_INTEGER,RWRITE,var_BasicDynamicRoutingBGPTable, 3,  { 3 , 1, 4 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UIBasicRoutingDynamicBGP module */
void
init_UIBasicRoutingDynamicBGP(void)
{
    DEBUGMSGTL(("UIBasicRoutingDynamicBGP", "Initializing\n"));
    REGISTER_MIB("UIBasicRoutingDynamicBGP", UIBasicRoutingDynamicBGP_variables, variable7,
               UIBasicRoutingDynamicBGP_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{

	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif

	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UIBasicRoutingDynamicBGP(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICROUTINGDRBGPENABLE:
        *write_method = write_BasicRoutingDRBGPEnable;
        read_csman(fd,0x00130050,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICROUTINGDRBGPSELFID:
        *write_method = write_BasicRoutingDRBGPSelfID;
        read_csman(fd,0x00130051,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_BasicDynamicRoutingBGPTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UIBasicRoutingDynamicBGP above.
 */
unsigned char *
var_BasicDynamicRoutingBGPTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    /* get index */
    register int index;
    index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_BAS_ROUTING_DYNAMIC_BGP);
    w_index_bas_routing_dynamic_bgp=index;

    if ( index == MATCH_FAILED )
        return NULL;


    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICROUTINGDRBGPINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case BASICROUTINGDRBGPNEIGHBORIP:
	{
		*write_method = write_BasicRoutingDRBGPNeighBorIP;
		unsigned char bgp_neighbor_ip[4];
		memset(bgp_neighbor_ip,0,sizeof(bgp_neighbor_ip));
		read_csman(fd,0x00133100+index-1,bgp_neighbor_ip,sizeof(bgp_neighbor_ip),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) bgp_neighbor_ip;
	}
    case BASICROUTINGDRBGPNEIGHBORID:
	{
        *write_method = write_BasicRoutingDRBGPNeighBorID;
        unsigned int bgp_neighbor_id;
		read_csman(fd,0x00133200+index-1,&bgp_neighbor_id,sizeof(unsigned int),TYPE_U32);
		long_ret = bgp_neighbor_id;
	    return (unsigned char *) &long_ret;
    }
    case BASICROUTINGDRBGPINDEXENABLE:
	{
        *write_method = write_BasicRoutingDRBGPIndexEnable;
		unsigned char bgp_table_enable;
		read_csman(fd,0x00133000+index-1,&bgp_table_enable,sizeof(unsigned int),TYPE_U32);
		long_ret = bgp_table_enable;
		return (unsigned char *) &long_ret;
    }
    default:
      ERROR_MSG("");
    }
    return NULL;
}



int
write_BasicRoutingDRBGPEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicRoutingDynamicBGP not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicRoutingDynamicBGP: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;


        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00130050,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicRoutingDRBGPSelfID(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicRoutingDynamicBGP not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicRoutingDynamicBGP: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00130051,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}

int
write_BasicRoutingDRBGPNeighBorIP(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS ) {
              fprintf(stderr, "write to UIBasicRoutingDynamicBGP not \n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicRoutingDynamicBGP: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			memcpy(string , (char *) var_val, 4);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00133100+w_index_bas_routing_dynamic_bgp-1,string,strlen(string) ,TYPE_IPV4);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicRoutingDRBGPNeighBorID(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicRoutingDynamicBGP not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicRoutingDynamicBGP: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00133200+w_index_bas_routing_dynamic_bgp-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicRoutingDRBGPIndexEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicRoutingDynamicBGP not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicRoutingDynamicBGP: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00133000+w_index_bas_routing_dynamic_bgp-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
