#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIBasicRoutingInformation.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIBasicRoutingInformation_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,7,5 };
#else
oid UIBasicRoutingInformation_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,7,5 };
#endif

struct variable7 UIBasicRoutingInformation_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */


{BASICROUTINGINFORMATIONINDEX,ASN_INTEGER,RONLY,var_BasicRoutingInfoTable, 3,  { 1 , 1, 1 }},
{BASICROUTINGINFORMATIONDESTINATION,ASN_OCTET_STR,RONLY,var_BasicRoutingInfoTable, 3,  { 1 , 1, 2 }},
{BASICROUTINGINFORMATIONGATEWAY,ASN_OCTET_STR,RONLY,var_BasicRoutingInfoTable, 3,  { 1 , 1, 3 }},
{BASICROUTINGINFORMATIONSUBNETMASK,ASN_OCTET_STR,RONLY,var_BasicRoutingInfoTable, 3,  { 1 , 1, 4 }},
{BASICROUTINGINFORMATIONMETRIC,ASN_INTEGER,RONLY,var_BasicRoutingInfoTable, 3,  { 1 , 1, 5 }},
{BASICROUTINGINFORMATIONINTERFACE,ASN_OCTET_STR,RONLY,var_BasicRoutingInfoTable, 3,  { 1 , 1, 6 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UIBasicRoutingInformation module */
void
init_UIBasicRoutingInformation(void)
{
    DEBUGMSGTL(("UIBasicRoutingInformation", "Initializing\n"));
    /* register ourselves with the agent to handle our mib tree */
    REGISTER_MIB("UIBasicRoutingInformation", UIBasicRoutingInformation_variables, variable7,
               UIBasicRoutingInformation_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}


unsigned char *
var_UIBasicRoutingInformation(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_BasicRoutingInfoTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UIBasicRoutingInformation above.
 */
unsigned char *
var_BasicRoutingInfoTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0,route_number=0;

	/* get index */
    register int index;
	/*Get Routing Information Table*/
	system("get_route_info");
	/* The really index is following : total / number of table */
	read_csman(fd,0x80131500,&route_number,sizeof(unsigned int),TYPE_U32);

	if(route_number == 0)
		route_number = 1 ;
	
    index = header_Entry(vp, name, length, exact, var_len, write_method,route_number);

    if ( index == MATCH_FAILED )
        return NULL;
    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICROUTINGINFORMATIONINDEX:
        long_ret = index;
        return (unsigned char *) &long_ret;
    case BASICROUTINGINFORMATIONDESTINATION:
	{
		memset(string,0,sizeof(string));
	   	read_csman(fd,0x80131000+index-1,string,sizeof(string),TYPE_STR);
	   	*var_len = strlen(string)-1;
	   	return (unsigned char *) string;
	}
    case BASICROUTINGINFORMATIONGATEWAY:
	{
		memset(string,0,sizeof(string));
	   	read_csman(fd,0x80131100+index-1,string,sizeof(string),TYPE_STR);
	   	*var_len = strlen(string)-1;
	   	return (unsigned char *) string;
	}	
    case BASICROUTINGINFORMATIONSUBNETMASK:
	{
		memset(string,0,sizeof(string));
	   	read_csman(fd,0x80131200+index-1,string,sizeof(string),TYPE_STR);
	   	*var_len = strlen(string)-1;
	   	return (unsigned char *) string;
	}	
    case BASICROUTINGINFORMATIONMETRIC:
	{
		read_csman(fd,0x80131300+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    }
    case BASICROUTINGINFORMATIONINTERFACE:
 	{
		memset(string,0,sizeof(string));
	   	read_csman(fd,0x80131400+index-1,string,sizeof(string),TYPE_STR);
	   	*var_len = strlen(string)-1;
	   	return (unsigned char *) string;
    }
    default:
      ERROR_MSG("");
    }
    return NULL;
}
