#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIBasicRoutingStatic.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_BAS_ROUTING_STATIC	32
int w_index_bas_routing_static;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIBasicRoutingStatic_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,7,1 };
#else
oid UIBasicRoutingStatic_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,7,1 };
#endif

struct variable7 UIBasicRoutingStatic_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{BASICROUTINGSTATICROUTINGENABLE,ASN_INTEGER,RWRITE,var_UIBasicRoutingStatic, 1,  { 1 }},
{BASICROUTINGSTATICROUTINGINDEX,ASN_INTEGER,RONLY,var_BasicRoutingStaticRoutingTable, 3,  { 2 , 1, 1 }},
{BASICROUTINGSTATICROUTINGDESTINATIONIP,ASN_IPADDRESS,RWRITE,var_BasicRoutingStaticRoutingTable, 3,  { 2 , 1, 2 }},
{BASICROUTINGSTATICROUTINGSUBNETMASK,ASN_IPADDRESS,RWRITE,var_BasicRoutingStaticRoutingTable, 3,  { 2 , 1, 3 }},
{BASICROUTINGSTATICROUTINGGATEWAYIP,ASN_IPADDRESS,RWRITE,var_BasicRoutingStaticRoutingTable, 3,  { 2 , 1, 4 }},
{BASICROUTINGSTATICROUTINGMETRIC,ASN_INTEGER,RWRITE,var_BasicRoutingStaticRoutingTable, 3,  { 2 , 1, 5 }},
{BASICROUTINGSTATICROUTINGINDEXENABLE,ASN_INTEGER,RWRITE,var_BasicRoutingStaticRoutingTable, 3,  { 2 , 1, 6 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UIBasicRoutingStatic module */
void
init_UIBasicRoutingStatic(void)
{
    DEBUGMSGTL(("UIBasicRoutingStatic", "Initializing\n"));
    REGISTER_MIB("UIBasicRoutingStatic", UIBasicRoutingStatic_variables, variable7,
               UIBasicRoutingStatic_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{

	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif

	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UIBasicRoutingStatic(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICROUTINGSTATICROUTINGENABLE:
        *write_method = write_BasicRoutingStaticRoutingEnable;
        read_csman(fd,0x00130002,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


unsigned char *
var_BasicRoutingStaticRoutingTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    /* get index */
    register int index;
    index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_BAS_ROUTING_STATIC);
    w_index_bas_routing_static=index;

    if ( index == MATCH_FAILED )
        return NULL;


    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICROUTINGSTATICROUTINGINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case BASICROUTINGSTATICROUTINGDESTINATIONIP:
        *write_method = write_BasicRoutingStaticRoutingDestinationIP;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00131100+index-1,string,sizeof(string),TYPE_IPV4);
		*var_len = 4;
		return (unsigned char *) string;
    case BASICROUTINGSTATICROUTINGSUBNETMASK:
        *write_method = write_BasicRoutingStaticRoutingSubnetMask;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00131200+index-1,string,sizeof(string),TYPE_IPV4);
		*var_len = 4;
		return (unsigned char *) string;
    case BASICROUTINGSTATICROUTINGGATEWAYIP:
        *write_method = write_BasicRoutingStaticRoutingGateWayIP;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00131300+index-1,string,sizeof(string),TYPE_IPV4);
		*var_len = 4;
		return (unsigned char *) string;
    case BASICROUTINGSTATICROUTINGMETRIC:
        *write_method = write_BasicRoutingStaticRoutingMetric;
        read_csman(fd,0x00131400+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICROUTINGSTATICROUTINGINDEXENABLE:
        *write_method = write_BasicRoutingStaticRoutingIndexEnable;
        read_csman(fd,0x00131000+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}



int
write_BasicRoutingStaticRoutingEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicRoutingStatic not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicRoutingStatic: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
            int fd=0;
            write_csman(fd,0x00130002,&long_ret,sizeof(unsigned int) ,TYPE_U32);
            break;
        }
    }
    return SNMP_ERR_NOERROR;
}

int
write_BasicRoutingStaticRoutingDestinationIP(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIBasicRoutingStatic not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicRoutingStatic: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			memcpy(string , (char *) var_val, 4);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00131100+w_index_bas_routing_static-1,string,strlen(string) ,TYPE_IPV4);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicRoutingStaticRoutingSubnetMask(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
     static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIBasicRoutingStatic not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicRoutingStatic: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			memcpy(string , (char *) var_val, 4);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
		 	int fd=0;
			write_csman(fd,0x00131200+w_index_bas_routing_static-1,string,strlen(string) ,TYPE_IPV4);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicRoutingStaticRoutingGateWayIP(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIBasicRoutingStatic not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicRoutingStatic: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  memcpy(string , (char *) var_val, 4);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		  {
			  int fd;
			  write_csman(fd,0x00131300+w_index_bas_routing_static-1,string,strlen(string) ,TYPE_IPV4);
			  break;
		  }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicRoutingStaticRoutingMetric(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicRoutingStatic not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicRoutingStatic: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00131400+w_index_bas_routing_static-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicRoutingStaticRoutingIndexEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicRoutingStatic not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicRoutingStatic: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00131000+w_index_bas_routing_static-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
