#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIBasicVLANTagBaseVLANList.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIBasicVLANTagBaseVLANList_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,2,8 };
#else
oid UIBasicVLANTagBaseVLANList_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,2,8 };
#endif

#define NUM_BAS_VLAN_TB_ETHERNET	32
int w_index_bas_lan_tb_ethernet;

struct variable7 UIBasicVLANTagBaseVLANList_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{BASICVLANTBVLANINDEX,ASN_INTEGER,RONLY,var_BasicVLANTBVLANIndexTable, 3,  { 1 , 1, 1 }},
{BASICVLANTBVLANID,ASN_INTEGER,RWRITE,var_BasicVLANTBVLANIndexTable, 3,  { 1 , 1, 2 }},
{BASICVLANTBPORTENABLE,ASN_INTEGER,RWRITE,var_BasicVLANTBVLANIndexTable, 3,  { 1 , 1, 3 }},
{BASICVLANTBVIRTUALAPENABLE,ASN_INTEGER,RWRITE,var_BasicVLANTBVLANIndexTable, 3,  { 1 , 1, 4 }},
{BASICVLANTBVIRTUALAP2ENABLE,ASN_INTEGER,RWRITE,var_BasicVLANTBVLANIndexTable, 3,  { 1 , 1, 5 }},
{BASICVLANTBDHCPSERVER,ASN_INTEGER,RWRITE,var_BasicVLANTBVLANIndexTable, 3,  { 1 , 1, 6 }},
{BASICVLANTBINTERNET,ASN_INTEGER,RWRITE,var_BasicVLANTBVLANIndexTable, 3,  { 1 , 1, 7 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UIBasicVLANTagBaseVLANList module */
void
init_UIBasicVLANTagBaseVLANList(void)
{
    DEBUGMSGTL(("UIBasicVLANTagBaseVLANList", "Initializing\n"));
    /* register ourselves with the agent to handle our mib tree */
    REGISTER_MIB("UIBasicVLANTagBaseVLANList", UIBasicVLANTagBaseVLANList_variables, variable7,
               UIBasicVLANTagBaseVLANList_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{

	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif

	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UIBasicVLANTagBaseVLANList(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}



unsigned char *
var_BasicVLANTBVLANIndexTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    /* get index */
    register int index;
    index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_BAS_VLAN_TB_ETHERNET);
    w_index_bas_lan_tb_ethernet=index;

    if ( index == MATCH_FAILED )
        return NULL;


    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICVLANTBVLANINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case BASICVLANTBVLANID:
        *write_method = write_BasicVLANTBVLANID;
        read_csman(fd,0x00932400+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICVLANTBPORTENABLE:
        *write_method = write_BasicVLANTBPortEnable;
        read_csman(fd,0x00932500+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICVLANTBVIRTUALAPENABLE:
        *write_method = write_BasicVLANTBVirtualAPEnable;
        read_csman(fd,0x00932600+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICVLANTBVIRTUALAP2ENABLE:
        *write_method = write_BasicVLANTBVirtualAP2Enable;
        read_csman(fd,0x00932700+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICVLANTBDHCPSERVER:
        *write_method = write_BasicVLANTBDHCPServer;
        read_csman(fd,0x00932480+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICVLANTBINTERNET:
        *write_method = write_BasicVLANTBInternet;
        read_csman(fd,0x00932900+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


int
write_BasicVLANTBVLANID(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;
	
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicVLANTagBaseVLANList not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicVLANTagBaseVLANList: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
  			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
            int fd=0;
            write_csman(fd,0x00932400+w_index_bas_lan_tb_ethernet-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
            break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicVLANTBPortEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicVLANTagBaseVLANList not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicVLANTagBaseVLANList: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
  			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
            int fd=0;
            write_csman(fd,0x00932500+w_index_bas_lan_tb_ethernet-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
            break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicVLANTBVirtualAPEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicVLANTagBaseVLANList not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicVLANTagBaseVLANList: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
  			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
            int fd=0;
            write_csman(fd,0x00932600+w_index_bas_lan_tb_ethernet-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
            break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicVLANTBVirtualAP2Enable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicVLANTagBaseVLANList not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicVLANTagBaseVLANList: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
  			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
            int fd=0;
            write_csman(fd,0x00932700+w_index_bas_lan_tb_ethernet-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
            break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicVLANTBDHCPServer(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicVLANTagBaseVLANList not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicVLANTagBaseVLANList: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
  			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
            int fd=0;
            write_csman(fd,0x00932480+w_index_bas_lan_tb_ethernet-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
            break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicVLANTBInternet(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicVLANTagBaseVLANList not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicVLANTagBaseVLANList: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
  			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
            int fd=0;
            write_csman(fd,0x00932900+w_index_bas_lan_tb_ethernet-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
            break;
        }
    }
    return SNMP_ERR_NOERROR;
}
