#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIBasicWiFi50ClientList.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIBasicWiFi50ClientList_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,4,4 };
#else
oid UIBasicWiFi50ClientList_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,4,4 };
#endif

struct variable7 UIBasicWiFi50ClientList_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{BASICWIFI50CLIENTLISTINDEX,ASN_INTEGER,RONLY,var_BasicWiFi50ClientListTable, 3,  { 1 , 1, 1 }},
{BASICWIFI50CLIENTLISTIPADDRESS,ASN_IPADDRESS,RONLY,var_BasicWiFi50ClientListTable, 3,  { 1 , 1, 2 }},
{BASICWIFI50CLIENTLISTHOSTNAME,ASN_OCTET_STR,RONLY,var_BasicWiFi50ClientListTable, 3,  { 1 , 1, 3 }},
{BASICWIFI50CLIENTLISTMACADDRESS,ASN_OCTET_STR,RONLY,var_BasicWiFi50ClientListTable, 3,  { 1 , 1, 4 }},
{BASICWIFI50CLIENTLISTMODE,ASN_OCTET_STR,RONLY,var_BasicWiFi50ClientListTable, 3,  { 1 , 1, 5 }},
{BASICWIFI50CLIENTLISTRATE,ASN_INTEGER,RONLY,var_BasicWiFi50ClientListTable, 3,  { 1 , 1, 6 }},
{BASICWIFI50CLIENTLISTSIGNAL,ASN_INTEGER,RONLY,var_BasicWiFi50ClientListTable, 3,  { 1 , 1, 7 }},
{BASICWIFI50CLIENTLISTINTERFACE,ASN_OCTET_STR,RONLY,var_BasicWiFi50ClientListTable, 3,  { 1 , 1, 8 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UIBasicWiFi50ClientList module */
void
init_UIBasicWiFi50ClientList(void)
{
    DEBUGMSGTL(("UIBasicWiFi50ClientList", "Initializing\n"));
    REGISTER_MIB("UIBasicWiFi50ClientList", UIBasicWiFi50ClientList_variables, variable7,
               UIBasicWiFi50ClientList_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{

	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}


unsigned char *
var_UIBasicWiFi50ClientList(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_BasicWiFi50ClientListTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UIBasicWiFi50ClientList above.
 */
unsigned char *
var_BasicWiFi50ClientListTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
	int number=0;
	static char w_client_list_mac[6];
	/* Get 5G total client number */
	system("stainfo2");
	
	read_csman(fd,0x80447300,&number,sizeof(int),TYPE_U32);
	
	if(number <= 0)	number = 1;
	
	/* get index */
	register int index;
	index = header_Entry(vp, name, length, exact, var_len, write_method,number);
	
	if ( index == MATCH_FAILED )
	   return NULL;


    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICWIFI50CLIENTLISTINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case BASICWIFI50CLIENTLISTIPADDRESS:
        memset(string,0,sizeof(string));
		read_csman(fd,0x80447400+index-1,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
    case BASICWIFI50CLIENTLISTHOSTNAME:
        memset(string,0,sizeof(string));
		read_csman(fd,0x80447500+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICWIFI50CLIENTLISTMACADDRESS:
        memset(w_client_list_mac,0,sizeof(w_client_list_mac));
		read_csman(fd,0x80446000 +index-1,w_client_list_mac,sizeof(w_client_list_mac),TYPE_MAC);
		*var_len = sizeof(w_client_list_mac);
		return (unsigned char *) w_client_list_mac;
    case BASICWIFI50CLIENTLISTMODE:
        memset(string,0,sizeof(string));
		read_csman(fd,0x80447600+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICWIFI50CLIENTLISTRATE:
        read_csman(fd,0x80446f00+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
    case BASICWIFI50CLIENTLISTSIGNAL:
        read_csman(fd,0x80447200+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
	    return (unsigned char *) &long_ret;
    case BASICWIFI50CLIENTLISTINTERFACE:
        memset(string,0,sizeof(string));
		read_csman(fd,0x80447700+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


