#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIBasicWiFi50WPSProtectedSetup.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIBasicWiFi50WPSProtectedSetup_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,4,1 };
#else
oid UIBasicWiFi50WPSProtectedSetup_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,4,1 };
#endif

struct variable7 UIBasicWiFi50WPSProtectedSetup_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */


{BASICWIFI50WPSENABLE,ASN_INTEGER,RWRITE,var_UIBasicWiFi50WPSProtectedSetup, 1,  { 1 }},
{BASICWIFI50WPSCONFIGURATIONSTATUS,ASN_INTEGER,RONLY,var_UIBasicWiFi50WPSProtectedSetup, 1,  { 2 }},
{BASICWIFI50WPSCONFIGURATIONMODE,ASN_INTEGER,RWRITE,var_UIBasicWiFi50WPSProtectedSetup, 1,  { 3 }},
{BASICWIFI50WPSCONFIGURATIONMODEENROLLEECONDEGENERATION,ASN_OCTET_STR,RONLY,var_UIBasicWiFi50WPSProtectedSetup, 1,  { 4 }},
{BASICWIFI50WPSCONFIGMETHOD,ASN_INTEGER,RWRITE,var_UIBasicWiFi50WPSProtectedSetup, 1,  { 5 }},
{BASICWIFI50WPSCONFIGMETHODPINCODECODE,ASN_OCTET_STR,RWRITE,var_UIBasicWiFi50WPSProtectedSetup, 1,  { 6 }},
{BASICWIFI50WPSCONFIGWPSTRIGGER,ASN_INTEGER,RWRITE,var_UIBasicWiFi50WPSProtectedSetup, 1,  { 7 }},

};
/*    (L = length of the oidsuffix) */


/** Initializes the UIBasicWiFi50WPSProtectedSetup module */
void
init_UIBasicWiFi50WPSProtectedSetup(void)
{
    DEBUGMSGTL(("UIBasicWiFi50WPSProtectedSetup", "Initializing\n"));
    REGISTER_MIB("UIBasicWiFi50WPSProtectedSetup", UIBasicWiFi50WPSProtectedSetup_variables, variable7,
               UIBasicWiFi50WPSProtectedSetup_variables_oid);
}

unsigned char *
var_UIBasicWiFi50WPSProtectedSetup(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICWIFI50WPSENABLE:
        *write_method = write_BasicWiFi50WPSEnable;
        read_csman(fd,0x0044a000,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICWIFI50WPSCONFIGURATIONSTATUS:
        read_csman(fd,0x0044a200,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICWIFI50WPSCONFIGURATIONMODE:
        *write_method = write_BasicWiFi50WPSConfigurationMode;
        read_csman(fd,0x0044a100,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICWIFI50WPSCONFIGURATIONMODEENROLLEECONDEGENERATION:
		system("wpsv2-action gennewpin");
		memset(string,0,sizeof(string));
		read_csman(fd,0x80440104,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICWIFI50WPSCONFIGMETHOD:
        *write_method = write_BasicWiFi50WPSConfigMethod;
        read_csman(fd,0x0044a300,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICWIFI50WPSCONFIGMETHODPINCODECODE:
        *write_method = write_BasicWiFi50WPSConfigMethodPinCodeCode;
        memset(string,0,sizeof(string));
		read_csman(fd,0x80440100,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICWIFI50WPSCONFIGWPSTRIGGER:
        *write_method = write_BasicWiFi50WPSConfigWPSTrigger;
        read_csman(fd,0x80440102,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}

int
write_BasicWiFi50WPSEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicWiFi50WPSProtectedSetup not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicWiFi50WPSProtectedSetup: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x0044a000,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicWiFi50WPSConfigurationMode(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicWiFi50WPSProtectedSetup not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicWiFi50WPSProtectedSetup: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
              int fd=0;
              write_csman(fd,0x0044a100,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
         }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicWiFi50WPSConfigMethod(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicWiFi50WPSProtectedSetup not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicWiFi50WPSProtectedSetup: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;


        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
              int fd=0;
              write_csman(fd,0x0044a300,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
         }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicWiFi50WPSConfigMethodPinCodeCode(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIBasicWiFi50WPSProtectedSetup not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIBasicWiFi50WPSProtectedSetup: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
             int fd=0;
             write_csman(fd,0x80440100,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicWiFi50WPSConfigWPSTrigger(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIBasicWiFi50WPSProtectedSetup not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIBasicWiFi50WPSProtectedSetup: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x80440102,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			write_csman(fd,0x80440002,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}

