#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIIPSecStatus.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_StatusIPSec 32

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIIPSecStatus_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,1,5,1 };
#else
oid UIIPSecStatus_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,1,5,1 };
#endif

struct variable7 UIIPSecStatus_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{IPSECSTATUSIDINDEX,ASN_INTEGER,RONLY,var_UIIPSecStatusTable, 3,  { 1 , 1, 1 }},
{IPSECSTATUSTUNNELNAME,ASN_OCTET_STR,RONLY,var_UIIPSecStatusTable, 3,  { 1 , 1, 2 }},
{IPSECSTATUSTUNNELSCENARIO,ASN_INTEGER,RONLY,var_UIIPSecStatusTable, 3,  { 1 , 1, 3 }},
{IPSECSTATUSLOCALSUBNET,ASN_IPADDRESS,RONLY,var_UIIPSecStatusTable, 3,  { 1 , 1, 4 }},
{IPSECSTATUSLOCALSUBNETMASK,ASN_IPADDRESS,RONLY,var_UIIPSecStatusTable, 3,  { 1 , 1, 5 }},
{IPSECSTATUSREMOTEIPFQDN,ASN_OCTET_STR,RONLY,var_UIIPSecStatusTable, 3,  { 1 , 1, 6 }},
{IPSECSTATUSREMOTESUBNET,ASN_IPADDRESS,RONLY,var_UIIPSecStatusTable, 3,  { 1 , 1, 7 }},
{IPSECSTATUSREMOTESUBNETMASK,ASN_IPADDRESS,RONLY,var_UIIPSecStatusTable, 3,  { 1 , 1, 8 }},
{IPSECSTATUS,ASN_INTEGER,RONLY,var_UIIPSecStatusTable, 3,  { 1 , 1, 9 }},
};
/*    (L = length of the oidsuffix) */

void init_UIIPSecStatus(void)
{
    DEBUGMSGTL(("UIIPSecStatus", "Initializing\n"));
    REGISTER_MIB("UIIPSecStatus", UIIPSecStatus_variables, variable7,UIIPSecStatus_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UIIPSecStatus(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}

unsigned char *
var_UIIPSecStatusTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;

	register int index;
	index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_StatusIPSec);
		
	if ( index == MATCH_FAILED )
		return NULL;

    switch(vp->magic) {
    case IPSECSTATUSIDINDEX:
        long_ret = index;
        return (unsigned int *) &long_ret;
    case IPSECSTATUSTUNNELNAME:
	{
		memset(string,0,sizeof(string));
		read_csman(fd,0x00700100+index-1,string,sizeof(string),TYPE_STR);	
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    }
    case IPSECSTATUSTUNNELSCENARIO:
	{
		read_csman(fd,0x0020101d+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    }
    case IPSECSTATUSLOCALSUBNET:
	{
		int key_method=0;
		unsigned char local_subnet[4];
		memset(local_subnet,0,sizeof(local_subnet));
		if(index == 1)	read_csman(fd,0x00700400+index-1,local_subnet,sizeof(local_subnet),TYPE_IPV4);
		else{
			read_csman(fd,0x00051700+index-1,&key_method,sizeof(unsigned int),TYPE_U32);
			if(key_method == 0)	read_csman(fd,0x00700400+index-1,local_subnet,sizeof(local_subnet),TYPE_IPV4);
			else				read_csman(fd,0x00700900+index-1,local_subnet,sizeof(local_subnet),TYPE_IPV4);
		}
		*var_len = 4; //IP address length = 4
		return (unsigned char *) local_subnet;
	}
    case IPSECSTATUSLOCALSUBNETMASK:
	{
		int key_method=0;
		unsigned char local_subnet_mask[4];
		memset(local_subnet_mask,0,sizeof(local_subnet_mask));
		if(index == 1)	read_csman(fd,0x00700500+index-1,local_subnet_mask,sizeof(local_subnet_mask),TYPE_IPV4);
		else{
			read_csman(fd,0x00051700+index-1,&key_method,sizeof(unsigned int),TYPE_U32);
			if(key_method == 0) read_csman(fd,0x00700500+index-1,local_subnet_mask,sizeof(local_subnet_mask),TYPE_IPV4);
			else				read_csman(fd,0x00700A00+index-1,local_subnet_mask,sizeof(local_subnet_mask),TYPE_IPV4);
		}
		*var_len = 4; //IP address length = 4
		return (unsigned char *) local_subnet_mask;
	}
    case IPSECSTATUSREMOTEIPFQDN:
	{
		int key_method=0;
		memset(string,0,sizeof(string));
		if(index == 1)	read_csman(fd,0x80702300+index-1,string,sizeof(string),TYPE_STR);	
		else{
			read_csman(fd,0x00051700+index-1,&key_method,sizeof(unsigned int),TYPE_U32);
			if(key_method == 0) read_csman(fd,0x00700800+index-1,string,sizeof(string),TYPE_STR);	
			else				read_csman(fd,0x00700D00+index-1,string,sizeof(string),TYPE_STR);	
		}
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
	}
    case IPSECSTATUSREMOTESUBNET:
	{
		int key_method=0;
		unsigned char remote_subnet[4];
		memset(remote_subnet,0,sizeof(remote_subnet));
		if(index == 1)	read_csman(fd,0x80702100+index-1,remote_subnet,sizeof(remote_subnet),TYPE_IPV4);
		else{
			read_csman(fd,0x00051700+index-1,&key_method,sizeof(unsigned int),TYPE_U32);
			if(key_method == 0) read_csman(fd,0x00700600+index-1,remote_subnet,sizeof(remote_subnet),TYPE_IPV4);
			else				read_csman(fd,0x00700B00+index-1,remote_subnet,sizeof(remote_subnet),TYPE_IPV4);
		}
		*var_len = 4; //IP address length = 4
		return (unsigned char *) remote_subnet;
	}
    case IPSECSTATUSREMOTESUBNETMASK:
	{
		int key_method=0;
		unsigned char remote_subnet_mask[4];
		memset(remote_subnet_mask,0,sizeof(remote_subnet_mask));
		if(index == 1)	read_csman(fd,0x80702200+index-1,remote_subnet_mask,sizeof(remote_subnet_mask),TYPE_IPV4);
		else{
			read_csman(fd,0x00051700+index-1,&key_method,sizeof(unsigned int),TYPE_U32);
			if(key_method == 0) read_csman(fd,0x00700700+index-1,remote_subnet_mask,sizeof(remote_subnet_mask),TYPE_IPV4);
			else				read_csman(fd,0x00700C00+index-1,remote_subnet_mask,sizeof(remote_subnet_mask),TYPE_IPV4);
		}
		*var_len = 4; //IP address length = 4
		return (unsigned char *) remote_subnet_mask;
	}
    case IPSECSTATUS:
	{
		unsigned char dynamic_ip[4];
		unsigned char compare_ip[32];
		memset(dynamic_ip,0,sizeof(dynamic_ip));
		if(index == 1){
			read_csman(fd,0x80702100+index-1,dynamic_ip,sizeof(dynamic_ip),TYPE_IPV4);
			strcpy(compare_ip,dynamic_ip);
			if(!strcmp("0.0.0.0",compare_ip))	long_ret=0;
			else	long_ret=1;
		}
		else	read_csman(fd,0x80700100,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
	}
    default:
      ERROR_MSG("");
    }
    return NULL;
}
