#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UIInterfaceTrafficStatisticsStatus.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIInterfaceTrafficStatisticsStatus_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,1,1,5 };
#else
oid UIInterfaceTrafficStatisticsStatus_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,1,1,5 };
#endif

struct variable7 UIInterfaceTrafficStatisticsStatus_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{STATUSWANID1,ASN_OCTET_STR,RONLY,var_UIInterfaceTrafficStatisticsStatus, 1,  { 1 }},
{STATUSPHYSICALINTERFACE1,ASN_OCTET_STR,RONLY,var_UIInterfaceTrafficStatisticsStatus, 1,  { 2 }},
{STATUSRECEIVEDPACKETS1,ASN_INTEGER,RONLY,var_UIInterfaceTrafficStatisticsStatus, 1,  { 3 }},
{STATUSTRANSMITPACKETS1,ASN_INTEGER,RONLY,var_UIInterfaceTrafficStatisticsStatus, 1,  { 4 }},
{STATUSWANID2,ASN_OCTET_STR,RONLY,var_UIInterfaceTrafficStatisticsStatus, 1,  { 5 }},
{STATUSPHYSICALINTERFACE2,ASN_OCTET_STR,RONLY,var_UIInterfaceTrafficStatisticsStatus, 1,  { 6 }},
{STATUSRECEIVEDPACKETS2,ASN_INTEGER,RONLY,var_UIInterfaceTrafficStatisticsStatus, 1,  { 7 }},
{STATUSTRANSMITPACKETS2,ASN_INTEGER,RONLY,var_UIInterfaceTrafficStatisticsStatus, 1,  { 8 }},
{STATUSWANID3,ASN_OCTET_STR,RONLY,var_UIInterfaceTrafficStatisticsStatus, 1,  { 9 }},
{STATUSPHYSICALINTERFACE3,ASN_OCTET_STR,RONLY,var_UIInterfaceTrafficStatisticsStatus, 1,  { 10 }},
{STATUSRECEIVEDPACKETS3,ASN_INTEGER,RONLY,var_UIInterfaceTrafficStatisticsStatus, 1,  { 11 }},
{STATUSTRANSMITPACKETS3,ASN_INTEGER,RONLY,var_UIInterfaceTrafficStatisticsStatus, 1,  { 12 }},
};

struct statistic_mib
{
	unsigned long	Wan1Transmit,Wan2Transmit;
	unsigned long	Wan1Receive,Wan2Receive;
	unsigned long	m3g1Transmit,m3g2Transmit;
	unsigned long	m3g1Receive,m3g2Receive;
	unsigned long	LanTransmit,LanReceive;
	unsigned long	WlanTransmit,WlanReceive;
	unsigned long	TrashData;
};

void
init_UIInterfaceTrafficStatisticsStatus(void)
{
    DEBUGMSGTL(("UIInterfaceTrafficStatisticsStatus", "Initializing\n"));
    REGISTER_MIB("UIInterfaceTrafficStatisticsStatus", UIInterfaceTrafficStatisticsStatus_variables, variable7,
               UIInterfaceTrafficStatisticsStatus_variables_oid);
}

static void
linux_read_statistic (struct statistic_mib *statistic)
{
  char line [2048];

  system("traffic_stat > /tmp/snmp_multiwan_statistic");
  FILE *in = fopen ("/tmp/snmp_multiwan_statistic", "r");

  memset ((char *) statistic,(0), sizeof (*statistic));

  if (! in)
    return;

  while(line == fgets (line, sizeof(line), in))
  {
  	if (10 == sscanf (line, "WAN %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu\n",
		&statistic->TrashData,&statistic->Wan1Transmit,&statistic->TrashData, 
		&statistic->TrashData,&statistic->TrashData,&statistic->TrashData,
		&statistic->Wan1Receive,&statistic->TrashData,&statistic->TrashData,
		&statistic->TrashData))
	break;
  }

  while(line == fgets (line, sizeof(line), in))
  {
	if (10 == sscanf (line, "WAN2 %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu\n",
		  &statistic->TrashData,&statistic->Wan2Transmit,&statistic->TrashData, 
		  &statistic->TrashData,&statistic->TrashData,&statistic->TrashData,
		  &statistic->Wan2Receive,&statistic->TrashData,&statistic->TrashData,
		  &statistic->TrashData))
	break;
  }

  while (line == fgets (line, sizeof(line), in))
  {
	 if (10 == sscanf (line, "LAN %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu\n",
		  &statistic->TrashData,&statistic->LanTransmit,&statistic->TrashData, 
		  &statistic->TrashData,&statistic->TrashData,&statistic->TrashData,
		  &statistic->LanReceive,&statistic->TrashData,&statistic->TrashData,
		  &statistic->TrashData))
	break;
  }

  while (line == fgets (line, sizeof(line), in))
  {
	if (10 == sscanf (line, "WLAN %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu\n",
		   &statistic->TrashData,&statistic->WlanTransmit,&statistic->TrashData, 
		   &statistic->TrashData,&statistic->TrashData,&statistic->TrashData,
		   &statistic->WlanReceive,&statistic->TrashData,&statistic->TrashData,
		   &statistic->TrashData))
	break;
  }

  while(line == fgets (line, sizeof(line), in))
  {
	if (10 == sscanf (line, "3G1 %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu\n",
			 &statistic->TrashData,&statistic->m3g1Transmit,&statistic->TrashData, 
			 &statistic->TrashData,&statistic->TrashData,&statistic->TrashData,
			 &statistic->m3g1Receive,&statistic->TrashData,&statistic->TrashData,
			 &statistic->TrashData))
	break;
  }	

  while(line == fgets (line, sizeof(line), in))
  {
	if (10 == sscanf (line, "3G2 %lu %lu %lu %lu %lu %lu %lu %lu %lu %lu\n",
			 &statistic->TrashData,&statistic->m3g2Transmit,&statistic->TrashData, 
			 &statistic->TrashData,&statistic->TrashData,&statistic->TrashData,
			 &statistic->m3g2Receive,&statistic->TrashData,&statistic->TrashData,
			 &statistic->TrashData))
	break;
  }	


  fclose (in);
}



unsigned char *
var_UIInterfaceTrafficStatisticsStatus(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	static struct statistic_mib statistic; // for get statistic data
	int fd=0;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

	linux_read_statistic(&statistic); //get statistic data	

    switch(vp->magic) {
    case STATUSWANID1:
		memset(string,0,sizeof(string));
		strcpy(string,"WAN-1 ");
        *var_len = strlen(string)-1;
		return (unsigned char *) string;
    case STATUSPHYSICALINTERFACE1:
	{
		int wanindex=0;
		memset(string,0,sizeof(string));
		read_csman(fd,0x00010600,&wanindex,sizeof(unsigned int),TYPE_U32);
		if(wanindex == 99 || wanindex == 1 || wanindex ==2 || wanindex == 3 || wanindex == 4 || wanindex == 5){
			strcpy(string,"Ethernet ");
		}
		else if(wanindex == 16){
			strcpy(string,"Dial-up ");
		}
		else if(wanindex == 14 || wanindex == 15){
			strcpy(string,"3G/4G ");
		}
		else strcpy(string,"Disable ");
	
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    }
    case STATUSRECEIVEDPACKETS1:
	{
		int wanindex=0;
		read_csman(fd,0x00010600,&wanindex,sizeof(unsigned int),TYPE_U32);
		if(wanindex == 99)	long_ret = statistic.Wan1Receive;
		else if(wanindex == 1 || wanindex == 2 || wanindex == 3 || wanindex == 4 || wanindex == 5){
			long_ret = statistic.Wan2Receive;
		}
		else if(wanindex == 16){
			long_ret = statistic.m3g2Receive;
		}
		else if(wanindex == 14 || wanindex == 15){
			long_ret = statistic.m3g1Receive;
		}
		else	long_ret=0;
	
	    return (unsigned char *) &long_ret;
    }
    case STATUSTRANSMITPACKETS1:
	{
		int wanindex=0;
		read_csman(fd,0x00010600,&wanindex,sizeof(unsigned int),TYPE_U32);
		if(wanindex == 99)	long_ret = statistic.Wan1Transmit;
		else if(wanindex == 1 || wanindex == 2 || wanindex == 3 || wanindex == 4 || wanindex == 5){
			long_ret = statistic.Wan2Transmit;
		}
		else if(wanindex == 16){
			long_ret = statistic.m3g2Transmit;
		}
		else if(wanindex == 14 || wanindex == 15){
			long_ret = statistic.m3g1Transmit;
		}
		else	long_ret=0;
		
		return (unsigned char *) &long_ret;
	}
    case STATUSWANID2:
		memset(string,0,sizeof(string));
		strcpy(string,"WAN-2 ");
        *var_len = strlen(string)-1;
		return (unsigned char *) string;
    case STATUSPHYSICALINTERFACE2:
	{
		int wanindex=0;
		memset(string,0,sizeof(string));
		read_csman(fd,0x00010601,&wanindex,sizeof(unsigned int),TYPE_U32);
		if(wanindex == 99 || wanindex == 1 || wanindex ==2 || wanindex == 3 || wanindex == 4 || wanindex == 5){
			strcpy(string,"Ethernet ");
		}
		else if(wanindex == 16){
			strcpy(string,"Dial-up ");
		}
		else if(wanindex == 14 || wanindex == 15){
			strcpy(string,"3G/4G ");
		}
		else strcpy(string,"Disable ");
		
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
	}
    case STATUSRECEIVEDPACKETS2:
	{
		int wanindex=0;
		read_csman(fd,0x00010601,&wanindex,sizeof(unsigned int),TYPE_U32);
		if(wanindex == 99)	long_ret = statistic.Wan1Receive;
		else if(wanindex == 1 || wanindex == 2 || wanindex == 3 || wanindex == 4 || wanindex == 5){
			long_ret = statistic.Wan2Receive;
		}
		else if(wanindex == 16){
			long_ret = statistic.m3g2Receive;
		}
		else if(wanindex == 14 || wanindex == 15){
			long_ret = statistic.m3g1Receive;
		}
		else	long_ret=0;
	
		return (unsigned char *) &long_ret;
	}
    case STATUSTRANSMITPACKETS2:
	{
		int wanindex=0;
		read_csman(fd,0x00010601,&wanindex,sizeof(unsigned int),TYPE_U32);
		if(wanindex == 99)	long_ret = statistic.Wan1Transmit;
		else if(wanindex == 1 || wanindex == 2 || wanindex == 3 || wanindex == 4 || wanindex == 5){
			long_ret = statistic.Wan2Transmit;
		}
		else if(wanindex == 16){
			long_ret = statistic.m3g2Transmit;
		}
		else if(wanindex == 14 || wanindex == 15){
			long_ret = statistic.m3g1Transmit;
		}
		else	long_ret=0;
			
		return (unsigned char *) &long_ret;
	}
    case STATUSWANID3:
		memset(string,0,sizeof(string));
		strcpy(string,"WAN-3 ");
        *var_len = strlen(string)-1;
		return (unsigned char *) string;
    case STATUSPHYSICALINTERFACE3:
	{
		int wanindex=0;
		memset(string,0,sizeof(string));
		read_csman(fd,0x00010602,&wanindex,sizeof(unsigned int),TYPE_U32);
		if(wanindex == 99 || wanindex == 1 || wanindex ==2 || wanindex == 3 || wanindex == 4 || wanindex == 5){
			strcpy(string,"Ethernet ");
		}
		else if(wanindex == 16){
			strcpy(string,"Dial-up ");
		}
		else if(wanindex == 14 || wanindex == 15){
			strcpy(string,"3G/4G ");
		}
		else strcpy(string,"Disable ");
		
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
	}
    case STATUSRECEIVEDPACKETS3:
	{
		int wanindex=0;
		read_csman(fd,0x00010602,&wanindex,sizeof(unsigned int),TYPE_U32);
		if(wanindex == 99)	long_ret = statistic.Wan1Receive;
		else if(wanindex == 1 || wanindex == 2 || wanindex == 3 || wanindex == 4 || wanindex == 5){
			long_ret = statistic.Wan2Receive;
		}
		else if(wanindex == 16){
			long_ret = statistic.m3g2Receive;
		}
		else if(wanindex == 14 || wanindex == 15){
			long_ret = statistic.m3g1Receive;
		}
		else	long_ret=0;
		
		return (unsigned char *) &long_ret;
	}
    case STATUSTRANSMITPACKETS3:
	{
		int wanindex=0;
		read_csman(fd,0x00010602,&wanindex,sizeof(unsigned int),TYPE_U32);
		if(wanindex == 99)	long_ret = statistic.Wan1Transmit;
		else if(wanindex == 1 || wanindex == 2 || wanindex == 3 || wanindex == 4 || wanindex == 5){
			long_ret = statistic.Wan2Transmit;
		}
		else if(wanindex == 16){
			long_ret = statistic.m3g2Transmit;
		}
		else if(wanindex == 14 || wanindex == 15){
			long_ret = statistic.m3g1Transmit;
		}
		else	long_ret=0;
				
		return (unsigned char *) &long_ret;
	}
    default:
      ERROR_MSG("");
    }
    return NULL;
}




