#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UIInternetSetupDynamic.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIInternetSetupDynamic_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,1,5 };
#else
oid UIInternetSetupDynamic_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,1,5 };
#endif

struct variable7 UIInternetSetupDynamic_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{BASICWANSETUPDYNAMICHOSTNAME,ASN_OCTET_STR,RWRITE,var_UIInternetSetupDynamic, 1,  { 1 }},
{BASICWANSETUPDYNAMICISPREGMACADDRESS,ASN_OCTET_STR,RWRITE,var_UIInternetSetupDynamic, 1,  { 2 }},
{BASICWANSETUPDYNAMICCONNECTIONCONTROL,ASN_INTEGER,RWRITE,var_UIInternetSetupDynamic, 1,  { 3 }},
{BASICWANSETUPDYNAMICMAXIDLETIME,ASN_INTEGER,RWRITE,var_UIInternetSetupDynamic, 1,  { 4 }},
{BASICWANSETUPDYNAMICMTU,ASN_INTEGER,RWRITE,var_UIInternetSetupDynamic, 1,  { 5 }},
{BASICWANSETUPDYNAMICNATENABLE,ASN_INTEGER,RWRITE,var_UIInternetSetupDynamic, 1,  { 6 }},

};
/*    (L = length of the oidsuffix) */


/** Initializes the UIInternetSetupDynamic module */
void
init_UIInternetSetupDynamic(void)
{
    DEBUGMSGTL(("UIInternetSetupDynamic", "Initializing\n"));
    REGISTER_MIB("UIInternetSetupDynamic", UIInternetSetupDynamic_variables, variable7,
               UIInternetSetupDynamic_variables_oid);
}

unsigned char *
var_UIInternetSetupDynamic(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
	case BASICWANSETUPDYNAMICHOSTNAME:
		*write_method = write_BasicWANSetupDynamicHostName;
		memset(string,0,sizeof(string));
		read_csman(fd,0x00035002,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
	case BASICWANSETUPDYNAMICISPREGMACADDRESS:
        *write_method = write_BasicWANSetupDynamicISPRegMACAddress;
        memset(string,0,sizeof(string));
		read_csman(fd,0x0001000f,string,sizeof(string),TYPE_MAC);
		*var_len = 6;
		return (unsigned char *) string;
    case BASICWANSETUPDYNAMICCONNECTIONCONTROL:
        *write_method = write_BasicWANSetupDynamicConnectionControl;
        read_csman(fd,0x0003500e,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICWANSETUPDYNAMICMAXIDLETIME:
        *write_method = write_BasicWANSetupDynamicMaxIdleTime;
        read_csman(fd,0x0003500d,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICWANSETUPDYNAMICMTU:
        *write_method = write_BasicWANSetupDynamicMTU;
        read_csman(fd,0x00035009,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICWANSETUPDYNAMICNATENABLE:
        *write_method = write_BasicWANSetupDynamicNATEnable;
        read_csman(fd,0x0003500f,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}

int
write_BasicWANSetupDynamicHostName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIInternetSetupDynamic not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIInternetSetupDynamic: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
             int fd=0;
             write_csman(fd,0x00035002,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}

int
write_BasicWANSetupDynamicISPRegMACAddress(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to NetworkSetup not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to NetworkSetup: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  //for STR/MAC data type
			  memset(string,0,sizeof(string));
			  strncpy(string , (char *) var_val, var_val_len);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
		  {
			  int fd=0;
			  write_csman(fd,0x0001000f,string,strlen(string) ,TYPE_MAC);
			  break;
		  }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicWANSetupDynamicConnectionControl(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIInternetSetupDynamic not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIInternetSetupDynamic: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
              int fd=0;
              write_csman(fd,0x0003500e,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
         }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicWANSetupDynamicMaxIdleTime(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIInternetSetupDynamic not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIInternetSetupDynamic: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x0003500d,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicWANSetupDynamicMTU(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIInternetSetupDynamic not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIInternetSetupDynamic: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00035009,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicWANSetupDynamicNATEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIInternetSetupDynamic not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIInternetSetupDynamic: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x0003500f,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}

