#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UIInternetSetupMultiL2TP.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_BAS_WAN_MULTI_L2TP	16
int w_index_bas_wan_multi_l2tp;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIInternetSetupMultiL2TP_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,1,14 };
#else
oid UIInternetSetupMultiL2TP_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,1,14 };
#endif

struct variable7 UIInternetSetupMultiL2TP_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{BASICWANSETUPMULTIL2TPINDEX,ASN_INTEGER,RONLY,var_BasicWANSetupMultiL2TPTable, 3,  { 1 , 1, 1 }},
{BASICWANSETUPMULTIL2TPIPMODE,ASN_INTEGER,RWRITE,var_BasicWANSetupMultiL2TPTable, 3,  { 1 , 1, 2 }},
{BASICWANSETUPMULTIL2TPWANIPADDRESS,ASN_IPADDRESS,RWRITE,var_BasicWANSetupMultiL2TPTable, 3,  { 1 , 1, 3 }},
{BASICWANSETUPMULTIL2TPWANSUBNETMASK,ASN_IPADDRESS,RWRITE,var_BasicWANSetupMultiL2TPTable, 3,  { 1 , 1, 4 }},
{BASICWANSETUPMULTIL2TPWANGATEWAY,ASN_IPADDRESS,RWRITE,var_BasicWANSetupMultiL2TPTable, 3,  { 1 , 1, 5 }},
{BASICWANSETUPMULTIL2TPSERVICEIPADDRESSNAME,ASN_OCTET_STR,RWRITE,var_BasicWANSetupMultiL2TPTable, 3,  { 1 , 1, 6 }},
{BASICWANSETUPMULTIL2TPACCOUNT,ASN_OCTET_STR,RWRITE,var_BasicWANSetupMultiL2TPTable, 3,  { 1 , 1, 7 }},
{BASICWANSETUPMULTIL2TPPASSWORD,ASN_OCTET_STR,RWRITE,var_BasicWANSetupMultiL2TPTable, 3,  { 1 , 1, 8 }},
{BASICWANSETUPMULTIL2TPCONNECTIONCONTROL,ASN_INTEGER,RWRITE,var_BasicWANSetupMultiL2TPTable, 3,  { 1 , 1, 9 }},
{BASICWANSETUPMULTIL2TPMAXIDLETIME,ASN_INTEGER,RWRITE,var_BasicWANSetupMultiL2TPTable, 3,  { 1 , 1, 10 }},
{BASICWANSETUPMULTIL2TPMTU,ASN_INTEGER,RWRITE,var_BasicWANSetupMultiL2TPTable, 3,  { 1 , 1, 11 }},
{BASICWANSETUPMULTIL2TPMPPEENABLE,ASN_INTEGER,RWRITE,var_BasicWANSetupMultiL2TPTable, 3,  { 1 , 1, 12 }},
{BASICWANSETUPMULTIL2TPNATENABLE,ASN_INTEGER,RWRITE,var_BasicWANSetupMultiL2TPTable, 3,  { 1 , 1, 13 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UIInternetSetupMultiL2TP module */
void
init_UIInternetSetupMultiL2TP(void)
{
    DEBUGMSGTL(("UIInternetSetupMultiL2TP", "Initializing\n"));
    REGISTER_MIB("UIInternetSetupMultiL2TP", UIInternetSetupMultiL2TP_variables, variable7,
               UIInternetSetupMultiL2TP_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UIInternetSetupMultiL2TP(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}

unsigned char *
var_BasicWANSetupMultiL2TPTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    /* get index */
    register int index;
    index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_BAS_WAN_MULTI_L2TP);
    w_index_bas_wan_multi_l2tp=index;

    if ( index == MATCH_FAILED )
        return NULL;


    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICWANSETUPMULTIL2TPINDEX:
        long_ret = index;
       	return (unsigned int *) &long_ret;
    case BASICWANSETUPMULTIL2TPIPMODE:
        *write_method = write_BasicWANSetupMultiL2TPIPMode;
        read_csman(fd,0x00062e00+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICWANSETUPMULTIL2TPWANIPADDRESS:
        *write_method = write_BasicWANSetupMultiL2TPWANIPAddress;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00062300+index-1,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
    case BASICWANSETUPMULTIL2TPWANSUBNETMASK:
        *write_method = write_BasicWANSetupMultiL2TPWANSubnetMask;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00062400+index-1,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
    case BASICWANSETUPMULTIL2TPWANGATEWAY:
        *write_method = write_BasicWANSetupMultiL2TPWANGateway;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00062500+index-1,string,sizeof(string),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) string;
    case BASICWANSETUPMULTIL2TPSERVICEIPADDRESSNAME:
        *write_method = write_BasicWANSetupMultiL2TPServiceIPAddressName;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00062600+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICWANSETUPMULTIL2TPACCOUNT:
        *write_method = write_BasicWANSetupMultiL2TPAccount;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00062100+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICWANSETUPMULTIL2TPPASSWORD:
        *write_method = write_BasicWANSetupMultiL2TPPassword;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00062200+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case BASICWANSETUPMULTIL2TPCONNECTIONCONTROL:
        *write_method = write_BasicWANSetupMultiL2TPConnectionControl;
        read_csman(fd,0x00062900+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICWANSETUPMULTIL2TPMAXIDLETIME:
        *write_method = write_BasicWANSetupMultiL2TPMaxIdleTime;
        read_csman(fd,0x00062b00+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICWANSETUPMULTIL2TPMTU:
        *write_method = write_BasicWANSetupMultiL2TPMTU;
        read_csman(fd,0x00062a00+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICWANSETUPMULTIL2TPMPPEENABLE:
        *write_method = write_BasicWANSetupMultiL2TPMPPEEnable;
        read_csman(fd,0x00064300+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case BASICWANSETUPMULTIL2TPNATENABLE:
        *write_method = write_BasicWANSetupMultiL2TPNATEnable;
        read_csman(fd,0x00060030+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


int
write_BasicWANSetupMultiL2TPIPMode(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIInternetSetupMultiL2TP not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIInternetSetupMultiL2TP: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00062e00+w_index_bas_wan_multi_l2tp-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicWANSetupMultiL2TPWANIPAddress(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIInternetSetupMultiL2TP not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIInternetSetupMultiL2TP: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  memcpy(string , (char *) var_val, 4);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		  {
			  int fd=0;
			  write_csman(fd,0x00062300+w_index_bas_wan_multi_l2tp-1,string,strlen(string) ,TYPE_IPV4);
			  break;
		  }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicWANSetupMultiL2TPWANSubnetMask(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIInternetSetupMultiL2TP not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIInternetSetupMultiL2TP: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  memcpy(string , (char *) var_val, 4);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		  {
			  int fd=0;
			  write_csman(fd,0x00062400+w_index_bas_wan_multi_l2tp-1,string,strlen(string) ,TYPE_IPV4);
			  break;
		  }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicWANSetupMultiL2TPWANGateway(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_IPADDRESS) {
              fprintf(stderr, "write to UIInternetSetupMultiL2TP not ASN_IPADDRESS\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIInternetSetupMultiL2TP: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			memcpy(string , (char *) var_val, 4);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		  {
			  int fd=0;
			  write_csman(fd,0x00062500+w_index_bas_wan_multi_l2tp-1,string,strlen(string) ,TYPE_IPV4);
			  break;
		  }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicWANSetupMultiL2TPServiceIPAddressName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIInternetSetupMultiL2TP not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIInternetSetupMultiL2TP: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  strncpy(string , (char *) var_val, var_val_len);
			  break;
			  
        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
             int fd=0;
             write_csman(fd,0x00062600+w_index_bas_wan_multi_l2tp-1,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicWANSetupMultiL2TPAccount(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIInternetSetupMultiL2TP not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIInternetSetupMultiL2TP: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  strncpy(string , (char *) var_val, var_val_len);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
             int fd=0;
             write_csman(fd,0x00062100+w_index_bas_wan_multi_l2tp-1,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicWANSetupMultiL2TPPassword(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIInternetSetupMultiL2TP not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIInternetSetupMultiL2TP: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  strncpy(string , (char *) var_val, var_val_len);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
             int fd=0;
             write_csman(fd,0x00062200+w_index_bas_wan_multi_l2tp-1,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicWANSetupMultiL2TPConnectionControl(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIInternetSetupMultiL2TP not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIInternetSetupMultiL2TP: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00062900+w_index_bas_wan_multi_l2tp-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicWANSetupMultiL2TPMaxIdleTime(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIInternetSetupMultiL2TP not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIInternetSetupMultiL2TP: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00062b00+w_index_bas_wan_multi_l2tp-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicWANSetupMultiL2TPMTU(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIInternetSetupMultiL2TP not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIInternetSetupMultiL2TP: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
              int fd=0;
              write_csman(fd,0x00062a00+w_index_bas_wan_multi_l2tp-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
         }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicWANSetupMultiL2TPMPPEEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIInternetSetupMultiL2TP not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIInternetSetupMultiL2TP: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00064300+w_index_bas_wan_multi_l2tp-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicWANSetupMultiL2TPNATEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIInternetSetupMultiL2TP not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to UIInternetSetupMultiL2TP: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00060030+w_index_bas_wan_multi_l2tp-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
