#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UIL2TPServerStatus.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIL2TPServerStatus_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,1,5,4 };
#else
oid UIL2TPServerStatus_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,1,5,4 };
#endif

#define NUM_StatusL2TPServer 64

struct variable7 UIL2TPServerStatus_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{L2TPSERVERSTATUSIDINDEX,  ASN_INTEGER,  RONLY,var_UIL2TPServerStatusTable, 3,  { 1 , 1, 1 }},
{L2TPSERVERSTATUSUSERNAME,  ASN_OCTET_STR,  RONLY,var_UIL2TPServerStatusTable, 3,  { 1 , 1, 2 }},
{L2TPSERVERSTATUSPEERIPFQDN,  ASN_IPADDRESS,  RONLY,var_UIL2TPServerStatusTable, 3,  { 1 , 1, 3 }},
{L2TPSERVERSTATUSVIRTUALIP,  ASN_IPADDRESS,  RONLY,var_UIL2TPServerStatusTable, 3,  { 1 , 1, 4 }},
{L2TPSERVERSTATUSCALLID,  ASN_INTEGER,  RONLY,var_UIL2TPServerStatusTable, 3,  { 1 , 1, 5 }},
{L2TPSERVERSTATUS,  ASN_INTEGER,  RONLY,var_UIL2TPServerStatusTable, 3,  { 1 , 1, 6 }},
};

/** Initializes the UIL2TPServerStatus module */
void
init_UIL2TPServerStatus(void)
{
    DEBUGMSGTL(("UIL2TPServerStatus", "Initializing\n"));
    REGISTER_MIB("UIL2TPServerStatus", UIL2TPServerStatus_variables, variable7, UIL2TPServerStatus_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UIL2TPServerStatus(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}

unsigned char *
var_UIL2TPServerStatusTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;

	register int index;
	index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_StatusL2TPServer);
		
	if ( index == MATCH_FAILED )
		return NULL;

    switch(vp->magic) {
    case L2TPSERVERSTATUSIDINDEX:
        long_ret = index;
        return (unsigned int *) &long_ret;
    case L2TPSERVERSTATUSUSERNAME:
        memset(string,0,sizeof(string));
		read_csman(fd,0x801b0100+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case L2TPSERVERSTATUSPEERIPFQDN:
	{
		unsigned char peer_ip[4];
		memset(peer_ip,0,sizeof(peer_ip));
		read_csman(fd,0x801b0200+index-1,peer_ip,sizeof(peer_ip),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) peer_ip;
	}
    case L2TPSERVERSTATUSVIRTUALIP:
	{
		unsigned char virtual_ip[4];
		memset(virtual_ip,0,sizeof(virtual_ip));
		read_csman(fd,0x801b0300+index-1,virtual_ip,sizeof(virtual_ip),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) virtual_ip;
	}
    case L2TPSERVERSTATUSCALLID:
        read_csman(fd,0x801b0400+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case L2TPSERVERSTATUS:
        read_csman(fd,0x801b0400+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


