#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UILANClientListStatus.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_StatusLANClientList 256

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UILANClientListStatus_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,1,3,1 };
#else
oid UILANClientListStatus_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,1,3,1 };
#endif

struct variable7 UILANClientListStatus_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{LANCLIENTLISTSTATUSIDINDEX,ASN_INTEGER,RONLY,var_UIStatusLANClientListTable, 3,  { 1 , 1, 1 }},
{LANCLIENTLISTSTATUSINTERFACE,ASN_INTEGER,RONLY,var_UIStatusLANClientListTable, 3,  { 1 , 1, 2 }},
{LANCLIENTLISTSTATUSTYPE,ASN_OCTET_STR,RONLY,var_UIStatusLANClientListTable, 3,  { 1 , 1, 3 }},
{LANCLIENTLISTSTATUSIPADDRESS,ASN_IPADDRESS,RONLY,var_UIStatusLANClientListTable, 3,  { 1 , 1, 4 }},
{LANCLIENTLISTSTATUSHOSTNAME,ASN_OCTET_STR,RONLY,var_UIStatusLANClientListTable, 3,  { 1 , 1, 5 }},
{LANCLIENTLISTSTATUSMACADDRESS,ASN_OCTET_STR,RONLY,var_UIStatusLANClientListTable, 3,  { 1 , 1, 6 }},
{LANCLIENTLISTSTATUSREMAININGLEASETIME,ASN_INTEGER,RONLY,var_UIStatusLANClientListTable, 3,  { 1 , 1, 7 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UILANClientListStatus module */
void
init_UILANClientListStatus(void)
{
    DEBUGMSGTL(("UILANClientListStatus", "Initializing\n"));
    REGISTER_MIB("UILANClientListStatus", UILANClientListStatus_variables, variable7, UILANClientListStatus_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}


unsigned char *
var_UILANClientListStatus(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}

unsigned char *
var_UIStatusLANClientListTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;

	register int index;
	index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_StatusLANClientList);
		
	if ( index == MATCH_FAILED )
		return NULL;

	system("udhcpd-action lease");

    switch(vp->magic) {
    case LANCLIENTLISTSTATUSIDINDEX:
        long_ret = index;
        return (unsigned int *) &long_ret;
    case LANCLIENTLISTSTATUSINTERFACE:
        read_csman(fd,0x80030500+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case LANCLIENTLISTSTATUSTYPE:
		memset(string,0,sizeof(string));
		strcpy(string,"Dynamic ");
        *var_len = strlen(string)-1;
		return (unsigned char *) string;
    case LANCLIENTLISTSTATUSIPADDRESS:
	{
		unsigned char LANClientList_ip[4];
		memset(LANClientList_ip,0,sizeof(LANClientList_ip));
		read_csman(fd,0x80030100+index-1,LANClientList_ip,sizeof(LANClientList_ip),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) LANClientList_ip;
    }
    case LANCLIENTLISTSTATUSHOSTNAME:
		memset(string,0,sizeof(string));
		read_csman(fd,0x80030200+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case LANCLIENTLISTSTATUSMACADDRESS:
		memset(string,0,sizeof(string));
		read_csman(fd,0x80030300+index-1,string,sizeof(string),TYPE_MAC);
		*var_len = 6;
		return (unsigned char *) string;
    case LANCLIENTLISTSTATUSREMAININGLEASETIME:
		read_csman(fd,0x80030400+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}
