#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UILANInterfaceStatus.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UILANInterfaceStatus_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,1,1,3 };
#else
oid UILANInterfaceStatus_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,1,1,3 };
#endif

struct variable7 UILANInterfaceStatus_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{STATUSIPV4ADDRESS,ASN_IPADDRESS,RONLY,var_UILANInterfaceStatus, 1,  { 1 }},
{STATUSIPV4SUBNETMASK,ASN_IPADDRESS,RONLY,var_UILANInterfaceStatus, 1,  { 2 }},
{STATUSIPV6LINKLOCALADDRESS,ASN_OCTET_STR,RONLY,var_UILANInterfaceStatus, 1,  { 3 }},
{STATUSIPV6GLOBALADDRESS,ASN_OCTET_STR,RONLY,var_UILANInterfaceStatus, 1,  { 4 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UILANInterfaceStatus module */
void
init_UILANInterfaceStatus(void)
{
    DEBUGMSGTL(("UILANInterfaceStatus", "Initializing\n"));
    REGISTER_MIB("UILANInterfaceStatus", UILANInterfaceStatus_variables, variable7,
               UILANInterfaceStatus_variables_oid);
}

/*
 * var_UILANInterfaceStatus():
 *   This function is called every time the agent gets a request for
 *   a scalar variable that might be found within your mib section
 *   registered above.  It is up to you to do the right thing and
 *   return the correct value.
 *     You should also correct the value of "var_len" if necessary.
 *
 *   Please see the documentation for more information about writing
 *   module extensions, and check out the examples in the examples
 *   and mibII directories.
 */
unsigned char *
var_UILANInterfaceStatus(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    switch(vp->magic) {
	case STATUSIPV4ADDRESS:
	{
		unsigned char ip[4];
		memset(ip,0,sizeof(ip));
		read_csman(fd,0x00010004,ip,sizeof(ip),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) ip;
	}
	case STATUSIPV4SUBNETMASK:
	{
		unsigned char submask[4];
		memset(submask,0,sizeof(submask));
		read_csman(fd,0x00010005,submask,sizeof(submask),TYPE_IPV4);
		*var_len = 4; //IP address length = 4
		return (unsigned char *) submask;
	}
    case STATUSIPV6LINKLOCALADDRESS:
        memset(string,0,sizeof(string));
		read_csman(fd,0x80900008,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case STATUSIPV6GLOBALADDRESS:
        memset(string,0,sizeof(string));
		read_csman(fd,0x80900009,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    default:
      ERROR_MSG("");
    }
    return NULL;
}




