#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UIMobileApplicationCreateMessage.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_SMS_CREATEMESSAGE	16
int w_index_app_sms_createmessage;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIMobileApplicationCreateMessage_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,4,1,1 };
#else
oid UIMobileApplicationCreateMessage_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,4,1,1 };
#endif

struct variable7 UIMobileApplicationCreateMessage_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{APPMACREATEMESSAGEINDEX,ASN_INTEGER,RONLY,var_AppMACreateMessageTable, 3,  { 1 , 1, 1 }},
{APPMACREATEMESSAGE,ASN_OCTET_STR,RWRITE,var_AppMACreateMessageTable, 3,  { 1 , 1, 2 }},
{APPMACREATEMESSAGERECEIVER,ASN_OCTET_STR,RWRITE,var_AppMACreateMessageTable, 3,  { 1 , 1, 3 }},
{APPMACREATEMESSAGESENDTO,ASN_INTEGER,RWRITE,var_AppMACreateMessageTable, 3,  { 1 , 1, 4 }},
};
/*    (L = length of the oidsuffix) */

void
init_UIMobileApplicationCreateMessage(void)
{
    DEBUGMSGTL(("UIMobileApplicationCreateMessage", "Initializing\n"));
    REGISTER_MIB("UIMobileApplicationCreateMessage", UIMobileApplicationCreateMessage_variables, variable7,
               UIMobileApplicationCreateMessage_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}


unsigned char *
var_UIMobileApplicationCreateMessage(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_AppMACreateMessageTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UIMobileApplicationCreateMessage above.
 */
unsigned char *
var_AppMACreateMessageTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;

	register int index;
	index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_SMS_CREATEMESSAGE);
	w_index_app_sms_createmessage=index;
	
	if( index == MATCH_FAILED)
	 return NULL;


    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case APPMACREATEMESSAGEINDEX:
        long_ret = index;
        return (unsigned int *) &long_ret;
    case APPMACREATEMESSAGE:
        *write_method = write_AppMACreateMessage;
        memset(string,0,sizeof(string));
		read_csman(fd,0x801e2801+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case APPMACREATEMESSAGERECEIVER:
        *write_method = write_AppMACreateMessageReceiver;
       	memset(string,0,sizeof(string));
		read_csman(fd,0x801e2701+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case APPMACREATEMESSAGESENDTO:
        *write_method = write_AppMACreateMessageSendto;
        long_ret = 0;
        return (unsigned int *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


int
write_AppMACreateMessage(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIMobileApplicationCreateMessage not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIMobileApplicationCreateMessage: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			//for STR/MAC data type
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x801e2801+w_index_app_sms_createmessage-1,string,sizeof(string)+1 ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_AppMACreateMessageReceiver(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIMobileApplicationCreateMessage not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIMobileApplicationCreateMessage: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			//for STR/MAC data type
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x801e2701+w_index_app_sms_createmessage-1,string,sizeof(string)+1 ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_AppMACreateMessageSendto(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIMobileApplicationCreateMessage not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to UIMobileApplicationCreateMessage: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;


        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			char cmd_buf[256];
			sprintf(cmd_buf,"sms_server %d",w_index_app_sms_createmessage-1);
			system(cmd_buf);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
