#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UIPPTPClientStatus.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIPPTPClientStatus_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,1,5,3 };
#else
oid UIPPTPClientStatus_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,1,5,3 };
#endif

#define NUM_StatusPPTPClient 10

struct variable7 UIPPTPClientStatus_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{PPTPCLIENTSTATUSIDINDEX,ASN_INTEGER,RONLY,var_UIPPTPClientStatusTable, 3,  { 1 , 1, 1 }},
{PPTPCLIENTSTATUSCLIENTNAME,ASN_OCTET_STR,RONLY,var_UIPPTPClientStatusTable, 3,  { 1 , 1, 2 }},
{PPTPCLIENTSTATUSINTERFACE,ASN_OCTET_STR,RONLY,var_UIPPTPClientStatusTable, 3,  { 1 , 1, 3 }},
{PPTPCLIENTSTATUSVIRTUALIP,ASN_IPADDRESS,RONLY,var_UIPPTPClientStatusTable, 3,  { 1 , 1, 4 }},
{PPTPCLIENTSTATUSREMOTEIPFQDN,ASN_IPADDRESS,RONLY,var_UIPPTPClientStatusTable, 3,  { 1 , 1, 5 }},
{PPTPCLIENTSTATUSDEFAULTGWREMOTESUBNET,ASN_OCTET_STR,RONLY,var_UIPPTPClientStatusTable, 3,  { 1 , 1, 6 }},
{PPTPCLIENTSTATUS,ASN_INTEGER,RONLY,var_UIPPTPClientStatusTable, 3,  { 1 , 1, 7 }},
};
/*    (L = length of the oidsuffix) */


void
init_UIPPTPClientStatus(void)
{
    DEBUGMSGTL(("UIPPTPClientStatus", "Initializing\n"));
    REGISTER_MIB("UIPPTPClientStatus", UIPPTPClientStatus_variables, variable7,UIPPTPClientStatus_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UIPPTPClientStatus(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}

unsigned char *
var_UIPPTPClientStatusTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
	register int index;
	index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_StatusPPTPClient);
		
	if ( index == MATCH_FAILED )
		return NULL;


    switch(vp->magic) {
    case PPTPCLIENTSTATUSIDINDEX:
        long_ret = index;
        return (unsigned int *) &long_ret;
    case PPTPCLIENTSTATUSCLIENTNAME:
        memset(string,0,sizeof(string));
		read_csman(fd,0x00051200+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case PPTPCLIENTSTATUSINTERFACE:
		memset(string,0,sizeof(string));
		strcpy(string,"WAN 1 ");
        *var_len = strlen(string)-1;
		return (unsigned char *) string;
    case PPTPCLIENTSTATUSVIRTUALIP:
	{
        unsigned char virtual_ip[4];
		memset(virtual_ip,0,sizeof(virtual_ip));
		read_csman(fd,0x80052300+index-1,virtual_ip,sizeof(virtual_ip),TYPE_IPV4);
        *var_len = 4; //IP address length = 4
		return (unsigned char *) virtual_ip;
    }
    case PPTPCLIENTSTATUSREMOTEIPFQDN:
	{
        unsigned char remote_ip[4];
		memset(remote_ip,0,sizeof(remote_ip));
		read_csman(fd,0x80052600+index-1,remote_ip,sizeof(remote_ip),TYPE_IPV4);
        *var_len = 4; //IP address length = 4
		return (unsigned char *) remote_ip;
    }
    case PPTPCLIENTSTATUSDEFAULTGWREMOTESUBNET:
        memset(string,0,sizeof(string));
		read_csman(fd,0x00051600+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case PPTPCLIENTSTATUS:
        read_csman(fd,0x00051700+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}
