#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UIPhysicalInterfaceFailover.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_WAN_MULIT_FAILOVER 16
int w_index_bas_wan_multifailover;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIPhysicalInterfaceFailover_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,1,1 };
#else
oid UIPhysicalInterfaceFailover_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,1,1 };
#endif

struct variable7 UIPhysicalInterfaceFailover_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{BASICWANFAILOVERENABLE,ASN_INTEGER,RWRITE,var_UIPhysicalInterfaceFailover, 1,  { 1 }},
{BASICWANFAILOVERCHECKMODE,ASN_INTEGER,RWRITE,var_UIPhysicalInterfaceFailover, 1,  { 2 }},
{BASICWANFAILOVERCHECKPINGHOST,ASN_OCTET_STR,RWRITE,var_UIPhysicalInterfaceFailover, 1,  { 3 }},

{BASICWANMULTIFAILOVERINDEX,ASN_INTEGER,RONLY,var_BasicWANMultiFailoverTable, 3,  { 4 , 1, 1 }},
{BASICWANMULTIFAILOVERENABLE,ASN_INTEGER,RWRITE,var_BasicWANMultiFailoverTable, 3,  { 4 , 1, 2 }},
{BASICWANMULTIFAILOVERCHECKMODE,ASN_INTEGER,RWRITE,var_BasicWANMultiFailoverTable, 3,  { 4 , 1, 3 }},
{BASICWANMULTIFAILOVERCHECKPINGHOST,ASN_OCTET_STR,RWRITE,var_BasicWANMultiFailoverTable, 3,  { 4 , 1, 4 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UIPhysicalInterfaceFailover module */
void
init_UIPhysicalInterfaceFailover(void)
{
    DEBUGMSGTL(("UIPhysicalInterfaceFailover", "Initializing\n"));
    REGISTER_MIB("UIPhysicalInterfaceFailover", UIPhysicalInterfaceFailover_variables, variable7,
               UIPhysicalInterfaceFailover_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}


unsigned char *
var_UIPhysicalInterfaceFailover(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICWANFAILOVERENABLE:
        *write_method = write_BasicWANFailoverEnable;
        read_csman(fd,0x00430001,&long_ret,sizeof(unsigned int),TYPE_U32);	
	 	return (unsigned char *) &long_ret;
    case BASICWANFAILOVERCHECKMODE:
        *write_method = write_BasicWANFailoverCheckMode;
        read_csman(fd,0x0043000c,&long_ret,sizeof(unsigned int),TYPE_U32);	
	 	return (unsigned char *) &long_ret;
    case BASICWANFAILOVERCHECKPINGHOST:
        *write_method = write_BasicWANFailoverCheckPingHost;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00430002,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_BasicWANMultiFailoverTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UIPhysicalInterfaceFailover above.
 */
unsigned char *
var_BasicWANMultiFailoverTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    /* get index */
    register int index;
    index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_WAN_MULIT_FAILOVER);
    w_index_bas_wan_multifailover=index;

    if ( index == MATCH_FAILED )
        return NULL;


    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICWANMULTIFAILOVERINDEX:
        long_ret = index;
        return (unsigned int *) &long_ret;
    case BASICWANMULTIFAILOVERENABLE:
        *write_method = write_BasicWANMultiFailoverEnable;
        read_csman(fd,0x00431000+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
	 	return (unsigned char *) &long_ret;
    case BASICWANMULTIFAILOVERCHECKMODE:
        *write_method = write_BasicWANMultiFailoverCheckMode;
        read_csman(fd,0x00431020+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
	 	return (unsigned char *) &long_ret;
    case BASICWANMULTIFAILOVERCHECKPINGHOST:
        *write_method = write_BasicWANMultiFailoverCheckPingHost;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00431010+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    default:
      ERROR_MSG("");
    }
    return NULL;
}

int
write_BasicWANFailoverEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIPhysicalInterfaceFailover not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to UIPhysicalInterfaceFailover: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00430001,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicWANFailoverCheckMode(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIPhysicalInterfaceFailover not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIPhysicalInterfaceFailover: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x0043000c,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicWANFailoverCheckPingHost(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIPhysicalInterfaceFailover not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UIPhysicalInterfaceFailover: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			  memset(string,0,sizeof(string));
			  strncpy(string , (char *) var_val, var_val_len);
			  break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
         {
             int fd=0;
             write_csman(fd,0x00430002,string,strlen(string)+1,TYPE_STR);
             break;
	  	  }
    }
    return SNMP_ERR_NOERROR;
}

int
write_BasicWANMultiFailoverEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIPhysicalInterfaceFailover not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIPhysicalInterfaceFailover: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00431000+w_index_bas_wan_multifailover-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicWANMultiFailoverCheckMode(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIPhysicalInterfaceFailover not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIPhysicalInterfaceFailover: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00431020+w_index_bas_wan_multifailover-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}
int
write_BasicWANMultiFailoverCheckPingHost(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UIPhysicalInterfaceFailover not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UIPhysicalInterfaceFailover: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		  {
			  int fd=0;
			  write_csman(fd,0x00431010+w_index_bas_wan_multifailover-1,string,strlen(string),TYPE_STR);
			  break;
		  }
    }
    return SNMP_ERR_NOERROR;
}
