#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UIPhysicalInterfaceWAN2.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIPhysicalInterfaceWAN2_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,2,1,2,2 };
#else
oid UIPhysicalInterfaceWAN2_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,2,1,2,2 };
#endif

struct variable7 UIPhysicalInterfaceWAN2_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */


{BASICWAN2PHYSICALINTERFACE,ASN_INTEGER,RWRITE,var_UIPhysicalInterfaceWAN2, 1,  { 1 }},
{BASICWAN2OPERATIONMODE,ASN_INTEGER,RWRITE,var_UIPhysicalInterfaceWAN2, 1,  { 2 }},
{BASICWAN2LINESPEEDUP,ASN_INTEGER,RWRITE,var_UIPhysicalInterfaceWAN2, 1,  { 3 }},
{BASICWAN2LINESPEEDDOWN,ASN_INTEGER,RWRITE,var_UIPhysicalInterfaceWAN2, 1,  { 4 }},
{BASICWAN2VLANTAGENABLE,ASN_INTEGER,RWRITE,var_UIPhysicalInterfaceWAN2, 1,  { 5 }},
{BASICWAN2VLANTAGVALUE,ASN_INTEGER,RWRITE,var_UIPhysicalInterfaceWAN2, 1,  { 6 }},

};

void
init_UIPhysicalInterfaceWAN2(void)
{
    DEBUGMSGTL(("UIPhysicalInterfaceWAN2", "Initializing\n"));
    REGISTER_MIB("UIPhysicalInterfaceWAN2", UIPhysicalInterfaceWAN2_variables, variable7,
               UIPhysicalInterfaceWAN2_variables_oid);
}

unsigned char *
var_UIPhysicalInterfaceWAN2(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case BASICWAN2PHYSICALINTERFACE:
        *write_method = write_BasicWAN2PhysicalInterface;
        read_csman(fd,0x00010601,&long_ret,sizeof(unsigned int),TYPE_U32);
	 	return (unsigned char *) &long_ret;
    case BASICWAN2OPERATIONMODE:
	{
        int wan_index=0,c_failover_enable=0;
		read_csman(fd,0x00010601,&wan_index,sizeof(unsigned int),TYPE_U32);
		
		if(wan_index == 99){
			read_csman(fd,0x00430001,&c_failover_enable,sizeof(unsigned int),TYPE_U32);	
			if(c_failover_enable > 1 || c_failover_enable == 0)	long_ret = 0;	
			else if(c_failover_enable == 1)	long_ret = 1;
		}
		else if(wan_index == 0)	long_ret=2;
		else{
			read_csman(fd,0x00430001,&c_failover_enable,sizeof(unsigned int),TYPE_U32);	

			if(c_failover_enable == 0)	long_ret=0;
			else{
				if(wan_index == c_failover_enable)	long_ret=1;
				else{
					int failover_check=0;
					if(wan_index == 14 || wan_index == 15 || wan_index == 16){
						read_csman(fd,0x00431000+wan_index,&failover_check,sizeof(unsigned int),TYPE_U32);
						if(wan_index == failover_check)	long_ret=1;
						else	long_ret=2;
					}
					else{ 
						read_csman(fd,0x00431000+wan_index-1,&failover_check,sizeof(unsigned int),TYPE_U32);
						if(wan_index == failover_check)	long_ret=1;
						else	long_ret=2;
					}
				}
			}
		}
		*write_method = write_BasicWAN2OperationMode;
        //read_csman(fd,0x00010611,&long_ret,sizeof(unsigned int),TYPE_U32);
	 	return (unsigned char *) &long_ret;
    }
    case BASICWAN2LINESPEEDUP:
	{
		int value=0;
        *write_method = write_BasicWAN2LineSpeedUp;
        read_csman(fd,0x00120021,&value,sizeof(unsigned int),TYPE_U32);
		long_ret = value / 1024;
	 	return (unsigned char *) &long_ret;
    }
    case BASICWAN2LINESPEEDDOWN:
	{
		int value=0;
        *write_method = write_BasicWAN2LineSpeedDown;
        read_csman(fd,0x00120031,&value,sizeof(unsigned int),TYPE_U32);
		long_ret = value / 1024;
	 	return (unsigned char *) &long_ret;
    }
    case BASICWAN2VLANTAGENABLE:
        *write_method = write_BasicWAN2VlanTagEnable;
        read_csman(fd,0x009320c2,&long_ret,sizeof(unsigned int),TYPE_U32);
	 	return (unsigned char *) &long_ret;
    case BASICWAN2VLANTAGVALUE:
        *write_method = write_BasicWAN2VlanTagvalue;
        read_csman(fd,0x00932002,&long_ret,sizeof(unsigned int),TYPE_U32);
	 	return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}

int
write_BasicWAN2PhysicalInterface(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIPhysicalInterfaceWAN2 not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIPhysicalInterfaceWAN2: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x00010601,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicWAN2OperationMode(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIPhysicalInterfaceWAN2 not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIPhysicalInterfaceWAN2: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0,wan_index=0;
			  int def_value=0;
			  read_csman(fd,0x00010601,&wan_index,sizeof(unsigned int),TYPE_U32);

			  if(long_ret == 1){
			  	if(wan_index == 14 || wan_index == 15 || wan_index == 16){
			  		write_csman(fd,0x00430001,&wan_index,sizeof(unsigned int) ,TYPE_U32);
					write_csman(fd,0x00431000+wan_index,&wan_index,sizeof(unsigned int) ,TYPE_U32);
			  	}
			  	else{
					write_csman(fd,0x00430001,&wan_index,sizeof(unsigned int) ,TYPE_U32);
					write_csman(fd,0x00431000+wan_index-1,&wan_index,sizeof(unsigned int) ,TYPE_U32);			  	
			  	}
			  }
			  else if(long_ret == 0){
				write_csman(fd,0x00430001,&def_value,sizeof(unsigned int) ,TYPE_U32);
				if(wan_index == 14 || wan_index == 15 || wan_index == 16){
			  		write_csman(fd,0x00430001,&def_value,sizeof(unsigned int) ,TYPE_U32);
					write_csman(fd,0x00431000+wan_index,&def_value,sizeof(unsigned int) ,TYPE_U32);
			  	}
				else{
					write_csman(fd,0x00430001,&def_value,sizeof(unsigned int) ,TYPE_U32);
					write_csman(fd,0x00431000+wan_index-1,&def_value,sizeof(unsigned int) ,TYPE_U32);
				}
			  }
			  else if(long_ret == 2){
			  	write_csman(fd,0x00010601,&def_value,sizeof(unsigned int) ,TYPE_U32);
				if(wan_index == 14 || wan_index == 15 || wan_index == 16){
			  		//write_csman(fd,0x00430001,&def_value,sizeof(unsigned int) ,TYPE_U32);
					write_csman(fd,0x00431000+wan_index,&def_value,sizeof(unsigned int) ,TYPE_U32);
			  	}
				else{
					//write_csman(fd,0x00430001,&def_value,sizeof(unsigned int) ,TYPE_U32);
					write_csman(fd,0x00431000+wan_index-1,&def_value,sizeof(unsigned int) ,TYPE_U32);
				}
			  }
              //write_csman(fd,0x00010611,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicWAN2LineSpeedUp(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIPhysicalInterfaceWAN2 not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIPhysicalInterfaceWAN2: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0,value=0;
			  value = long_ret * 1024;
              write_csman(fd,0x00120021,&value,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicWAN2LineSpeedDown(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIPhysicalInterfaceWAN2 not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIPhysicalInterfaceWAN2: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0,value=0;
			  value = long_ret * 1024;
              write_csman(fd,0x00120031,&value,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicWAN2VlanTagEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIPhysicalInterfaceWAN2 not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIPhysicalInterfaceWAN2: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
              write_csman(fd,0x009320c2,&long_ret,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}


int
write_BasicWAN2VlanTagvalue(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UIPhysicalInterfaceWAN2 not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UIPhysicalInterfaceWAN2: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
          {
              int fd=0;
			  unsigned int value=0;
			  value = long_ret;
              write_csman(fd,0x00932002,&value,sizeof(unsigned int) ,TYPE_U32);
              break;
          }
    }
    return SNMP_ERR_NOERROR;
}

