#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UISNMPLinkingStatus.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_STATUS_SNMP_LINKING 30

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UISNMPLinkingStatus_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,1,6,1 };
#else
oid UISNMPLinkingStatus_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,1,6,1 };
#endif

struct variable7 UISNMPLinkingStatus_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{STATUSSNMPINDEX,ASN_INTEGER,RONLY,var_UISNMPlinkingStatusTable, 3,  { 1 , 1, 1 }},
{STATUSSNMPUSERNAME,ASN_OCTET_STR,RONLY,var_UISNMPlinkingStatusTable, 3,  { 1 , 1, 2 }},
{STATUSSNMPIPADDRESS,ASN_OCTET_STR,RONLY,var_UISNMPlinkingStatusTable, 3,  { 1 , 1, 3 }},
{STATUSSNMPPORT,ASN_INTEGER,RONLY,var_UISNMPlinkingStatusTable, 3,  { 1 , 1, 4 }},
{STATUSSNMPCOMMUNITY,ASN_OCTET_STR,RONLY,var_UISNMPlinkingStatusTable, 3,  { 1 , 1, 5 }},
{STATUSSNMPAUTHMODE,ASN_INTEGER,RONLY,var_UISNMPlinkingStatusTable, 3,  { 1 , 1, 6 }},
{STATUSSNMPPRIVACYMODE,ASN_INTEGER,RONLY,var_UISNMPlinkingStatusTable, 3,  { 1 , 1, 7 }},
{STATUSSNMPSNMPVERSION,ASN_INTEGER,RONLY,var_UISNMPlinkingStatusTable, 3,  { 1 , 1, 8 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UISNMPLinkingStatus module */
void
init_UISNMPLinkingStatus(void)
{
    DEBUGMSGTL(("UISNMPLinkingStatus", "Initializing\n"));
    REGISTER_MIB("UISNMPLinkingStatus", UISNMPLinkingStatus_variables, variable7,UISNMPLinkingStatus_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UISNMPLinkingStatus(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_UISNMPlinkingStatusTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UISNMPLinkingStatus above.
 */
unsigned char *
var_UISNMPlinkingStatusTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    
   	register int index;
	index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_STATUS_SNMP_LINKING);
		
	if ( index == MATCH_FAILED )
		return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case STATUSSNMPINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case STATUSSNMPUSERNAME:
        memset(string,0,sizeof(string));
		read_csman(fd,0x80165300+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case STATUSSNMPIPADDRESS:
        memset(string,0,sizeof(string));
		read_csman(fd,0x80165800+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case STATUSSNMPPORT:
        read_csman(fd,0x80165900+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case STATUSSNMPCOMMUNITY:
        memset(string,0,sizeof(string));
		read_csman(fd,0x80165200+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case STATUSSNMPAUTHMODE:
        read_csman(fd,0x80165600+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case STATUSSNMPPRIVACYMODE:
        read_csman(fd,0x80165500+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case STATUSSNMPSNMPVERSION:
        read_csman(fd,0x80165100+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


