#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UISNMPTrapInformation.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_STATUS_SNMP_TRAP 30


#ifdef SNMP_CUSTOMER_SMARTLINK
oid UISNMPTrapInformation_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,1,6,2 };
#else
oid UISNMPTrapInformation_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,1,6,2 };
#endif


struct variable7 UISNMPTrapInformation_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{STATUSSNMPTRAPINDEX,ASN_INTEGER,RONLY,var_StatusSNMPTrapTable, 3,  { 1 , 1, 1 }},
{STATUSSNMPTRAPLEVEL,ASN_INTEGER,RONLY,var_StatusSNMPTrapTable, 3,  { 1 , 1, 2 }},
{STATUSSNMPTRAPTIME,ASN_OCTET_STR,RONLY,var_StatusSNMPTrapTable, 3,  { 1 , 1, 3 }},
{STATUSSNMPTRAPEVENT,ASN_OCTET_STR,RONLY,var_StatusSNMPTrapTable, 3,  { 1 , 1, 4 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UISNMPTrapInformation module */
void
init_UISNMPTrapInformation(void)
{
    DEBUGMSGTL(("UISNMPTrapInformation", "Initializing\n"));
    REGISTER_MIB("UISNMPTrapInformation", UISNMPTrapInformation_variables, variable7,
               UISNMPTrapInformation_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UISNMPTrapInformation(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}

unsigned char *
var_StatusSNMPTrapTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    
   	register int index;
	index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_STATUS_SNMP_TRAP);
		
	if ( index == MATCH_FAILED )
		return NULL;

    switch(vp->magic) {
    case STATUSSNMPTRAPINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case STATUSSNMPTRAPLEVEL:
        read_csman(fd,0x80164100+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case STATUSSNMPTRAPTIME:
        memset(string,0,sizeof(string));
		read_csman(fd,0x80164300+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case STATUSSNMPTRAPEVENT:
        memset(string,0,sizeof(string));
		read_csman(fd,0x80164200+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


