#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UISystemExternalServers.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>


#define NUM_EXTERNAL_SERVERS	64

int w_index_external_servers;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UISystemExternalServers_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,5,9,1 };
#else
oid UISystemExternalServers_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,5,9,1 };
#endif

struct variable7 UISystemExternalServers_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{SYSTEMEXTERNALSERVERINDEX,ASN_INTEGER,RONLY,var_SystemExternalServersTable, 3,  { 1 , 1, 1 }},
{SYSTEMEXTERNALSERVERNAME,ASN_OCTET_STR,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 2 }},
{SYSTEMEXTERNALSERVERIPFQDN,ASN_OCTET_STR,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 3 }},
{SYSTEMEXTERNALSERVERPORT,ASN_INTEGER,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 4 }},
{SYSTEMEXTERNALSERVERTYPE,ASN_INTEGER,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 5 }},
{SYSTEMEXTERNALSERVERENABLE,ASN_INTEGER,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 6 }},
{SYSTEMEXTERNALSERVEREMAILUSERNAME,ASN_OCTET_STR,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 7 }},
{SYSTEMEXTERNALSERVEREMAILPASSWORD,ASN_OCTET_STR,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 8 }},
{SYSTEMEXTERNALSERVERRADIUSPRIKEY,ASN_OCTET_STR,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 9 }},
{SYSTEMEXTERNALSERVERRADIUSPRIAUTHPROTOCOL,ASN_INTEGER,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 10 }},
{SYSTEMEXTERNALSERVERRADIUSPRIACCOUNTINGPORT,ASN_INTEGER,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 11 }},
{SYSTEMEXTERNALSERVERRADIUSPRISESSIONTIMEOUT,ASN_INTEGER,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 12 }},
{SYSTEMEXTERNALSERVERRADIUSPRIIDLETIMEOUT,ASN_INTEGER,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 13 }},
{SYSTEMEXTERNALSERVERRADIUSSECKEY,ASN_OCTET_STR,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 14 }},
{SYSTEMEXTERNALSERVERRADIUSSECAUTHPROTOCOL,ASN_INTEGER,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 15 }},
{SYSTEMEXTERNALSERVERRADIUSSECACCOUNTINGPORT,ASN_INTEGER,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 16 }},
{SYSTEMEXTERNALSERVERRADIUSSECSESSIONTIMEOUT,ASN_INTEGER,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 17 }},
{SYSTEMEXTERNALSERVERRADIUSSECIDLETIMEOUT,ASN_INTEGER,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 18 }},
{SYSTEMEXTERNALSERVERADDOMAIN, ASN_OCTET_STR,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 19 }},
{SYSTEMEXTERNALSERVERLDAPBASEDN,ASN_OCTET_STR,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 20 }},
{SYSTEMEXTERNALSERVERNTDOMAINWORKGROUP,ASN_OCTET_STR,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 21 }},
{SYSTEMEXTERNALSERVERUAMLOGINURL,ASN_OCTET_STR,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 22 }},
{SYSTEMEXTERNALSERVERUAMSHAREDSECRCT,ASN_OCTET_STR,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 23 }},
{SYSTEMEXTERNALSERVERUAMNASGATEWAYID,ASN_OCTET_STR,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 24 }},
{SYSTEMEXTERNALSERVERUAMLOCATIONID,ASN_INTEGER,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 25 }},
{SYSTEMEXTERNALSERVERUAMLOCATIONNAME,ASN_OCTET_STR,RWRITE,var_SystemExternalServersTable, 3,  { 1 , 1, 26 }},

};
/*    (L = length of the oidsuffix) */


/** Initializes the UISystemExternalServers module */
void
init_UISystemExternalServers(void)
{
    DEBUGMSGTL(("UISystemExternalServers", "Initializing\n"));
    REGISTER_MIB("UISystemExternalServers", UISystemExternalServers_variables, variable7,
               UISystemExternalServers_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UISystemExternalServers(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_SystemExternalServersTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UISystemExternalServers above.
 */
unsigned char *
var_SystemExternalServersTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	
	int fd=0;
	register int index;
	index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_EXTERNAL_SERVERS);
	w_index_external_servers=index;
	if ( index == MATCH_FAILED )
		return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case SYSTEMEXTERNALSERVERINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case SYSTEMEXTERNALSERVERNAME:
        *write_method = write_SystemExternalServerName;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00b10000+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMEXTERNALSERVERIPFQDN:
        *write_method = write_SystemExternalServerIPFQDN;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00b10040+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMEXTERNALSERVERPORT:
        *write_method = write_SystemExternalServerPort;
        read_csman(fd,0x00b10090+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMEXTERNALSERVERTYPE:
        *write_method = write_SystemExternalServerType;
        read_csman(fd,0x00b100c0+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMEXTERNALSERVERENABLE:
        *write_method = write_SystemExternalServerEnable;
        read_csman(fd,0x00b10100+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMEXTERNALSERVEREMAILUSERNAME:
        *write_method = write_SystemExternalServerEmailUserName;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00b10140+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMEXTERNALSERVEREMAILPASSWORD:
        *write_method = write_SystemExternalServerEmailPassWord;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00b10180+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMEXTERNALSERVERRADIUSPRIKEY:
        *write_method = write_SystemExternalServerRadiusPriKey;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00b101c0+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMEXTERNALSERVERRADIUSPRIAUTHPROTOCOL:
        *write_method = write_SystemExternalServerRadiusPriAuthProtocol;
        read_csman(fd,0x00b10200+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMEXTERNALSERVERRADIUSPRIACCOUNTINGPORT:
        *write_method = write_SystemExternalServerRadiusPriAccountingPort;
        read_csman(fd,0x00b10240+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMEXTERNALSERVERRADIUSPRISESSIONTIMEOUT:
        *write_method = write_SystemExternalServerRadiusPriSessionTimeout;
        read_csman(fd,0x00b10280+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMEXTERNALSERVERRADIUSPRIIDLETIMEOUT:
        *write_method = write_SystemExternalServerRadiusPriIdleTimeout;
        read_csman(fd,0x00b102c0+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMEXTERNALSERVERRADIUSSECKEY:
        *write_method = write_SystemExternalServerRadiusSecKey;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00b10300+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMEXTERNALSERVERRADIUSSECAUTHPROTOCOL:
        *write_method = write_SystemExternalServerRadiusSecAuthProtocol;
        read_csman(fd,0x00b10340+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMEXTERNALSERVERRADIUSSECACCOUNTINGPORT:
        *write_method = write_SystemExternalServerRadiusSecAccountingPort;
        read_csman(fd,0x00b10380+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMEXTERNALSERVERRADIUSSECSESSIONTIMEOUT:
        *write_method = write_SystemExternalServerRadiusSecSessionTimeout;
        read_csman(fd,0x00b103c0+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMEXTERNALSERVERRADIUSSECIDLETIMEOUT:
        *write_method = write_SystemExternalServerRadiusSecIdleTimeout;
        read_csman(fd,0x00b10400+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMEXTERNALSERVERADDOMAIN:
        *write_method = write_SystemExternalServerADDomain;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00b10440+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMEXTERNALSERVERLDAPBASEDN:
        *write_method = write_SystemExternalServerLDAPBaseDN;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00b10480+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMEXTERNALSERVERNTDOMAINWORKGROUP:
        *write_method = write_SystemExternalServerNTDomainWorkGroup;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00b104c0+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMEXTERNALSERVERUAMLOGINURL:
        *write_method = write_SystemExternalServerUAMLoginURL;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00b10680+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMEXTERNALSERVERUAMSHAREDSECRCT:
        *write_method = write_SystemExternalServerUAMSharedSecrct;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00b10500+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMEXTERNALSERVERUAMNASGATEWAYID:
        *write_method = write_SystemExternalServerUAMNASGatewayID;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00b10540+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMEXTERNALSERVERUAMLOCATIONID:
        *write_method = write_SystemExternalServerUAMLocationID;
        read_csman(fd,0x00b10600+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMEXTERNALSERVERUAMLOCATIONNAME:
        *write_method = write_SystemExternalServerUAMLocationName;
        memset(string,0,sizeof(string));
		read_csman(fd,0x00b10640+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


int
write_SystemExternalServerName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00b10000 + w_index_external_servers - 1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerIPFQDN(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00b10040+ w_index_external_servers - 1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerPort(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
			int fd=0;
          	write_csman(fd,0x00b10090+ w_index_external_servers - 1,&long_ret,sizeof(unsigned int) ,TYPE_U32);    
          	break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerType(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
			int fd=0;
          	write_csman(fd,0x00b100c0+ w_index_external_servers - 1,&long_ret,sizeof(unsigned int) ,TYPE_U32);    
          	break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
 			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
			int fd=0;
          	write_csman(fd,0x00b10100+ w_index_external_servers - 1,&long_ret,sizeof(unsigned int) ,TYPE_U32);    
          	break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerEmailUserName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00b10140+ w_index_external_servers - 1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerEmailPassWord(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00b10180+ w_index_external_servers - 1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerRadiusPriKey(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00b101c0+ w_index_external_servers - 1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerRadiusPriAuthProtocol(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
			int fd=0;
          	write_csman(fd,0x00b10200+ w_index_external_servers - 1,&long_ret,sizeof(unsigned int) ,TYPE_U32);    
          	break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerRadiusPriAccountingPort(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
			int fd=0;
          	write_csman(fd,0x00b10240+ w_index_external_servers - 1,&long_ret,sizeof(unsigned int) ,TYPE_U32);    
          	break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerRadiusPriSessionTimeout(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
			int fd=0;
          	write_csman(fd,0x00b10280+ w_index_external_servers - 1,&long_ret,sizeof(unsigned int) ,TYPE_U32);    
          	break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerRadiusPriIdleTimeout(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
			int fd=0;
          	write_csman(fd,0x00b102c0+ w_index_external_servers - 1,&long_ret,sizeof(unsigned int) ,TYPE_U32);    
          	break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerRadiusSecKey(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00b10300+ w_index_external_servers - 1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerRadiusSecAuthProtocol(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
			int fd=0;
          	write_csman(fd,0x00b10340+ w_index_external_servers - 1,&long_ret,sizeof(unsigned int) ,TYPE_U32);    
          	break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerRadiusSecAccountingPort(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
			int fd=0;
          	write_csman(fd,0x00b10380+ w_index_external_servers - 1,&long_ret,sizeof(unsigned int) ,TYPE_U32);    
          	break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerRadiusSecSessionTimeout(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
			int fd=0;
          	write_csman(fd,0x00b103c0+ w_index_external_servers - 1,&long_ret,sizeof(unsigned int) ,TYPE_U32);    
          	break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerRadiusSecIdleTimeout(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
			int fd=0;
          	write_csman(fd,0x00b10400+ w_index_external_servers - 1,&long_ret,sizeof(unsigned int) ,TYPE_U32);    
          	break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerADDomain(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00b10440+ w_index_external_servers - 1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerLDAPBaseDN(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00b10480+ w_index_external_servers - 1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerNTDomainWorkGroup(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00b104c0+ w_index_external_servers - 1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerUAMLoginURL(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00b10680+ w_index_external_servers - 1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerUAMSharedSecrct(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00b10500+ w_index_external_servers - 1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerUAMNASGatewayID(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00b10540+ w_index_external_servers - 1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerUAMLocationID(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
        {
			int fd=0;
          	write_csman(fd,0x00b10600+ w_index_external_servers - 1,&long_ret,sizeof(unsigned int) ,TYPE_U32);    
          	break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemExternalServerUAMLocationName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemExternalServers not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemExternalServers: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x00b10640+ w_index_external_servers - 1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
