#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UISystemGroupingFileExtAdd.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_GROUPING_FILEEXTADD	32
int w_index_grouping_fileextadd;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UISystemGroupingFileExtAdd_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,5,8,3,1 };
#else
oid UISystemGroupingFileExtAdd_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,5,8,3,1 };
#endif

struct variable7 UISystemGroupingFileExtAdd_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */
{SYSTEMFILEEXTGROUPINGADDALTER,ASN_INTEGER,RWRITE,var_UISystemGroupingFileExtAdd, 1,  { 1 }},
{SYSTEMFILEEXTGROUPINGADDGROUPNAME,ASN_OCTET_STR,RWRITE,var_UISystemGroupingFileExtAdd, 1,  { 2 }},
{SYSTEMFILEEXTGROUPINGADDGROUPMEMBERLISTINDEX,ASN_INTEGER,RONLY,var_SystemFileExtGroupingAddTable, 3,  { 3 , 1, 1 }},
{SYSTEMFILEEXTGROUPINGADDGROUPMEMBERLISTTYPE,ASN_OCTET_STR,RWRITE,var_SystemFileExtGroupingAddTable, 3,  { 3 , 1, 2 }},
{SYSTEMFILEEXTGROUPINGADDGROUPMEMBERLISTFORMAT,ASN_OCTET_STR,RWRITE,var_SystemFileExtGroupingAddTable, 3,  { 3 , 1, 3 }},
{SYSTEMFILEEXTGROUPINGADDMUTIPLEBOUNDSERVICEFIREWALL,ASN_INTEGER,RWRITE,var_UISystemGroupingFileExtAdd, 1,  { 4 }},
{SYSTEMFILEEXTGROUPINGADDMUTIPLEBOUNDSERVICEQOS,ASN_INTEGER,RWRITE,var_UISystemGroupingFileExtAdd, 1,  { 5 }},
{SYSTEMFILEEXTGROUPINGADDENABLE,ASN_INTEGER,RWRITE,var_UISystemGroupingFileExtAdd, 1,  { 10 }},

};
/*    (L = length of the oidsuffix) */


/** Initializes the UISystemGroupingFileExtAdd module */
void
init_UISystemGroupingFileExtAdd(void)
{
    DEBUGMSGTL(("UISystemGroupingFileExtAdd", "Initializing\n"));
    REGISTER_MIB("UISystemGroupingFileExtAdd", UISystemGroupingFileExtAdd_variables, variable7,
               UISystemGroupingFileExtAdd_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     18
	#else
	#define IFENTRY_NAME_LENGTH     17
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UISystemGroupingFileExtAdd(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case SYSTEMFILEEXTGROUPINGADDALTER:
        *write_method = write_SystemFileExtGroupingAddAlter;
        read_csman(fd,0x809f0105,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMFILEEXTGROUPINGADDGROUPNAME:
        *write_method = write_SystemFileExtGroupingAddGroupName;
        memset(string,0,sizeof(string));
		read_csman(fd,0x809f5000,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMFILEEXTGROUPINGADDMUTIPLEBOUNDSERVICEFIREWALL:
        *write_method = write_SystemFileExtGroupingAddMutipleBoundServiceFirewall;
        read_csman(fd,0x809f5090,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMFILEEXTGROUPINGADDMUTIPLEBOUNDSERVICEQOS:
        *write_method = write_SystemFileExtGroupingAddMutipleBoundServiceQoS;
        read_csman(fd,0x809f5091,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMFILEEXTGROUPINGADDENABLE:
        *write_method = write_SystemFileExtGroupingAddEnable;
        read_csman(fd,0x809f5092,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_SystemFileExtGroupingAddTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UISystemGroupingFileExtAdd above.
 */
unsigned char *
var_SystemFileExtGroupingAddTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    register int index;
	index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_GROUPING_FILEEXTADD);
	w_index_grouping_fileextadd=index;
	if ( index == MATCH_FAILED )
		return NULL;


    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case SYSTEMFILEEXTGROUPINGADDGROUPMEMBERLISTINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case SYSTEMFILEEXTGROUPINGADDGROUPMEMBERLISTTYPE:
        *write_method = write_SystemFileExtGroupingAddGroupMemberListType;
        memset(string,0,sizeof(string));
		read_csman(fd,0x809f5010+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMFILEEXTGROUPINGADDGROUPMEMBERLISTFORMAT:
        *write_method = write_SystemFileExtGroupingAddGroupMemberListFormat;
        memset(string,0,sizeof(string));
		read_csman(fd,0x809f5050+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    default:
      ERROR_MSG("");
    }
    return NULL;
}



int
write_SystemFileExtGroupingAddAlter(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingFileExtAdd not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingFileExtAdd: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f0105,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			write_csman(fd,0x809f0100,&long_ret,sizeof(unsigned int) ,TYPE_U32);
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemFileExtGroupingAddGroupName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];
	
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemGroupingFileExtAdd not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemGroupingFileExtAdd: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f5000,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemFileExtGroupingAddMutipleBoundServiceFirewall(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingFileExtAdd not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingFileExtAdd: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f5090,&long_ret,sizeof(unsigned int) ,TYPE_U32);
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemFileExtGroupingAddMutipleBoundServiceQoS(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingFileExtAdd not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingFileExtAdd: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f5091,&long_ret,sizeof(unsigned int) ,TYPE_U32);
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemFileExtGroupingAddEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingFileExtAdd not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingFileExtAdd: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f5092,&long_ret,sizeof(unsigned int) ,TYPE_U32);
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}

int
write_SystemFileExtGroupingAddGroupMemberListType(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemGroupingFileExtAdd not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UISystemGroupingFileExtAdd: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f5010+w_index_grouping_fileextadd-1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemFileExtGroupingAddGroupMemberListFormat(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemGroupingFileExtAdd not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UISystemGroupingFileExtAdd: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f5050+w_index_grouping_fileextadd-1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
