#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UISystemGroupingFileExtMod.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_GROUPING_FILEEXTMOD	32
int w_index_grouping_fileextmod;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UISystemGroupingFileExtMod_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,5,8,3,3 };
#else
oid UISystemGroupingFileExtMod_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,5,8,3,3 };
#endif

struct variable7 UISystemGroupingFileExtMod_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{SYSTEMFILEEXTGROUPINGMODALTER,ASN_INTEGER,RWRITE,var_UISystemGroupingFileExtMod, 1,  { 1 }},
{SYSTEMFILEEXTGROUPINGMODGROUPNAMEORIGINAL,ASN_OCTET_STR,RWRITE,var_UISystemGroupingFileExtMod, 1,  { 2 }},
{SYSTEMFILEEXTGROUPINGMODGROUPNAME,ASN_OCTET_STR,RWRITE,var_UISystemGroupingFileExtMod, 1,  { 3 }},
{SYSTEMFILEEXTGROUPINGMODGROUPMEMBERLISTINDEX,ASN_INTEGER,RONLY,var_SystemFileExtGroupingModTable, 3,  { 4 , 1, 1 }},
{SYSTEMFILEEXTGROUPINGMODGROUPMEMBERLISTTYPE,ASN_OCTET_STR,RWRITE,var_SystemFileExtGroupingModTable, 3,  { 4 , 1, 2 }},
{SYSTEMFILEEXTGROUPINGMODGROUPMEMBERLISTFORMAT,ASN_OCTET_STR,RWRITE,var_SystemFileExtGroupingModTable, 3,  { 4 , 1, 3 }},
{SYSTEMFILEEXTGROUPINGMODMUTIPLEBOUNDSERVICEFIREWALL,ASN_INTEGER,RWRITE,var_UISystemGroupingFileExtMod, 1,  { 5 }},
{SYSTEMFILEEXTGROUPINGMODMUTIPLEBOUNDSERVICEQOS,ASN_INTEGER,RWRITE,var_UISystemGroupingFileExtMod, 1,  { 6 }},
{SYSTEMFILEEXTGROUPINGMODENABLE,ASN_INTEGER,RWRITE,var_UISystemGroupingFileExtMod, 1,  { 10 }},
};

void
init_UISystemGroupingFileExtMod(void)
{
    DEBUGMSGTL(("UISystemGroupingFileExtMod", "Initializing\n"));
    REGISTER_MIB("UISystemGroupingFileExtMod", UISystemGroupingFileExtMod_variables, variable7,
               UISystemGroupingFileExtMod_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     18
	#else
	#define IFENTRY_NAME_LENGTH     17
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UISystemGroupingFileExtMod(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case SYSTEMFILEEXTGROUPINGMODALTER:
        *write_method = write_SystemFileExtGroupingModAlter;
        read_csman(fd,0x809f0107,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMFILEEXTGROUPINGMODGROUPNAMEORIGINAL:
        *write_method = write_SystemFileExtGroupingModGroupNameOriginal;
        memset(string,0,sizeof(string));
		read_csman(fd,0x809f6000,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMFILEEXTGROUPINGMODGROUPNAME:
        *write_method = write_SystemFileExtGroupingModGroupName;
        memset(string,0,sizeof(string));
		read_csman(fd,0x809f6001,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMFILEEXTGROUPINGMODMUTIPLEBOUNDSERVICEFIREWALL:
        *write_method = write_SystemFileExtGroupingModMutipleBoundServiceFirewall;
        read_csman(fd,0x809f6090,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMFILEEXTGROUPINGMODMUTIPLEBOUNDSERVICEQOS:
        *write_method = write_SystemFileExtGroupingModMutipleBoundServiceQoS;
        read_csman(fd,0x809f6091,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMFILEEXTGROUPINGMODENABLE:
        *write_method = write_SystemFileExtGroupingModEnable;
        read_csman(fd,0x809f6092,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_SystemFileExtGroupingModTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UISystemGroupingFileExtMod above.
 */
unsigned char *
var_SystemFileExtGroupingModTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    register int index;
	index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_GROUPING_FILEEXTMOD);
	w_index_grouping_fileextmod=index;
	if ( index == MATCH_FAILED )
		return NULL;


    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case SYSTEMFILEEXTGROUPINGMODGROUPMEMBERLISTINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case SYSTEMFILEEXTGROUPINGMODGROUPMEMBERLISTTYPE:
        *write_method = write_SystemFileExtGroupingModGroupMemberListType;
        memset(string,0,sizeof(string));
		read_csman(fd,0x809f6010+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMFILEEXTGROUPINGMODGROUPMEMBERLISTFORMAT:
        *write_method = write_SystemFileExtGroupingModGroupMemberListFormat;
        memset(string,0,sizeof(string));
		read_csman(fd,0x809f6050+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    default:
      ERROR_MSG("");
    }
    return NULL;
}



int
write_SystemFileExtGroupingModAlter(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingFileExtMod not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingFileExtMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f0107,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			write_csman(fd,0x809f0100,&long_ret,sizeof(unsigned int) ,TYPE_U32);
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemFileExtGroupingModGroupNameOriginal(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemGroupingFileExtMod not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemGroupingFileExtMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f6000,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemFileExtGroupingModGroupName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemGroupingFileExtMod not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemGroupingFileExtMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f6001,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemFileExtGroupingModMutipleBoundServiceFirewall(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingFileExtMod not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingFileExtMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f6090,&long_ret,sizeof(unsigned int) ,TYPE_U32);
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemFileExtGroupingModMutipleBoundServiceQoS(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingFileExtMod not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingFileExtMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;
			
        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f6091,&long_ret,sizeof(unsigned int) ,TYPE_U32);
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemFileExtGroupingModEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingFileExtMod not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingFileExtMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f6092,&long_ret,sizeof(unsigned int) ,TYPE_U32);
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}

int
write_SystemFileExtGroupingModGroupMemberListType(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemGroupingFileExtMod not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemGroupingFileExtMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f6010+w_index_grouping_fileextmod-1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemFileExtGroupingModGroupMemberListFormat(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemGroupingFileExtMod not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemGroupingFileExtMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f6050+w_index_grouping_fileextmod-1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
