#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UISystemGroupingHostGroupingMod.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_GROUPING_HOSTGROUPINGMOD	32
int w_index_grouping_hostgroupingmod;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UISystemGroupingHostGroupingMod_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,5,8,2,3 };
#else
oid UISystemGroupingHostGroupingMod_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,5,8,2,3 };
#endif

struct variable7 UISystemGroupingHostGroupingMod_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{SYSTEMHOSTGROUPINGMODALTER,ASN_INTEGER,RWRITE,var_UISystemGroupingHostGroupingMod, 1,  { 1 }},
{SYSTEMHOSTGROUPINGMODGROUPNAMEORIGINAL,ASN_OCTET_STR,RWRITE,var_UISystemGroupingHostGroupingMod, 1,  { 2 }},
{SYSTEMHOSTGROUPINGMODGROUPNAME,ASN_OCTET_STR,RWRITE,var_UISystemGroupingHostGroupingMod, 1,  { 3 }},
{SYSTEMHOSTGROUPINGMODGROUPMEMBERLISTINDEX,ASN_INTEGER,RONLY,var_SystemHostGroupingModTable, 3,  { 4 , 1, 1 }},
{SYSTEMHOSTGROUPINGMODGROUPMEMBERLISTTYPE,ASN_OCTET_STR,RWRITE,var_SystemHostGroupingModTable, 3,  { 4 , 1, 2 }},
{SYSTEMHOSTGROUPINGMODGROUPMEMBERLISTID,ASN_OCTET_STR,RWRITE,var_SystemHostGroupingModTable, 3,  { 4 , 1, 3 }},
{SYSTEMHOSTGROUPINGMODMUTIPLEBOUNDSERVICEFIREWALL,ASN_INTEGER,RWRITE,var_UISystemGroupingHostGroupingMod, 1,  { 5 }},
{SYSTEMHOSTGROUPINGMODMUTIPLEBOUNDSERVICEQOS,ASN_INTEGER,RWRITE,var_UISystemGroupingHostGroupingMod, 1,  { 6 }},
{SYSTEMHOSTGROUPINGMODENABLE,ASN_INTEGER,RWRITE,var_UISystemGroupingHostGroupingMod, 1,  { 10 }},

};

void
init_UISystemGroupingHostGroupingMod(void)
{
    DEBUGMSGTL(("UISystemGroupingHostGroupingMod", "Initializing\n"));
    REGISTER_MIB("UISystemGroupingHostGroupingMod", UISystemGroupingHostGroupingMod_variables, variable7,
               UISystemGroupingHostGroupingMod_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     18
	#else
	#define IFENTRY_NAME_LENGTH     17
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UISystemGroupingHostGroupingMod(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case SYSTEMHOSTGROUPINGMODALTER:
        *write_method = write_SystemHostGroupingModAlter;
        read_csman(fd,0x809f0103,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMHOSTGROUPINGMODGROUPNAMEORIGINAL:
        *write_method = write_SystemHostGroupingModGroupNameOriginal;
        memset(string,0,sizeof(string));
		read_csman(fd,0x809f3000,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMHOSTGROUPINGMODGROUPNAME:
        *write_method = write_SystemHostGroupingModGroupName;
        memset(string,0,sizeof(string));
		read_csman(fd,0x809f3001,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMHOSTGROUPINGMODMUTIPLEBOUNDSERVICEFIREWALL:
        *write_method = write_SystemHostGroupingModMutipleBoundServiceFirewall;
        read_csman(fd,0x809f3090,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMHOSTGROUPINGMODMUTIPLEBOUNDSERVICEQOS:
        *write_method = write_SystemHostGroupingModMutipleBoundServiceQoS;
        read_csman(fd,0x809f3091,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMHOSTGROUPINGMODENABLE:
        *write_method = write_SystemHostGroupingModEnable;
        read_csman(fd,0x809f3092,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}

unsigned char *
var_SystemHostGroupingModTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    register int index;
	index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_GROUPING_HOSTGROUPINGMOD);
	w_index_grouping_hostgroupingmod=index;
	if ( index == MATCH_FAILED )
		return NULL;


    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case SYSTEMHOSTGROUPINGMODGROUPMEMBERLISTINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case SYSTEMHOSTGROUPINGMODGROUPMEMBERLISTTYPE:
        *write_method = write_SystemHostGroupingModGroupMemberListType;
        memset(string,0,sizeof(string));
		read_csman(fd,0x809f3010+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMHOSTGROUPINGMODGROUPMEMBERLISTID:
        *write_method = write_SystemHostGroupingModGroupMemberListID;
        memset(string,0,sizeof(string));
		read_csman(fd,0x809f3050+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    default:
      ERROR_MSG("");
    }
    return NULL;
}



int
write_SystemHostGroupingModAlter(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingHostGroupingMod not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingHostGroupingMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;
			
        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f0103,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			write_csman(fd,0x809f0100,&long_ret,sizeof(unsigned int) ,TYPE_U32);
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemHostGroupingModGroupNameOriginal(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemGroupingHostGroupingMod not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemGroupingHostGroupingMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f3000,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemHostGroupingModGroupName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemGroupingHostGroupingMod not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemGroupingHostGroupingMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f3001,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemHostGroupingModMutipleBoundServiceFirewall(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingHostGroupingMod not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingHostGroupingMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f3090,&long_ret,sizeof(unsigned int) ,TYPE_U32);    
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemHostGroupingModMutipleBoundServiceQoS(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingHostGroupingMod not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingHostGroupingMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f3091,&long_ret,sizeof(unsigned int) ,TYPE_U32);    
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemHostGroupingModEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingHostGroupingMod not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingHostGroupingMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f3092,&long_ret,sizeof(unsigned int) ,TYPE_U32);    
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}

int
write_SystemHostGroupingModGroupMemberListType(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemGroupingHostGroupingMod not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemGroupingHostGroupingMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f3010+w_index_grouping_hostgroupingmod-1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemHostGroupingModGroupMemberListID(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemGroupingHostGroupingMod not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemGroupingHostGroupingMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
 			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f3050+w_index_grouping_hostgroupingmod-1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
