#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UISystemGroupingL7AppAdd.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_GROUPING_L7APPADD	32
int w_index_grouping_l7appadd;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UISystemGroupingL7AppAdd_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,5,8,4,1 };
#else
oid UISystemGroupingL7AppAdd_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,5,8,4,1 };
#endif

struct variable7 UISystemGroupingL7AppAdd_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{SYSTEML7APPGROUPINGADDALTER,ASN_INTEGER,RWRITE,var_UISystemGroupingL7AppAdd, 1,  { 1 }},
{SYSTEML7APPGROUPINGADDGROUPNAME,ASN_OCTET_STR,RWRITE,var_UISystemGroupingL7AppAdd, 1,  { 2 }},
{SYSTEML7APPGROUPINGADDGROUPMEMBERLISTINDEX,ASN_INTEGER,RONLY,var_SystemL7AppGroupingAddTable, 3,  { 3 , 1, 1 }},
{SYSTEML7APPGROUPINGADDGROUPMEMBERLISTTYPE,ASN_OCTET_STR,RWRITE,var_SystemL7AppGroupingAddTable, 3,  { 3 , 1, 2 }},
{SYSTEML7APPGROUPINGADDGROUPMEMBERLISTAPPNAME,ASN_OCTET_STR,RWRITE,var_SystemL7AppGroupingAddTable, 3,  { 3 , 1, 3 }},
{SYSTEML7APPGROUPINGADDMUTIPLEBOUNDSERVICEFIREWALL,ASN_INTEGER,RWRITE,var_UISystemGroupingL7AppAdd, 1,  { 4 }},
{SYSTEML7APPGROUPINGADDMUTIPLEBOUNDSERVICEQOS,ASN_INTEGER,RWRITE,var_UISystemGroupingL7AppAdd, 1,  { 5 }},
{SYSTEML7APPGROUPINGADDENABLE,ASN_INTEGER,RWRITE,var_UISystemGroupingL7AppAdd, 1,  { 10 }},
};
/*    (L = length of the oidsuffix) */

void
init_UISystemGroupingL7AppAdd(void)
{
    DEBUGMSGTL(("UISystemGroupingL7AppAdd", "Initializing\n"));
    REGISTER_MIB("UISystemGroupingL7AppAdd", UISystemGroupingL7AppAdd_variables, variable7,
               UISystemGroupingL7AppAdd_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     18
	#else
	#define IFENTRY_NAME_LENGTH     17
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UISystemGroupingL7AppAdd(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;
	
    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case SYSTEML7APPGROUPINGADDALTER:
        *write_method = write_SystemL7AppGroupingAddAlter;
        read_csman(fd,0x809f0109,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEML7APPGROUPINGADDGROUPNAME:
        *write_method = write_SystemL7AppGroupingAddGroupName;
        memset(string,0,sizeof(string));
		read_csman(fd,0x809f8000,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEML7APPGROUPINGADDMUTIPLEBOUNDSERVICEFIREWALL:
        *write_method = write_SystemL7AppGroupingAddMutipleBoundServiceFirewall;
        read_csman(fd,0x809f8090,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEML7APPGROUPINGADDMUTIPLEBOUNDSERVICEQOS:
        *write_method = write_SystemL7AppGroupingAddMutipleBoundServiceQoS;
        read_csman(fd,0x809f8091,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEML7APPGROUPINGADDENABLE:
        *write_method = write_SystemL7AppGroupingAddEnable;
        read_csman(fd,0x809f8092,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_SystemL7AppGroupingAddTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UISystemGroupingL7AppAdd above.
 */
unsigned char *
var_SystemL7AppGroupingAddTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    register int index;
	index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_GROUPING_L7APPADD);
	w_index_grouping_l7appadd=index;
	if ( index == MATCH_FAILED )
		return NULL;


    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case SYSTEML7APPGROUPINGADDGROUPMEMBERLISTINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case SYSTEML7APPGROUPINGADDGROUPMEMBERLISTTYPE:
        *write_method = write_SystemL7AppGroupingAddGroupMemberListType;
        memset(string,0,sizeof(string));
		read_csman(fd,0x809f8010+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEML7APPGROUPINGADDGROUPMEMBERLISTAPPNAME:
        *write_method = write_SystemL7AppGroupingAddGroupMemberListAppName;
        memset(string,0,sizeof(string));
		read_csman(fd,0x809f8050+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    default:
      ERROR_MSG("");
    }
    return NULL;
}



int
write_SystemL7AppGroupingAddAlter(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingL7AppAdd not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingL7AppAdd: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f0109,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			write_csman(fd,0x809f0100,&long_ret,sizeof(unsigned int) ,TYPE_U32);
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemL7AppGroupingAddGroupName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];
	
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemGroupingL7AppAdd not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UISystemGroupingL7AppAdd: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f8000,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemL7AppGroupingAddMutipleBoundServiceFirewall(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingL7AppAdd not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingL7AppAdd: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f8090,&long_ret,sizeof(unsigned int) ,TYPE_U32);
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemL7AppGroupingAddMutipleBoundServiceQoS(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingL7AppAdd not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingL7AppAdd: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f8091,&long_ret,sizeof(unsigned int) ,TYPE_U32);
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemL7AppGroupingAddEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingL7AppAdd not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingL7AppAdd: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f8092,&long_ret,sizeof(unsigned int) ,TYPE_U32);
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}

int
write_SystemL7AppGroupingAddGroupMemberListType(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemGroupingL7AppAdd not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UISystemGroupingL7AppAdd: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f8010+w_index_grouping_l7appadd-1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemL7AppGroupingAddGroupMemberListAppName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemGroupingL7AppAdd not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(char)) {
              fprintf(stderr,"write to UISystemGroupingL7AppAdd: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f8050+w_index_grouping_l7appadd-1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
