#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UISystemGroupingL7AppMod.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_GROUPING_L7APPMOD	32
int w_index_grouping_l7appmod;

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UISystemGroupingL7AppMod_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,5,8,4,3 };
#else
oid UISystemGroupingL7AppMod_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,5,8,4,3 };
#endif

struct variable7 UISystemGroupingL7AppMod_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{SYSTEML7APPGROUPINGMODALTER,ASN_INTEGER,RWRITE,var_UISystemGroupingL7AppMod, 1,  { 1 }},
{SYSTEML7APPGROUPINGMODGROUPNAMEORIGINAL,ASN_OCTET_STR,RWRITE,var_UISystemGroupingL7AppMod, 1,  { 2 }},
{SYSTEML7APPGROUPINGMODGROUPNAME,ASN_OCTET_STR,RWRITE,var_UISystemGroupingL7AppMod, 1,  { 3 }},
{SYSTEML7APPGROUPINGMODGROUPMEMBERLISTINDEX,ASN_INTEGER,RONLY,var_SystemL7AppGroupingModTable, 3,  { 4 , 1, 1 }},
{SYSTEML7APPGROUPINGMODGROUPMEMBERLISTTYPE,ASN_OCTET_STR,RWRITE,var_SystemL7AppGroupingModTable, 3,  { 4 , 1, 2 }},
{SYSTEML7APPGROUPINGMODGROUPMEMBERLISTFORMAT,ASN_OCTET_STR,RWRITE,var_SystemL7AppGroupingModTable, 3,  { 4 , 1, 3 }},	
{SYSTEML7APPGROUPINGMODMUTIPLEBOUNDSERVICEFIREWALL,ASN_INTEGER,RWRITE,var_UISystemGroupingL7AppMod, 1,  { 5 }},
{SYSTEML7APPGROUPINGMODMUTIPLEBOUNDSERVICEQOS,ASN_INTEGER,RWRITE,var_UISystemGroupingL7AppMod, 1,  { 6 }},
{SYSTEML7APPGROUPINGMODENABLE,ASN_INTEGER,RWRITE,var_UISystemGroupingL7AppMod, 1,  { 10 }},

};

void
init_UISystemGroupingL7AppMod(void)
{
    DEBUGMSGTL(("UISystemGroupingL7AppMod", "Initializing\n"));
    REGISTER_MIB("UISystemGroupingL7AppMod", UISystemGroupingL7AppMod_variables, variable7,
               UISystemGroupingL7AppMod_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     18
	#else
	#define IFENTRY_NAME_LENGTH     17
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UISystemGroupingL7AppMod(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case SYSTEML7APPGROUPINGMODALTER:
        *write_method = write_SystemL7AppGroupingModAlter;
        read_csman(fd,0x809f010b,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEML7APPGROUPINGMODGROUPNAMEORIGINAL:
        *write_method = write_SystemL7AppGroupingModGroupNameOriginal;
        memset(string,0,sizeof(string));
		read_csman(fd,0x809f9000,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEML7APPGROUPINGMODGROUPNAME:
        *write_method = write_SystemL7AppGroupingModGroupName;
        memset(string,0,sizeof(string));
		read_csman(fd,0x809f9001,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEML7APPGROUPINGMODMUTIPLEBOUNDSERVICEFIREWALL:
        *write_method = write_SystemL7AppGroupingModMutipleBoundServiceFirewall;
        read_csman(fd,0x809f9090,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEML7APPGROUPINGMODMUTIPLEBOUNDSERVICEQOS:
        *write_method = write_SystemL7AppGroupingModMutipleBoundServiceQoS;
        read_csman(fd,0x809f9091,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEML7APPGROUPINGMODENABLE:
        *write_method = write_SystemL7AppGroupingModEnable;
        read_csman(fd,0x809f9092,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_SystemL7AppGroupingModTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UISystemGroupingL7AppMod above.
 */
unsigned char *
var_SystemL7AppGroupingModTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    register int index;
	index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_GROUPING_L7APPMOD);
	w_index_grouping_l7appmod=index;
	if ( index == MATCH_FAILED )
		return NULL;


    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case SYSTEML7APPGROUPINGMODGROUPMEMBERLISTINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case SYSTEML7APPGROUPINGMODGROUPMEMBERLISTTYPE:
        *write_method = write_SystemL7AppGroupingModGroupMemberListType;
        memset(string,0,sizeof(string));
		read_csman(fd,0x809f9010+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEML7APPGROUPINGMODGROUPMEMBERLISTFORMAT:
        *write_method = write_SystemL7AppGroupingModGroupMemberListFormat;
        memset(string,0,sizeof(string));
		read_csman(fd,0x809f9050+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    default:
      ERROR_MSG("");
    }
    return NULL;
}



int
write_SystemL7AppGroupingModAlter(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingL7AppMod not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingL7AppMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f010b,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			write_csman(fd,0x809f0100,&long_ret,sizeof(unsigned int) ,TYPE_U32);
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemL7AppGroupingModGroupNameOriginal(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];
	
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemGroupingL7AppMod not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemGroupingL7AppMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f9000,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemL7AppGroupingModGroupName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemGroupingL7AppMod not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemGroupingL7AppMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f9001,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemL7AppGroupingModMutipleBoundServiceFirewall(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingL7AppMod not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingL7AppMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
 		{
			int fd=0;
			write_csman(fd,0x809f9090,&long_ret,sizeof(unsigned int) ,TYPE_U32);    
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemL7AppGroupingModMutipleBoundServiceQoS(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingL7AppMod not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingL7AppMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f9091,&long_ret,sizeof(unsigned int) ,TYPE_U32);    
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemL7AppGroupingModEnable(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemGroupingL7AppMod not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemGroupingL7AppMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f9092,&long_ret,sizeof(unsigned int) ,TYPE_U32);    
          	break;
		}
    }
    return SNMP_ERR_NOERROR;
}

int
write_SystemL7AppGroupingModGroupMemberListType(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];
	
    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemGroupingL7AppMod not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemGroupingL7AppMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f9010+w_index_grouping_l7appmod-1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemL7AppGroupingModGroupMemberListFormat(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemGroupingL7AppMod not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemGroupingL7AppMod: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x809f9050+w_index_grouping_l7appmod-1,string,strlen(string)+1,TYPE_STR);
			break;
		}
    }
    return SNMP_ERR_NOERROR;
}
