#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UISystemInformation.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UISystemInformation_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,5,1 };
#else
oid UISystemInformation_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,5,1 };
#endif

struct variable7 UISystemInformation_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{SYSTEMINFOWANTYPE,ASN_OCTET_STR,RONLY,var_UISystemInformation, 1,  { 1 }},
{SYSTEMINFODISPLAYTIME,ASN_OCTET_STR,RONLY,var_UISystemInformation, 1,  { 2 }},

};
/*    (L = length of the oidsuffix) */


/** Initializes the UISystemInformation module */
void
init_UISystemInformation(void)
{
    DEBUGMSGTL(("UISystemInformation", "Initializing\n"));
    REGISTER_MIB("UISystemInformation", UISystemInformation_variables, variable7,UISystemInformation_variables_oid);
}

unsigned char *
var_UISystemInformation(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case SYSTEMINFOWANTYPE:
	{
		unsigned int wan1_index,local_wan_type,local_wan_interface;
		read_csman(fd,0x00010023,&local_wan_interface ,sizeof(unsigned int),TYPE_U32);
		read_csman(fd,0x00010003,&local_wan_type,sizeof(unsigned int),TYPE_U32);
		read_csman(fd,0x00010600,&wan1_index,sizeof(unsigned int),TYPE_U32);
		
		if(wan1_index == 99 || wan1_index == 1 || wan1_index == 2 || wan1_index == 3 || wan1_index == 4 || wan1_index == 5)
		{
			if(local_wan_type == SYSTEMINFO_WANTYPE_DHCP) //dynamic
				strcpy(string,"Dynamic IP Address ");
			else if(local_wan_type == SYSTEMINFO_WANTYPE_FIXED) //static
				strcpy(string,"Static IP Address ");
			else if(local_wan_type == SYSTEMINFO_WANTYPE_PPPOE) //PPPoE
				strcpy(string,"PPP over Ethernet ");
			else if(local_wan_type == SYSTEMINFO_WANTYPE_PPTP) //pptp
				strcpy(string,"PPTP ");
			else if(local_wan_type == SYSTEMINFO_WANTYPE_L2TP) //l2tp
				strcpy(string,"L2TP ");
		}
		else if(wan1_index == 14 || wan1_index == 15){
			strcpy(string,"3G4G ");
		}
		else if(wan1_index == 16){
			strcpy(string,"Dial-up Network ");
    	}

		*var_len = strlen(string)-1;
		return (unsigned char *) string;	
	}
    case SYSTEMINFODISPLAYTIME:
	{
		unsigned char info_time[128];
		memset(info_time,0,sizeof(info_time));
		system("date -R > /tmp/snmp_ui_time");

		FILE *fp;
		char buf[128];
		if((fp = fopen("/tmp/snmp_ui_time","r")) != NULL){
			fgets(info_time, sizeof(info_time), fp);
		}
		fclose(fp);
		system("rm -rf /tmp/snmp_ui_time");
		*var_len = strlen(info_time)-1;
		return (unsigned char *) info_time;		
	}
    default:
      ERROR_MSG("");
    }
    return NULL;
}
