#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */

#include "UISystemSchedulingRuleSetting.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_SCHEDULING_SETTING_TIM	100
#define NUM_SCHEDULING_SETTING_SCH	100

int w_index_sch_time;
int w_index_sch_sch;


#ifdef SNMP_CUSTOMER_SMARTLINK
oid UISystemSchedulingRuleSetting_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,5,5 };
#else
oid UISystemSchedulingRuleSetting_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,5,5 };
#endif

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     16
	#else
	#define IFENTRY_NAME_LENGTH     15
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

struct variable7 UISystemSchedulingRuleSetting_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{SYSTEMSCHEDULERULEIDINDEX,ASN_INTEGER,RONLY,var_UISystemScheduleTable, 3,  { 1 , 1, 1 }},
{SYSTEMSCHEDULERULENAME,ASN_OCTET_STR,RWRITE,var_UISystemScheduleTable, 3,  { 1 , 1, 2 }},
{SYSTEMSCHEDULERULEPOLICY,ASN_INTEGER,RWRITE,var_UISystemScheduleTable, 3,  { 1 , 1, 3 }},
{SYSTEMSCHEDULERULETIMEIDINDEX,ASN_INTEGER,RONLY,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 1 }},
{SYSTEMSCHEDULERULETIMEWEEKDAY1,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 2 }},
{SYSTEMSCHEDULERULETIMESTARTTIME1,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 3 }},
{SYSTEMSCHEDULERULETIMEENDTIME1,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 4 }},
{SYSTEMSCHEDULERULETIMEWEEKDAY2,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 5 }},
{SYSTEMSCHEDULERULETIMESTARTTIME2,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 6 }},
{SYSTEMSCHEDULERULETIMEENDTIME2,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 7 }},
{SYSTEMSCHEDULERULETIMEWEEKDAY3,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 8 }},
{SYSTEMSCHEDULERULETIMESTARTTIME3,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 9 }},
{SYSTEMSCHEDULERULETIMEENDTIME3,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 10 }},
{SYSTEMSCHEDULERULETIMEWEEKDAY4,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 11 }},
{SYSTEMSCHEDULERULETIMESTARTTIME4,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 12 }},
{SYSTEMSCHEDULERULETIMEENDTIME4,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 13 }},
{SYSTEMSCHEDULERULETIMEWEEKDAY5,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 14 }},
{SYSTEMSCHEDULERULETIMESTARTTIME5,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 15 }},
{SYSTEMSCHEDULERULETIMEENDTIME5,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 16 }},
{SYSTEMSCHEDULERULETIMEWEEKDAY6,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 17 }},
{SYSTEMSCHEDULERULETIMESTARTTIME6,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 18 }},
{SYSTEMSCHEDULERULETIMEENDTIME6,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 19 }},
{SYSTEMSCHEDULERULETIMEWEEKDAY7,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 20 }},
{SYSTEMSCHEDULERULETIMESTARTTIME7,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 21 }},
{SYSTEMSCHEDULERULETIMEENDTIME7,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 22 }},
{SYSTEMSCHEDULERULETIMEWEEKDAY8,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 23 }},
{SYSTEMSCHEDULERULETIMESTARTTIME8,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 24 }},
{SYSTEMSCHEDULERULETIMEENDTIME8,ASN_INTEGER,RWRITE,var_UISystemScheduleTimeTable, 3,  { 2 , 1, 25 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UISystemSchedulingRuleSetting module */
void
init_UISystemSchedulingRuleSetting(void)
{
    DEBUGMSGTL(("UISystemSchedulingRuleSetting", "Initializing\n"));
    REGISTER_MIB("UISystemSchedulingRuleSetting", UISystemSchedulingRuleSetting_variables, variable7,
               UISystemSchedulingRuleSetting_variables_oid);
}

unsigned char *
var_UISystemSchedulingRuleSetting(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}

unsigned char *
var_UISystemScheduleTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    register int index;
	index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_SCHEDULING_SETTING_SCH);
	w_index_sch_sch=index;
	if ( index == MATCH_FAILED )
		return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case SYSTEMSCHEDULERULEIDINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case SYSTEMSCHEDULERULENAME:
        *write_method = write_SystemScheduleRuleName;
        memset(string,0,sizeof(string));
		read_csman(fd,0x000b0400+index-1,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMSCHEDULERULEPOLICY:
        *write_method = write_SystemScheduleRulePolicy;
        read_csman(fd,0x000b0C00+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}
/*
 * var_UISystemScheduleTimeTable():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UISystemSchedulingRuleSetting above.
 */
unsigned char *
var_UISystemScheduleTimeTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    register int index;
	index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_SCHEDULING_SETTING_TIM);
	w_index_sch_time=index;
	if ( index == MATCH_FAILED )
		return NULL;


    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case SYSTEMSCHEDULERULETIMEIDINDEX:
        long_ret = index;
		return (unsigned int *) &long_ret;
    case SYSTEMSCHEDULERULETIMEWEEKDAY1:
        *write_method = write_SystemScheduleRuleTimeWeekDay1;
        read_csman(fd,0x000b2001+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMESTARTTIME1:
        *write_method = write_SystemScheduleRuleTimeStartTime1;
        read_csman(fd,0x000b4001+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMEENDTIME1:
        *write_method = write_SystemScheduleRuleTimeEndTime1;
        read_csman(fd,0x000b6001+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMEWEEKDAY2:
        *write_method = write_SystemScheduleRuleTimeWeekDay2;
        read_csman(fd,0x000b2401+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMESTARTTIME2:
        *write_method = write_SystemScheduleRuleTimeStartTime2;
        read_csman(fd,0x000b4401+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMEENDTIME2:
        *write_method = write_SystemScheduleRuleTimeEndTime2;
        read_csman(fd,0x000b6401+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMEWEEKDAY3:
        *write_method = write_SystemScheduleRuleTimeWeekDay3;
        read_csman(fd,0x000b2801+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMESTARTTIME3:
        *write_method = write_SystemScheduleRuleTimeStartTime3;
        read_csman(fd,0x000b4801+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMEENDTIME3:
        *write_method = write_SystemScheduleRuleTimeEndTime3;
        read_csman(fd,0x000b6801+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMEWEEKDAY4:
        *write_method = write_SystemScheduleRuleTimeWeekDay4;
        read_csman(fd,0x000b2c01+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMESTARTTIME4:
        *write_method = write_SystemScheduleRuleTimeStartTime4;
        read_csman(fd,0x000b4c01+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMEENDTIME4:
        *write_method = write_SystemScheduleRuleTimeEndTime4;
        read_csman(fd,0x000b6c01+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMEWEEKDAY5:
        *write_method = write_SystemScheduleRuleTimeWeekDay5;
        read_csman(fd,0x000b3001+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMESTARTTIME5:
        *write_method = write_SystemScheduleRuleTimeStartTime5;
        read_csman(fd,0x000b5001+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMEENDTIME5:
        *write_method = write_SystemScheduleRuleTimeEndTime5;
        read_csman(fd,0x000b7001+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMEWEEKDAY6:
        *write_method = write_SystemScheduleRuleTimeWeekDay6;
        read_csman(fd,0x000b3401+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMESTARTTIME6:
        *write_method = write_SystemScheduleRuleTimeStartTime6;
        read_csman(fd,0x000b5401+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMEENDTIME6:
        *write_method = write_SystemScheduleRuleTimeEndTime6;
        read_csman(fd,0x000b7401+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMEWEEKDAY7:
        *write_method = write_SystemScheduleRuleTimeWeekDay7;
        read_csman(fd,0x000b3801+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMESTARTTIME7:
        *write_method = write_SystemScheduleRuleTimeStartTime7;
        read_csman(fd,0x000b5801+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMEENDTIME7:
        *write_method = write_SystemScheduleRuleTimeEndTime7;
        read_csman(fd,0x000b7801+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMEWEEKDAY8:
        *write_method = write_SystemScheduleRuleTimeWeekDay8;
        read_csman(fd,0x000b3c01+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMESTARTTIME8:
        *write_method = write_SystemScheduleRuleTimeStartTime8;
        read_csman(fd,0x000b5c01+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case SYSTEMSCHEDULERULETIMEENDTIME8:
        *write_method = write_SystemScheduleRuleTimeEndTime8;
        read_csman(fd,0x000b7c01+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}



int
write_SystemScheduleRuleName(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemScheduling not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			//for STR/MAC data type
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
		{
			int fd=0;
			write_csman(fd,0x000b0401+w_index_sch_sch-1,string,strlen(string)+1,TYPE_STR);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRulePolicy(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			long_ret = *(unsigned int *) var_val;
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
		{
			int fd=0;
			write_csman(fd,0x000b0c01+w_index_sch_sch-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
		}

    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeWeekDay1(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b2001+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeStartTime1(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b4001+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeEndTime1(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b6001+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}

int
write_SystemScheduleRuleTimeWeekDay2(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b2401+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeStartTime2(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b4401+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeEndTime2(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b6401+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeWeekDay3(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b2801+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeStartTime3(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b4801+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeEndTime3(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b6801+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeWeekDay4(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b2c01+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeStartTime4(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b4c01+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeEndTime4(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b6c01+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeWeekDay5(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b3001+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeStartTime5(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b5001+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeEndTime5(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b7001+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeWeekDay6(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b3401+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeStartTime6(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b5401+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeEndTime6(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b7401+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeWeekDay7(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b3801+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeStartTime7(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b5801+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeEndTime7(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b7801+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeWeekDay8(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b3c01+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeStartTime8(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b5c01+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}
int
write_SystemScheduleRuleTimeEndTime8(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned int long_ret;

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_INTEGER) {
              fprintf(stderr, "write to UISystemScheduling not ASN_INTEGER\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(long_ret)) {
              fprintf(stderr,"write to UISystemScheduling: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
          long_ret = *(unsigned int *) var_val;
          break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
        {
			int fd=0;
			write_csman(fd,0x000b7c01+w_index_sch_time-1,&long_ret,sizeof(unsigned int) ,TYPE_U32);
			break;
        }
    }
    return SNMP_ERR_NOERROR;
}

