#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UISystemToolsChangePassword.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UISystemToolsChangePassword_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,5,3,1 };
#else
oid UISystemToolsChangePassword_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,5,3,1 };
#endif

struct variable7 UISystemToolsChangePassword_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{SYSTEMTOOLSPASSWORD,ASN_OCTET_STR,RWRITE,var_UISystemToolsChangePassword, 1,  { 1 }},
{SYSTEMTOOLSHOSTNAME,ASN_OCTET_STR,RWRITE,var_UISystemToolsChangePassword, 1,  { 2 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UISystemToolsChangePassword module */
void
init_UISystemToolsChangePassword(void)
{
    DEBUGMSGTL(("UISystemToolsChangePassword", "Initializing\n"));
    REGISTER_MIB("UISystemToolsChangePassword", UISystemToolsChangePassword_variables, variable7,
               UISystemToolsChangePassword_variables_oid);
}

unsigned char *
var_UISystemToolsChangePassword(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case SYSTEMTOOLSPASSWORD:
        *write_method = write_SystemToolsPassword;
		memset(string,0,sizeof(string));
		read_csman(fd,0x00010002,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case SYSTEMTOOLSHOSTNAME:
        *write_method = write_SystemToolsHostname;
		memset(string,0,sizeof(string));
		read_csman(fd,0x00010001,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    default:
      ERROR_MSG("");
    }
    return NULL;
}

int
write_SystemToolsPassword(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemToolsChangePassword not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemToolsChangePassword: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			//for STR/MAC data type
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
         {
             int fd=0;
             write_csman(fd,0x00010002,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}


int
write_SystemToolsHostname(int      action,
            u_char   *var_val,
            u_char   var_val_type,
            size_t   var_val_len,
            u_char   *statP,
            oid      *name,
            size_t   name_len)
{
    static unsigned char string[SPRINT_MAX_LEN];

    switch ( action ) {
        case RESERVE1:
          if (var_val_type != ASN_OCTET_STR) {
              fprintf(stderr, "write to UISystemToolsChangePassword not ASN_OCTET_STR\n");
              return SNMP_ERR_WRONGTYPE;
          }
          if (var_val_len > sizeof(string)) {
              fprintf(stderr,"write to UISystemToolsChangePassword: bad length\n");
              return SNMP_ERR_WRONGLENGTH;
          }
          break;

        case RESERVE2:
			//for STR/MAC data type
			memset(string,0,sizeof(string));
			strncpy(string , (char *) var_val, var_val_len);
			break;

        case FREE:
             /* Release any resources that have been allocated */
          break;

        case ACTION:
             /*
              * The variable has been stored in 'value' for you to use,
              * and you have just been asked to do something with it.
              * Note that anything done here must be reversable in the UNDO case
              */
          break;

        case UNDO:
             /* Back out any changes made in the ACTION case */
          break;

        case COMMIT:
             /*
              * Things are working well, so it's now safe to make the change
              * permanently.  Make sure that anything done here can't fail!
              */
         {
             int fd=0;
             write_csman(fd,0x00010001,string,strlen(string)+1,TYPE_STR);
             break;
		 }
    }
    return SNMP_ERR_NOERROR;
}

