#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIWANInterfaceIPv6NetworkStatus.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIWANInterfaceIPv6NetworkStatus_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,1,1,2 };
#else
oid UIWANInterfaceIPv6NetworkStatus_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,1,1,2 };
#endif

struct variable7 UIWANInterfaceIPv6NetworkStatus_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{STATUSIPV6WANID1,ASN_OCTET_STR,RONLY,var_UIWANInterfaceIPv6NetworkStatus, 1,  { 1 }},
{STATUSIPV6INTERFACE1,ASN_OCTET_STR,RONLY,var_UIWANInterfaceIPv6NetworkStatus, 1,  { 2 }},
{STATUSIPV6WANTYPE1,ASN_INTEGER,RONLY,var_UIWANInterfaceIPv6NetworkStatus, 1,  { 3 }},
{STATUSIPV6LINKLOCALIPADDRESS1,ASN_OCTET_STR,RONLY,var_UIWANInterfaceIPv6NetworkStatus, 1,  { 4 }},
{STATUSIPV6GLOBALIPADDRESS1,ASN_OCTET_STR,RONLY,var_UIWANInterfaceIPv6NetworkStatus, 1,  { 5 }},
{STATUSIPV6CONNECTIONSTATUS1,ASN_INTEGER,RONLY,var_UIWANInterfaceIPv6NetworkStatus, 1,  { 6 }},

};
/*    (L = length of the oidsuffix) */


/** Initializes the UIWANInterfaceIPv6NetworkStatus module */
void
init_UIWANInterfaceIPv6NetworkStatus(void)
{
    DEBUGMSGTL(("UIWANInterfaceIPv6NetworkStatus", "Initializing\n"));
    REGISTER_MIB("UIWANInterfaceIPv6NetworkStatus", UIWANInterfaceIPv6NetworkStatus_variables, variable7,
		UIWANInterfaceIPv6NetworkStatus_variables_oid);
}

/*
 * var_UIWANInterfaceIPv6NetworkStatus():
 *   This function is called every time the agent gets a request for
 *   a scalar variable that might be found within your mib section
 *   registered above.  It is up to you to do the right thing and
 *   return the correct value.
 *     You should also correct the value of "var_len" if necessary.
 *
 *   Please see the documentation for more information about writing
 *   module extensions, and check out the examples in the examples
 *   and mibII directories.
 */
unsigned char *
var_UIWANInterfaceIPv6NetworkStatus(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    switch(vp->magic) {
    case STATUSIPV6WANID1:
		memset(string,0,sizeof(string));
		strcpy(string,"WAN-1 ");
        *var_len = strlen(string)-1;
		return (unsigned char *) string;
    case STATUSIPV6INTERFACE1:
	{
		int ipv6_enable=0;
		read_csman(fd,0x00900000,&ipv6_enable,sizeof(unsigned int),TYPE_U32);
        memset(string,0,sizeof(string));
		if(ipv6_enable)	strcpy(string,"Ethernet ");
		else 			strcpy(string," ");
        *var_len = strlen(string)-1;
		return (unsigned char *) string;
    }
    case STATUSIPV6WANTYPE1:
	{
		unsigned int ipv6_wantype=0;
		read_csman(fd,0x00900001,&ipv6_wantype,sizeof(unsigned int),TYPE_U32);

		if(ipv6_wantype == IPV6_CONNECTTYPE_STATIC)		long_ret=0;
		else if(ipv6_wantype == IPV6_CONNECTTYPE_DHCP)	long_ret=1;
		else if(ipv6_wantype == IPV6_CONNECTTYPE_PPPoE)	long_ret=2;
		else if(ipv6_wantype == IPV6_CONNECTTYPE_6to4)	long_ret=3;
		else if(ipv6_wantype == IPV6_CONNECTTYPE_6in4)	long_ret=4;
		else if(ipv6_wantype == IPV6_CONNECTTYPE_PPPoA)	long_ret=5;

        return (unsigned char *) &long_ret;
    }
    case STATUSIPV6LINKLOCALIPADDRESS1:
        memset(string,0,sizeof(string));
		read_csman(fd,0x80900008,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case STATUSIPV6GLOBALIPADDRESS1:
        memset(string,0,sizeof(string));
		read_csman(fd,0x80900009,string,sizeof(string),TYPE_STR);
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case STATUSIPV6CONNECTIONSTATUS1:
	{
		unsigned int ipv6_wantype=0;
		read_csman(fd,0x00900001,&ipv6_wantype,sizeof(unsigned int),TYPE_U32);
		
		if(ipv6_wantype == IPV6_CONNECTTYPE_STATIC){
			long_ret=2;
		}
		else if(ipv6_wantype == IPV6_CONNECTTYPE_DHCP){
			read_csman(fd,0x80900701,&long_ret,sizeof(unsigned int),TYPE_U32);
    	}
		else if(ipv6_wantype == IPV6_CONNECTTYPE_PPPoE){
			read_csman(fd,0x80900030,&long_ret,sizeof(unsigned int),TYPE_U32);
		}
		else if(ipv6_wantype == IPV6_CONNECTTYPE_6to4){
			read_csman(fd,0x80900801,&long_ret,sizeof(unsigned int),TYPE_U32);
		}
		else if(ipv6_wantype == IPV6_CONNECTTYPE_6in4){
			read_csman(fd,0x00900001,&long_ret,sizeof(unsigned int),TYPE_U32);
		}
		else if(ipv6_wantype == IPV6_CONNECTTYPE_PPPoA){
			read_csman(fd,0x80900030,&long_ret,sizeof(unsigned int),TYPE_U32);
		}
		return (unsigned char *) &long_ret;
	}
    default:
      ERROR_MSG("");
    }
    return NULL;
}
