#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIWiFi24TrafficStatisticsStatus.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_WiFi24VAPTRAFFICSTATISTIC 8

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIWiFi24TrafficStatisticsStatus_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,1,2,2 };
#else
oid UIWiFi24TrafficStatisticsStatus_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,1,2,2 };
#endif

struct variable7 UIWiFi24TrafficStatisticsStatus_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{WIFI24TRAFFICSTATISTICIDINDEX,ASN_INTEGER,RONLY,var_UIStatusWiFi24TrafficStatisticTable, 3,  { 1 , 1, 1 }},
{WIFI24TRAFFICSTATISTICOPBAND,ASN_OCTET_STR,RONLY,var_UIStatusWiFi24TrafficStatisticTable, 3,  { 1 , 1, 2 }},
{WIFI24TRAFFICSTATISTICVAPID,ASN_OCTET_STR,RONLY,var_UIStatusWiFi24TrafficStatisticTable, 3,  { 1 , 1, 3 }},
{WIFI24TRAFFICSTATISTICRXPACKETS,ASN_INTEGER,RONLY,var_UIStatusWiFi24TrafficStatisticTable, 3,  { 1 , 1, 4 }},
{WIFI24TRAFFICSTATISTICTXPACKETS,ASN_INTEGER,RONLY,var_UIStatusWiFi24TrafficStatisticTable, 3,  { 1 , 1, 5 }},
};

/** Initializes the UIWiFi24TrafficStatisticsStatus module */
void
init_UIWiFi24TrafficStatisticsStatus(void)
{
    DEBUGMSGTL(("UIWiFi24TrafficStatisticsStatus", "Initializing\n"));
    REGISTER_MIB("UIWiFi24TrafficStatisticsStatus", UIWiFi24TrafficStatisticsStatus_variables, variable7,
               UIWiFi24TrafficStatisticsStatus_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UIWiFi24TrafficStatisticsStatus(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}


unsigned char *
var_UIStatusWiFi24TrafficStatisticTable(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;

	register int index;
	index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_WiFi24VAPTRAFFICSTATISTIC);
		
	if ( index == MATCH_FAILED )
		return NULL;

    switch(vp->magic) {
    case WIFI24TRAFFICSTATISTICIDINDEX:
        long_ret = index;
        return (unsigned int *) &long_ret;
    case WIFI24TRAFFICSTATISTICOPBAND:
		memset(string,0,sizeof(string));
		strcpy(string,"2.4G ");
        *var_len = strlen(string)-1;
		return (unsigned char *) string;
    case WIFI24TRAFFICSTATISTICVAPID:
		memset(string,0,sizeof(string));
		if(index == 1)	strcpy(string,"VAP-1 ");
		else if(index == 2)	strcpy(string,"VAP-2 ");
		else if(index == 3)	strcpy(string,"VAP-3 ");
		else if(index == 4)	strcpy(string,"VAP-4 ");
		else if(index == 5)	strcpy(string,"VAP-5 ");
		else if(index == 6)	strcpy(string,"VAP-6 ");
		else if(index == 7)	strcpy(string,"VAP-7 ");
		else if(index == 8)	strcpy(string,"VAP-8 ");
        *var_len = strlen(string)-1;
		return (unsigned char *) string;
    case WIFI24TRAFFICSTATISTICRXPACKETS:
		system("stainfo");
        read_csman(fd,0x80198020+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case WIFI24TRAFFICSTATISTICTXPACKETS:
		system("stainfo");
        read_csman(fd,0x80198030+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    default:
      ERROR_MSG("");
    }
    return NULL;
}
