#ifdef IN_UCD_SNMP_SOURCE
#include <config.h>
#include "mibincl.h"
#include "util_funcs.h"
#else /* !IN_UCD_SNMP_SOURCE */
#include <ucd-snmp-config.h>
#include <ucd-snmp-includes.h>
#include <ucd-snmp-agent-includes.h>
#endif /* !IN_UCD_SNMP_SOURCE */
#include "UIWiFi50VirtualAPListStatus.h"
#include "csman.h"
#include <stdio.h>
#include <string.h>

#define NUM_WiFi50VAP 8

#ifdef SNMP_CUSTOMER_SMARTLINK
oid UIWiFi50VirtualAPListStatus_variables_oid[] = { 1,3,6,1,4,1,36293,1,2,2,4,1,2,3 };
#else
oid UIWiFi50VirtualAPListStatus_variables_oid[] = { 1,3,6,1,4,1,12823,4,4,9,1,2,3 };
#endif

struct variable7 UIWiFi50VirtualAPListStatus_variables[] = {
/*  magic number        , variable type , ro/rw , callback fn  , L, oidsuffix */

{WIFI50VIRTUALAPSTATUSIDINDEX,ASN_INTEGER,RONLY,var_UIStatusWiFi50Table, 3,  { 1 , 1, 1 }},
{WIFI50VIRTUALAPSTATUSOPBAND,ASN_OCTET_STR,RONLY,var_UIStatusWiFi50Table, 3,  { 1 , 1, 2 }},
{WIFI50VIRTUALAPSTATUSVAPID,ASN_OCTET_STR,RONLY,var_UIStatusWiFi50Table, 3,  { 1 , 1, 3 }},
{WIFI50VIRTUALAPSTATUSWIFIENABLE,ASN_INTEGER,RONLY,var_UIStatusWiFi50Table, 3,  { 1 , 1, 4 }},
{WIFI50VIRTUALAPSTATUSOPMODE,ASN_INTEGER,RONLY,var_UIStatusWiFi50Table, 3,  { 1 , 1, 5 }},
{WIFI50VIRTUALAPSTATUSSSID,ASN_OCTET_STR,RONLY,var_UIStatusWiFi50Table, 3,  { 1 , 1, 6 }},
{WIFI50VIRTUALAPSTATUSCHANNEL,ASN_INTEGER,RONLY,var_UIStatusWiFi50Table, 3,  { 1 , 1, 7 }},
{WIFI50VIRTUALAPSTATUSWIFISYSTEM,ASN_INTEGER,RONLY,var_UIStatusWiFi50Table, 3,  { 1 , 1, 8 }},
{WIFI50VIRTUALAPSTATUSAUTH,ASN_INTEGER,RONLY,var_UIStatusWiFi50Table, 3,  { 1 , 1, 9 }},
{WIFI50VIRTUALAPSTATUSSECURITY,ASN_INTEGER,RONLY,var_UIStatusWiFi50Table, 3,  { 1 , 1, 10 }},
{WIFI50VIRTUALAPSTATUSMACADDRESS,ASN_OCTET_STR,RONLY,var_UIStatusWiFi50Table, 3,  { 1 , 1, 11 }},
};
/*    (L = length of the oidsuffix) */


/** Initializes the UIWiFi50VirtualAPListStatus module */
void
init_UIWiFi50VirtualAPListStatus(void)
{
    DEBUGMSGTL(("UIWiFi50VirtualAPListStatus", "Initializing\n"));
    REGISTER_MIB("UIWiFi50VirtualAPListStatus", UIWiFi50VirtualAPListStatus_variables, variable7,
		UIWiFi50VirtualAPListStatus_variables_oid);
}

static int
header_Entry(struct variable *vp,
               oid *name,
               size_t *length,
               int exact,
               size_t *var_len,
               WriteMethod **write_method, int count)
{
	#ifdef SNMP_CUSTOMER_SMARTLINK
	#define IFENTRY_NAME_LENGTH     17
	#else
	#define IFENTRY_NAME_LENGTH     16
	#endif
	/* in this case, oid len is 16*/
    	oid newname[MAX_OID_LEN];
    	register int        interface;
    	int result;

    	memcpy( (char *)newname,(char *)vp->name, (int)vp->namelen * sizeof(oid));

    	for(interface = 1; interface <= count; interface++){
        	newname[IFENTRY_NAME_LENGTH] = (oid)interface;
        	result = snmp_oid_compare(name, *length, newname, vp->namelen + 1);
        	if ((exact && (result == 0)) || (!exact && (result < 0)))
            	break;
    	}
    	if (interface > count) {
        	DEBUGMSGTL(("mibII/interfaces", "... index out of range\n"));
        	return MATCH_FAILED;
    	}

    	memcpy( (char *)name,(char *)newname, ((int)vp->namelen + 1) * sizeof(oid));
    	*length = vp->namelen + 1;
    	*write_method = 0;
    	*var_len = sizeof(long); 
    return interface; /* interface is "index" */
}

unsigned char *
var_UIWiFi50VirtualAPListStatus(struct variable *vp, 
                oid     *name, 
                size_t  *length, 
                int     exact, 
                size_t  *var_len, 
                WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;

    if (header_generic(vp,name,length,exact,var_len,write_method)
                                  == MATCH_FAILED )
    return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    default:
      ERROR_MSG("");
    }
    return NULL;
}


/*
 * var_UIStatusWiFi50Table():
 *   Handle this table separately from the scalar value case.
 *   The workings of this are basically the same as for var_UIWiFi50VirtualAPListStatus above.
 */
unsigned char *
var_UIStatusWiFi50Table(struct variable *vp,
    	    oid     *name,
    	    size_t  *length,
    	    int     exact,
    	    size_t  *var_len,
    	    WriteMethod **write_method)
{
    /* variables we may use later */
    static long long_ret;
    static u_long ulong_ret;
    static unsigned char string[SPRINT_MAX_LEN];
    static oid objid[MAX_OID_LEN];
    static struct counter64 c64;
	int fd=0;
	
	register int index;
	index = header_Entry(vp, name, length, exact, var_len, write_method,NUM_WiFi50VAP);
		
	if ( index == MATCH_FAILED )
		return NULL;

    /* 
   * this is where we do the value assignments for the mib results.
   */
    switch(vp->magic) {
    case WIFI50VIRTUALAPSTATUSIDINDEX:
        long_ret = index;
        return (unsigned int *) &long_ret;
    case WIFI50VIRTUALAPSTATUSOPBAND:
		memset(string,0,sizeof(string));
		strcpy(string,"5G ");
        *var_len = strlen(string)-1;
		return (unsigned char *) string;
    case WIFI50VIRTUALAPSTATUSVAPID:
		memset(string,0,sizeof(string));
		if(index == 1)	strcpy(string,"VAP-1 ");
		else if(index == 2)	strcpy(string,"VAP-2 ");
		else if(index == 3)	strcpy(string,"VAP-3 ");
		else if(index == 4)	strcpy(string,"VAP-4 ");
		else if(index == 5)	strcpy(string,"VAP-5 ");
		else if(index == 6)	strcpy(string,"VAP-6 ");
		else if(index == 7)	strcpy(string,"VAP-7 ");
		else if(index == 8)	strcpy(string,"VAP-8 ");
        *var_len = strlen(string)-1;
		return (unsigned char *) string;
    case WIFI50VIRTUALAPSTATUSWIFIENABLE:
        read_csman(fd,0x00440100+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case WIFI50VIRTUALAPSTATUSOPMODE:
        read_csman(fd,0x00440008,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case WIFI50VIRTUALAPSTATUSSSID:
        memset(string,0,sizeof(string));
		read_csman(fd,0x00440400+index-1,string,sizeof(string),TYPE_STR);
		if(!strcmp(string,"\0"))	strcpy(string,"default ");
		*var_len = strlen(string)-1;
		return (unsigned char *) string;
    case WIFI50VIRTUALAPSTATUSCHANNEL:
		read_csman(fd,0x00440200,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case WIFI50VIRTUALAPSTATUSWIFISYSTEM:
		read_csman(fd,0x00440c00+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case WIFI50VIRTUALAPSTATUSAUTH:
		read_csman(fd,0x00441300+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case WIFI50VIRTUALAPSTATUSSECURITY:
		read_csman(fd,0x00441400+index-1,&long_ret,sizeof(unsigned int),TYPE_U32);
		return (unsigned char *) &long_ret;
    case WIFI50VIRTUALAPSTATUSMACADDRESS:
		system("apinfo50");
        memset(string,0,sizeof(string));
		read_csman(fd,0x80448000+index-1,string,sizeof(string),TYPE_MAC);
		*var_len = 6;
		return (unsigned char *) string;
    default:
      ERROR_MSG("");
    }
    return NULL;
}
